; EVENTURELAND: LEVEL 1 - BYTE ME
; http://www.retrosoftware.co.uk/eventureland

; Written by Jon Cauldwell in 2011

; ByteBack Bites.
; Project started 07.09.2011 @ 17:44.
; Project completed 30.10.2011 @ 16:15.

; Byte Me was written using the built-in assembler in ZX SPIN, a very popular Spectrum
; emulator.

; Based on the events at Byte Back in 2009, the game revolves around the antics of the
; kitchen staff and their attempts to feed an impatient crowd of retro gamers. The idea
; is to throw sausages at the hot dog rolls and burgers at burger baps which then drop
; on to a conveyor belt at the bottom of the screen. Send the food to the queuing
; customers at the sides of the screen using the conveyor - but try not to keep them
; waiting too long! The direction of the belt is controlled via the buttons in the top
; corners of the screen. You can only carry six burgers or sausages at a time, but fresh
; filling supplies are found in crates which parachute in regularly from the top of the
; screen. Use this ammunition wisely as it only comes in packs of six. Drinks cans yield
; bonuses. Fail to serve a customer in time and a life is lost. Contact with baps or any
; other nasties is fatal, as one might expect.

; It's effectively classic early 1980s style gameplay which starts off frantic and gets
; even more so the further you get, adding an extra sprite to the proceedings every two
; levels. Craig Stevenson's excellent graphics bring the whole thing to life, and there
; are two more incredible tunes from Yerzmyey. It's worth playing the game in an emulator
; which supports the 128K/+2/+3 machines, as that way you'll hear the music. Byte Me
; supports joysticks, the popular Kempston and Sinclair formats.  It will support cursor
; joysticks too, if you select the redefine keys option from the menu and manipulate the
; joystick in response to the prompts.

; Have fun... it's one of the more frantic Spectrum games you'll play. I hope you have as
; much fun playing this game as I did developing it. 
;
;    (c) Copyright 2011 Jonathan Cauldwell
;    http://members.fortunecity.com/jonathan6/egghead

;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.



; Global definitions.

; General.

MAXLEV equ 9               ; last level.
HSENT  equ 8               ; high score entries in table.
HSROW  equ 7               ; first name x position.

; Sprites.

NUMSPR equ 11              ; number of sprites.
SPRSIZ equ 11              ; size of each entry.
LEN    equ NUMSPR * SPRSIZ ; size of entire table.
MAXSPW equ 8               ; most we can spawn on hardest levels.

PLRIMG equ 0               ; player image.
PLLIMG equ 1               ; player image.
PRJIMG equ PLLIMG + 1      ; sausage projectile image.
PR2IMG equ PRJIMG + 1      ; burger projectile image.
SPWIMG equ PR2IMG + 1      ; spawning image.
BBLIMG equ SPWIMG + 1      ; burger bap left image.
SUPIMG equ 6               ; supplies image.
EXPIMG equ SUPIMG + 1      ; explosion image.
TMLIMG equ EXPIMG + 1      ; tomato, left.
DOGIMG equ TMLIMG + 1      ; complete hot dog.
FBGIMG equ DOGIMG + 1      ; falling burger image.
SP2IMG equ FBGIMG + 1      ; burger supplies.
CANIMG equ SP2IMG + 1      ; can of soft drink.
BBRIMG equ CANIMG + 1      ; burger bap right.
FRLIMG equ BBRIMG + 1      ; finger roll, left.
FRRIMG equ FRLIMG + 1      ; finger roll, right.
TMRIMG equ FRRIMG + 1      ; tomato, right.
SEDIMG equ TMRIMG + 1      ; sedative image.
BRLIMG equ SEDIMG + 1      ; bread left image.
BRRIMG equ BRLIMG + 1      ; bread right image.

; Types.

PLYTYP equ 0               ; player.
AMMTYP equ 1               ; ammunition.
SUPTYP equ 2               ; supplies to collect.
BNCTYP equ 3               ; bouncing enemy.
SPWTYP equ 4               ; spawning enemy.
FBGTYP equ 5               ; falling burger.
EXPTYP equ 6               ; explosion.
BOWTYP equ 7               ; bouncing then walking.
FRLTYP equ 8               ; finger roll type.
SEDTYP equ 9               ; customer sedative.
HUNTYP equ 10              ; hunter-killer toast slices.


; Slots.

SLOTS  equ 12              ; total customer slots in queues.
NQUEUE equ 6               ; number in each queue.


; Font/character display.

MSG    equ 97
ATPOS  equ 98
ENDCHR equ 99


; Sounds.

FIRSND equ 1               ; player fire sound.
SPWSND equ 4               ; spawning sound.
BUTSND equ 3               ; button pressed sound.
EXPSND equ 7               ; explosion sound.
GETSND equ 5               ; pick up object noise.
WALSND equ 2               ; bounce off wall sound.
FEDSND equ 8               ; customer fed noise.
TRASND equ 3               ; transform to burger or dog.


; High score entry stuff.

MINX   equ 124             ; minimum x coordinate.
MINY   equ 76              ; minimum y.
MAXX   equ 188             ; maximum x.
MAXY   equ 156             ; maximum y position.

       org 24000

       jp start

       org 27400

; Set up the interrupts.

start  di                  ; interrupts off.
       ld a,232            ; vector table address.
       ld i,a              ; point to vector table.
       im 2                ; mode 2 interrupts.
       ei                  ; start interrupts.
       call imusic         ; initialise music.

; Debounce keys and wait for keypress to start.

deb0   call 654            ; read keyboard.
       inc e               ; anything pressed?
       jr nz,deb0          ; yes, debounce it.
deb1   call 654            ; read keyboard.
       inc e               ; anything pressed?
       jr z,deb1           ; no, wait for keypress.
deb2   call 654            ; read keyboard.
       inc e               ; anything pressed?
       jr nz,deb2          ; yes, debounce it.

; Now set the seeds for the random number generator.

       ld a,r              ; get value of r register.
       ld (seed2),a        ; set second seed.
       ld a,(23672)        ; low byte of clock.
       ld (seed),a         ; set first seed.

; Set up screen display.

       xor a               ; zero for black.
       call 8859           ; set permanent border colour.
       ld hl,23693         ; attributes system variable.
       ld (hl),71          ; set permanent attributes.
       call cls            ; clear the screen.

       call setpal         ; set the ULAplus palette.

; Main menu - establish which screen to show next.

menu   ld hl,menuno        ; menu number to display.
       inc (hl)            ; move it along.
       ld a,(hl)           ; get menu number.
       and 3               ; only 4 to display.
       rlca                ; multiply byte by 2.
       ld e,a              ; displacement in de.
       ld d,0              ; no high byte for displacement.
       ld hl,menptr        ; pointer to menu routines.
       add hl,de           ; point to address of routine.
       ld e,(hl)           ; low byte of address in e.
       inc hl              ; next byte.
       ld d,(hl)           ; high byte of address in e.
       ex de,hl            ; put address in hl.
       jp (hl)             ; jump to routine.

menptr defw menua,dishs    ; menu routine sequence.
       defw menua,menuc
menuno defb 255

menua  ld hl,23693         ; attributes system variable.
       ld (hl),70          ; set permanent attributes.
       call cls            ; clear attributes too.
       call dlogo          ; display logo.
       ld hl,mtxt0         ; menu text.
       call dfont          ; show the text.
       ld hl,23200         ; last 3 attribute rows.
       ld b,96             ; attribute cells to write.
menua0 ld (hl),7           ; write white on black ink.
       inc l               ; next cell.
       djnz menua0         ; repeat.
       ld a,244            ; delay of approx 10 seconds.
menu1  ld (mentim),a       ; set menu timer.
menu2  halt                ; pause for 2 50ths of a second.
       halt
       push bc             ; store count.
       call 654            ; keyboard routine.
       pop bc              ; restore count.
       ld a,e              ; test key returned.
       cp 35               ; is it fire button?
       jp z,gamek          ; yes, start the game.
       cp 36               ; keyboard selected?
       jp z,gamek          ; yes, select starting level.
       cp 28               ; Kempston joystick option?
       jp z,gamejk         ; yes.
       cp 20               ; is it Sinclair?
       jp z,gamejs         ; yes.
       cp 12               ; is it redefine keys?
       jp z,redkey         ; redefine them then.
       ld hl,mentim        ; menu timer.
       dec (hl)            ; one less iteration to go.
       jr nz,menu2
       jp menu
mentim defb 0              ; menu timer.

; Credits screen.

menuc  ld hl,23693         ; attributes system variable.
       ld (hl),71          ; set permanent attributes.
       call cls            ; clear the screen.
       ld hl,crtxt         ; credits text.
       call dfont          ; show the text.
       ld a,244            ; delay of approx 10 seconds.
       jr menu1            ; rest of menu.

mtxt0  defb ATPOS,9,8
       defb 33,27,0,11,5,25,2,15,1,18,4
       defb ATPOS,11,8
       defb 34,27,0,11,5,13,16,19,20,15,14
       defb ATPOS,13,8
       defb 35,27,0,19,9,14,3,12,1,9,18
       defb ATPOS,15,8
       defb 36,27,0,18,5,4,5,6,9,14,5,0,11,5,25,19
       defb ATPOS,21,6
       defb 5,22,5,14,20,21,18,5,12,1,14,4,28,0,16,1,18,20,0,33
       defb ATPOS,23,4
       defb 18,5,12,5,1,19,5,4,0,1,20,0,18,5,16,12,1,25,0,34,32,33,33
;       defb ATPOS,23,9
;       defb 3,15,16,25,18,9,7,8,20,0,34,32,33,33
       defb ENDCHR

crtxt  defb ATPOS,3,14
       defb 7,1,13,5
       defb ATPOS,5,7
       defb 10,15,14,1,20,8,1,14,0,3,1,21,12,4,23,5,12,12
       defb ATPOS,10,12
       defb 7,18,1,16,8,9,3,19
       defb ATPOS,12,8
       defb 3,18,1,9,7,0,19,20,5,22,5,14,19,15,14
       defb ATPOS,17,13
       defb 13,21,19,9,3
       defb ATPOS,19,12
       defb 25,5,18,26,13,25,5,25
       defb ENDCHR


; High score table text.

hstxt0 defb ATPOS,2,9
       defb 3,15,15,11,9,14,7,0,15,14,0,7,1,19

; High score table text.

hstab  defb ATPOS,HSROW,8
       defb 10,15,14,1,20,8,1,14,0
topsc  defb 32,32,32,33,32,32,32
       defb ATPOS,HSROW+2,8
       defb 3,18,1,9,7,0,0,0,0
       defb 32,32,32,32,37,32,32
       defb ATPOS,HSROW+4,8
       defb 25,5,18,26,13,25,5,25,0
       defb 32,32,32,32,36,32,32
       defb ATPOS,HSROW+6,8
       defb 4,1,22,5,0,0,0,0,0
       defb 32,32,32,32,35,32,32
       defb ATPOS,HSROW+8,8
       defb 13,1,20,0,0,0,0,0,0
       defb 32,32,32,32,34,32,32
       defb ATPOS,HSROW+10,8
       defb 13,15,10,15,0,0,0,0,0
       defb 32,32,32,32,33,32,32
       defb ATPOS,HSROW+12,8
       defb 13,9,12,11,1,20,0,0,0
       defb 32,32,32,32,32,37,32
hslast defb ATPOS,HSROW+14,8
       defb 9,1,14,0,0,0,0,0,0
hsend  defb 32,32,32,32,32,33,35
       defb ENDCHR


; Display high-score table.

dishs  ld hl,23693         ; attributes system variable.
       ld (hl),71          ; set permanent attributes.
       call cls            ; clear screen.
       ld hl,hstxt0        ; title at the top.
       call dfont          ; show text.
       ld hl,22592         ; third row of screen attributes.
       ld b,32             ; cells to write.
dishs0 ld (hl),69          ; write cell colour.
       inc l               ; next cell.
       djnz dishs0         ; repeat for rest of line.
       ld a,244            ; delay of approx 10 seconds.
       jp menu1            ; return to menu.

; Try high score table to see if score qualifies.

tryhs  ld hl,hsend         ; end of high score table.
       ld b,HSENT          ; number of scores to check.
tryhs0 push bc             ; store list counter.
       ld de,score         ; player's score.
       push hl             ; store position in table.
       ld b,7              ; digits to compare.
tryhs3 ld a,(de)           ; player's score digit.
       cp (hl)             ; table digit.
       jr c,tryhs1         ; table's score is greater.
       jr nz,tryhs2        ; player's score is higher.
       inc hl              ; digit is the same - go to next column.
       inc de
       djnz tryhs3         ; repeat for all digits.
       jr tryhs1           ; score is same, first one registered comes first.
tryhs2 pop hl              ; restore position in table.
       ld de,65517         ; 65536 - 19.
       add hl,de           ; subtract 19, next entry in table.
       pop bc              ; restore entry counter.
       djnz tryhs0         ; repeat.

; Top of table.

       ld hl,topsc         ; first entry in table.

; Now move rest of table down.

tryhs4 push hl
       ld de,65527         ; beyond name.
       add hl,de           ; go back to start of entry.
       ex de,hl            ; put that in de.
       ld hl,hslast        ; past end of table.
       and a               ; clear carry flag.
       sbc hl,de           ; subtract last address to find length.
       jr c,hsat           ; zero, nothing to do.
       ld b,h
       ld c,l
       ld de,hslast+18     ; last physical byte of table.
       ld hl,hslast-1      ; next one up.
       lddr                ; move table down.

; Now reset the at codes.

hsat   ld hl,hstab+1       ; high score at codes.
       ld b,HSENT          ; entries in table.
       ld a,HSROW          ; first row number.
       ld de,19            ; distance between entries.
hsat0  ld (hl),a           ; reset code.
       add hl,de           ; point to next at code.
       inc a               ; down 2 lines for next one.
       inc a
       djnz hsat0          ; repeat for all entries.

       pop de              ; restore pointer to score.
       ld hl,score         ; player's score.
       ld bc,7             ; number of digits to transfer.
       ldir                ; copy to table.
       ld hl,65520         ; minus 16, length of name and score.
       add hl,de           ; go back to start of name.
       push hl             ; store address on stack.
       call ent            ; enter name.
       call rude           ; test for obscenities, cheats etc.
       pop de
       ld hl,buffer        ; place where name is stored.
       ld bc,8             ; length of name.
       ldir                ; copy to high score table.
       ret
tryhs1 pop hl
       pop bc
       ld a,b
       cp HSENT            ; failed at bottom of table?
       ret z               ; yes - no high score then.
       ld de,19            ; next element down.
       add hl,de           ; point to next in list.
       jr tryhs4           ; put in table at this point.

; Check if a known word has been entered.
; On entry HL points to the buffer.

word   ld de,vocab             ; table of naughty words.
word4  ld hl,buffer            ; point to buffer.
word0  ld a,(de)               ; DE=char in vocabulary table.
       inc de
       cp (hl)                 ; do they match?
       jr nz,word1             ; no - find next word.
       cp 255                  ; end of listed word?
       jr z,word2              ; yes - try next word in table.
       inc hl                  ; next buffer char.
       jr word0
word1  cp 255                  ; end of word?
       jr z,word5              ; yes - return verb number.
word2  ld a,(de)
       inc de
       cp 255                  ; have we reached end of word in table?
       jr nz,word2             ; no, loop round again.
       ld a,(de)               ; which word is this?
       inc de
       cp 255                  ; is it the last one?
       jr nz,word4             ; no.
       xor a                   ; return 0 (not found) in A.
       ret
word5  ld a,(de)               ; name number returned in A.
       ret

; This is the recognised naughty word list.


vocab  defb 0,0,0,0,0,0,0,0,255,1      ; empty.
       defb 6,21,3,11,255,2            ; swear words.
       defb 3,21,14,20,255,2           ; swear words.
       defb 2,15,12,12,15,3,11,19,255,2
       defb 1,18,19,5,255,2
       defb 19,8,9,20,255,2
       defb 23,1,14,11,255,2
       defb 16,9,19,19,255,2
       defb 20,23,1,20,255,2
       defb 20,9,20,19,255,2
       defb 16,18,9,3,11,255,2
       defb 3,15,13,13,15,4,255,2
       defb 18,7,255,4                 ; RG cheat.
       defb 10,16,13,255,4             ; JPM cheat.
       defb 2,6,7,255,4                ; BFG cheat.
       defb 7,12,21,7,255,4            ; GLUG cheat.
       defb 255,255
namec  defb 19,1,21,19,1,7,5,0         ; sausage.
name   defb 16,15,1,3,8,5,4,0          ; poached.
namep  defb 6,18,9,5,4,0,0,0           ; fried.
nameq  defb 11,14,15,2,0,0,0,0         ; custom name.


; Oh dear, how boring.  Let's put something else in for him.

entemp ld hl,namep         ; blank name substitute.
entob0 ld de,buffer        ; text buffer.
       ld bc,8             ; number of chars to copy.
       ldir                ; copy name into buffer.
       ret

rude   ld hl,buffer        ; point to buffer.
       call word           ; find word in list.
       and a               ; is it recognised?
       ret z               ; no, it's okay as it is then.
       dec a               ; is it empty?
       jr z,entemp         ; replace rudeness.
       dec a               ; is it naughty?
       jr z,entob          ; replace rudeness.
       dec a               ; is it the custom stuff?
       jp z,endcus         ; yes, let's be silly.
       dec a               ; is it the cheat?
       ret nz              ; no, it's okay as it is then.
       ld a,37             ; value for five lives.
       ld (cheatp+1),a     ; set cheat mode.
       ld a,43             ; value for fourth life symbol.
       ld (cheatt),a       ; obscure end marker in panel.
       ld hl,namec         ; cheat name.
       jr entob0

entob  ld hl,name          ; obscenity substitute.
       jr entob0           ; use this name instead.

endcus ld a,33             ; value for one life.
       ld (cheatp+1),a     ; set cheat mode.
       ld hl,nameq         ; custom name.
       jr entob0           ; copy name across.

circa  defb 0              ; circle angle.
circd  defb 0              ; circle direction.
npos   defw buffer         ; address in buffer.
namstr defb ATPOS,15,12


redkey call 654            ; D=shift, E=key.
       ld a,e              ; get value.
       inc a               ; is it 255?
       jr nz,redkey        ; yes - loop until key is released.
       call cls            ; ROM routine to clear screen.
       ld hl,keys          ; redefined keys.
       ld de,keys+1
       ld bc,5
       ld (hl),255
       ldir                ; clear keys.
       ld hl,ktxt0         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys),a         ; set this as up key.
       ld hl,ktxt1         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+1),a       ; set this as down key.
       ld hl,ktxt2         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+2),a       ; set this as left key.
       ld hl,ktxt3         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+3),a       ; set this as right key.
       ld hl,ktxt4         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+4),a       ; set this as fire key.
       ld hl,ktxt5         ; prompt to display.
       call dfont          ; display prompt for next key.
       call redky0         ; wait for keypress.
       ld a,e
       ld (keys+6),a       ; set this as pause key.
       jp menua            ; back to main menu now.

ktxt0  defb ATPOS,2,9
       defb 16,18,5,19,19,0,11,5,25,0,6,15,18
       defb ATPOS,5,7
       defb 21,16,ATPOS,5,24
       defb ENDCHR
ktxt1  defb ATPOS,7,7
       defb 4,15,22,14,ATPOS,7,24
       defb ENDCHR
ktxt2  defb ATPOS,9,7
       defb 12,5,6,20,ATPOS,9,24
       defb ENDCHR
ktxt3  defb ATPOS,11,7
       defb 18,9,7,8,20,ATPOS,11,24
       defb ENDCHR
ktxt4  defb ATPOS,13,7
       defb 6,9,18,5,ATPOS,13,24
       defb ENDCHR
ktxt5  defb ATPOS,17,7
       defb 16,1,21,19,5,ATPOS,18,24
       defb ENDCHR
ktxt7  defb 2,8,25,38,37,20,7,22
       defb 14,10,21,39,36,18,6,3
       defb 13,11,9,28,35,5,4,24
       defb 27,12,15,41,34,23,19,26
       defb 27,27,16,32,33,17,1,27

redky0 call 654            ; D=shift, E=key.
       ld a,e              ; get value.
       inc a               ; is it 255?
       jr z,redky0         ; yes - loop until something pressed.
       ld a,(keys)         ; first key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+1)       ; down key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+2)       ; left key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+3)       ; right key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+4)       ; fire key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
       ld a,(keys+6)       ; pause key.
       cp e                ; is this the same?
       jr z,redky0         ; yes - try again.
redky1 push de
       ld hl,ktxt7         ; table of key names.
       ld d,0
       add hl,de           ; name of key in look-up table.
       ld a,(hl)           ; want result in DE.
;       push af             ; store character.
;       call scadd          ; find screen address.
;       pop af              ; restore character.
       call dfont1         ; routine to display character.
       pop de
       ret
redky2 ld hl,ktxt7         ; table of key names.
       ld d,0              ; no high byte for displacement.
       add hl,de           ; name of key in look-up table.
       ld a,(hl)           ; want result in DE.
       call dfont1         ; routine to display character.
       ret

gamek  xor a               ; keyboard setting.
       jr howc             ; start.
gamejk ld a,1              ; Kempston setting.
       jr howc             ; start.
gamejs ld a,2              ; Sinclair setting.
       jr howc             ; start.
howc   ld (contrl),a       ; set control type.

; Game initialisation.

       call cls            ; first we clear the screen.

       ld hl,23266         ; score digits attributes.
       ld b,7              ; number of digits.
attsc  ld (hl),66          ; set colour to red on black.
       inc l               ; next attribute cell.
       djnz attsc          ; repeat attributes for rest of score.

       ld hl,score         ; player's score.
       ld b,6              ; number of digits.
scinit ld (hl),32          ; write a zero in this column.
       inc hl              ; next column.
       djnz scinit         ; repeat for all columns.

       ld hl,33*256+32     ; zero and one.
       ld (stage),hl       ; set up display stage number.

       xor a               ; put zero in accumulator.
       ld (levno),a        ; initialise level.
cheatp ld a,3              ; put 3 in accumulator.
       ld (numlif),a       ; set initial lives.
       call dpanel         ; display the panel.

; Restart after end of life.

rstart xor a               ; put zero in accumulator.
       ld (dflag),a        ; reset player dead flag.
       ld (convd),a        ; reset conveyor direction.
       ld (amotyp),a       ; set up ammo type.
       ld (lcount),a       ; reset level completed counter.
       ld (snooze),a       ; reset snooze period.
       ld (timoff),a       ; reset conveyor downtime.
       ld a,1              ; one in accumulator.
       ld (joylas),a       ; set up firing direction.
       ld a,2              ; bonuses before sedative arrives.
       ld (sedcnt),a       ; set sedative counter.
       ld a,6              ; starting ammunition.
       ld (ammo),a         ; set up starting ammo.

       ld a,8              ; time until ammo appears.
       ld (amtim),a        ; set ammo time delay.
       ld (botim),a        ; set bonus time delay.
       ld a,(levno)        ; level number.
       rlca                ; multiply by 2.
       add a,10            ; customers to serve.
       ld (appear),a       ; number left to appear.
       ld (served),a       ; number left to serve.

       call dsausg         ; show ammunition in status panel.
       call xspr           ; clear sprites.
       call xslot          ; clear slots.

       ld a,(levno)        ; level number.
       rlca                ; multiply by sixteen.
       rlca
       rlca
       rlca
       ld e,a              ; copy to de.
       ld d,0              ; no high byte.
       ld hl,spwtab        ; sprite spawning tables.
       add hl,de           ; point to spawn table.
       ld (spwptr),hl      ; set up relevant spawn table pointer.

       ld a,(levno)        ; level number.
       ld e,a              ; copy to de.
       ld hl,spodtb        ; sprite spawn odds tables.
       add hl,de           ; point to spawn table.
       ld a,(hl)           ; get odds of spawning.
       ld (spodds),a       ; set spawning odds for this level.

       call dscor          ; display score.
       call chwinx         ; draw display window.
       ld hl,redtxt        ; get ready message.
       call dfont          ; display message.

       ld b,100            ; delay length.
       call delay          ; short pause.

       ld hl,remtxt        ; removal message.
       call dfont          ; display message.

; Spawn the player sprite.

       ld ix,sprtab        ; first item in sprite table.
       ld (ix+6),PLYTYP    ; player type.
       ld (ix+7),PLRIMG    ; player right image.
       ld (ix+8),88        ; set x coordinate.
       ld (ix+9),120       ; set y coordinate.
       ld (ix+10),0        ; rest frame number.
       call dspr           ; display player sprite.
       call cpspr          ; copy sprites.

       jp mloop            ; all done, start main loop.

gfram  defb 0              ; game frame number.
amtim  defb 7              ; ammunition timer.
botim  defb 3              ; bonus time delay.

; Clear slots.

xslot  ld hl,cslotl        ; customer slots.
       ld b,SLOTS          ; number of slots.
xslot0 ld (hl),0           ; clear queue.
       inc hl              ; next slot.
       djnz xslot0         ; repeat for all slots.
       ret

; Clear sprite table.

xspr   ld ix,sprtab        ; sprite table.
       ld b,NUMSPR         ; sprites to initialise.
       ld de,SPRSIZ        ; bytes per table entry.
       ld c,255            ; value for "off".
xspri0 ld (ix+0),c         ; switch off entry.
       ld (ix+6),c         ; switch off entry for next frame too.
       add ix,de           ; point to next one.
       djnz xspri0         ; repeat for all items.
       ret


; Joystick and keyboard reading routines.

joykey ld a,(contrl)       ; control flag.
       and a               ; is it the keyboard?
       jr nz,joyjoy        ; no, it's joystick.

; Keyboard controls.

       ld e,0              ; control.
       ld a,(keys+4)       ; fire key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys)         ; up key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys+1)       ; down key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into e register.
       ld a,(keys+2)       ; left key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; rotate into bit d0 of e register.
       ld a,(keys+3)       ; left key.
       call ktest          ; being pressed?
       ccf                 ; complement the carry.
       rl e                ; destined for bit d1 of e register.
       ld a,e              ; place result in accumulator.
       ld (joyval),a       ; put that into joyval.
       ret

joyjoy dec a               ; is it Kempston?
       jr z,joykem         ; yes.
       dec a               ; is it Sinclair?
       jr z,joysin         ; yes.

; Cursor joystick controls.

joycur ld bc,61438         ; port for keys 6-9.
       ld e,0              ; default zeroes.
       in a,(c)            ; read joystick.
       ld d,a              ; copy to d register.
       rra                 ; rotate bit out.
       ccf                 ; toggle bit.
       rl e                ; rotate into e register.
       bit 3,d             ; key pressed?
       jr nz,joycu0        ; no, zero bit.
       scf                 ; set carry flag.
joycu0 rl e                ; rotate into e register.
       bit 4,d             ; key pressed?
       jr nz,joycu1        ; no, zero bit.
       scf                 ; set carry flag.
joycu1 rl e                ; rotate into e register.

       ld bc,63486         ; port for 1-5.
       in a,(c)            ; read joystick.
       and 16              ; innermost key of row.
       jr nz,joycu3        ; not pressed.
       ccf                 ; toggle bit.
joycu3 rl e                ; rotate into e register.

       bit 2,d             ; key pressed?
       jr nz,joycu2        ; no, zero bit.
       scf                 ; set carry flag.
joycu2 rl e                ; rotate into e register.
       ld a,e              ; copy to accumulator.
       ld (joyval),a       ; store value.
       ret

; Sinclair joystick controls.

joysin ld bc,61438         ; port for Sinclair 2.
       in a,(c)            ; read joystick.
       ld d,a              ; clear values.
       xor a               ; clear accumulator.
       ld e,16             ; Kempston fire bit value.
       bit 0,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,1              ; Kempston bit value.
       bit 3,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,2              ; Kempston bit value.
       bit 4,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,8              ; Kempston bit value.
       bit 1,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld e,4              ; Kempston bit value.
       bit 2,d             ; fire bit pressed?
       call z,joysi0       ; add bit.
       ld (joyval),a       ; remember value.
       ret

joysi0 add a,e             ; add bit value.
       ret


; Kempston joystick controls.

joykem ld bc,31            ; port for Kempston interface.
       in a,(c)            ; read it.
       ld (joyval),a       ; remember value.
       cp 32               ; is it within standard bit range?
       ret c               ; yes.

; Kempston isn't valid.  Force keyboard from now on.

       xor a               ; zero in accumulator.
       ld (contrl),a       ; use keyboard.
       ret

; Key test routine.

ktest  ld c,a              ; key to test in c.
       and 7               ; mask bits d0-d2 for row.
       inc a               ; in range 1-8.
       ld b,a              ; place in b.
       srl c               ; divide c by 8
       srl c               ; to find position within row.
       srl c
       ld a,5              ; only 5 keys per row.
       sub c               ; subtract position.
       ld c,a              ; put in c.
       ld a,254            ; high byte of port to read.
ktest0 rrca                ; rotate into position.
       djnz ktest0         ; repeat until we've found relevant row.
       in a,(254)          ; read port (a=high, 254=low).
ktest1 rra                 ; rotate bit out of result.
       dec c               ; loop counter.
       jp nz,ktest1        ; repeat until bit for position in carry.
       ret

; User defined keys.

keys   defb 28,29,27,35,16,1

joylas defb 0              ; previous joystick reading.
joyval defb 0              ; keyboard/joystick value.
contrl defb 0              ; control method chosen.
joydir defw 0              ; joystick horizontal and vertical flags.
pretim defb 0              ; previous clock reading.
sndtyp defb 0              ; sound to play.


; Synch routine.

haltr  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,haltr0        ; yes, no more processing please.

haltr1 call house          ; perform housekeeping.

; Sound stuff.

       ld a,(sndtyp)       ; type of sound.
       rlca                ; multiply by 2.
       ld e,a              ; copy to de.
       ld d,0              ; no high byte.
       ld hl,sndtab        ; sound table.
       add hl,de           ; point to sound effect routine.
       ld e,(hl)           ; sound address low.
       inc hl              ; second byte.
       ld d,(hl)           ; sound address high.
       ld hl,haltr4        ; return address once sound is finished.
       push hl             ; put address on the stack.
       ex de,hl            ; flip sound into hl.
       jp (hl)             ; play the sound.

haltr4 xor a               ; zero in accumulator.
       ld (sndtyp),a       ; reset sound.

haltr3 ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,haltr0        ; yes, no more processing please.
       jp haltr3

haltr0 ld a,(23672)        ; current clock setting.
       ld (pretim),a       ; new setting.
       ret

sndtab defw dron           ; background drone.
       defw snd4           ; chirp.
       defw snd2           ; explosion.
       defw snd3           ; confirmation sound.
       defw buzz           ; buzzy bee sound.
       defw chai           ; chain-like clink.
       defw cric           ; cricket-like sound.
       defw bang           ; bouncing sound.
       defw plop           ; white noise plop.

; Chirp.

snd4   ld b,40             ; outer loop.
       ld c,0              ; value to write to speaker.
snd4c  ld a,c              ; get speaker value.
       out (254),a         ; write to speaker.
       xor 248             ; toggle all except the border bits.
       ld c,a              ; store value for next time.
       push bc             ; store loop counter.
snd4a  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,snd4b         ; yes, no more processing please.
       djnz snd4a          ; loop.
       pop bc              ; restore loop counter.
       djnz snd4c          ; continue noise.
       ret

; Time up, cannot complete the sound.

snd4b  pop bc              ; restore loop counter.
       ret


; Confirmation sound.

snd3   ld b,100            ; outer loop.
       ld c,0              ; value to write to speaker.
snd3c  ld a,c              ; get speaker value.
       out (254),a         ; write to speaker.
       xor 248             ; toggle all except the border bits.
       ld c,a              ; store value for next time.
       push bc             ; store loop counter.
       ld a,101
       sub b
       ld b,a
snd3a  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,snd3b         ; yes, no more processing please.
       djnz snd3a          ; loop.
       pop bc              ; restore loop counter.
       djnz snd3c          ; continue noise.
       ret

; Time up, cannot complete the sound.

snd3b  pop bc              ; restore loop counter.
       ret

; Explosion sound.

snd2   ld b,80             ; outer loop.
snd2c  call random         ; get speaker value.
       and 248             ; toggle all except the border bits.
       out (254),a         ; write to speaker.
       ld c,40             ; inner loop counter.
       ld hl,pretim        ; previous clock setting.
snd2a  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       dec c               ; decrement inner loop.
       jr nz,snd2a         ; loop until c reaches zero.
       djnz snd2c          ; continue noise.
       ret


; Buzzy bee sound.

buzz   ld c,40             ; outer loop.
       call random         ; get speaker value.
       and 7               ; want range 0 - 7.
       add a,5             ; minimum value.
       ld e,a              ; set inner loop length.
buzz1  xor a               ; reset speaker bit.
       out (254),a         ; write to speaker.
       ld b,e              ; inner loop counter.
       ld hl,pretim        ; previous clock setting.
buzz0  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       djnz buzz0          ; repeat.
       ld a,248            ; set all except the border bits.
       out (254),a         ; write to speaker.
       ld hl,pretim        ; previous clock setting.
buzz2  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       djnz buzz2          ; repeat.
       dec c               ; decrement outer loop.
       jr nz,buzz1         ; loop until c reaches zero.
       ret

dron   ld a,(convd)        ; conveyor direction.
       and a               ; is it moving?
       ret z               ; no, so no sound.
       ld c,50             ; outer loop.
dron0  xor a               ; reset all bits.
       out (254),a         ; output to speaker.
       ld hl,pretim        ; previous clock setting.
       ld a,248            ; set non-border bits.
       out (254),a         ; output to speaker.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       ld a,r
       and 63
       add a,192
       ld b,a
dron1  djnz dron1          ; repeat.
       dec c               ; decrement outer loop.
       jr nz,dron0         ; loop until c reaches zero.
       ret

; Chain-like sound, a short, high-pitched whistle.
; Could be turned into a telephone noise?

chai   ld c,100            ; outer loop.
chai0  xor a               ; reset all bits.
       out (254),a         ; output to speaker.
       ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       ld b,20             ; frequency.
chai1  djnz chai1          ; repeat.
       ld a,248            ; set non-border bits.
       out (254),a         ; output to speaker.
       ld b,25             ; frequency.
chai2  djnz chai2          ; repeat.
       dec c               ; decrement outer loop.
       jr nz,chai0         ; loop until c reaches zero.
       ret

; Quiet cricket-like sound.

cric   ld c,30             ; outer loop.
       call random         ; get speaker value.
       and 3               ; want range 0 - 31.
       add a,5             ; minimum value.
       ld e,a              ; set inner loop length.
cric1  xor a               ; reset speaker bit.
       out (254),a         ; write to speaker.
       xor 248             ; toggle all except the border bits.
       out (254),a         ; write to speaker.
       ld b,e              ; inner loop counter.
       ld hl,pretim        ; previous clock setting.
cric0  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       djnz cric0          ; repeat.
       dec c               ; decrement outer loop.
       jr nz,cric1         ; loop until c reaches zero.
       ret

; Bang sound.

bang   ld b,100            ; outer loop.
bang0  call random         ; get speaker value.
       and 248             ; toggle all except the border bits.
       out (254),a         ; write to speaker.
       ld c,b              ; inner loop counter.
       ld hl,pretim        ; previous clock setting.
bang1  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       dec c               ; decrement inner loop.
       jr nz,bang1         ; loop until c reaches zero.
       djnz bang0          ; continue noise.
       ret

; Plop sound.

plop   ld b,50             ; outer loop.
       ld e,1              ; starting frequency.
plop0  call random         ; get speaker value.
       and 248             ; toggle all except the border bits.
       out (254),a         ; write to speaker.
       ld c,e              ; inner loop counter.
       ld hl,pretim        ; previous clock setting.
plop1  ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       dec c               ; decrement inner loop.
       jr nz,plop1         ; loop until c reaches zero.
       ld a,e
       add a,5
       ld e,a
       djnz plop0          ; continue noise.
       ret

; Housekeeping.

house  ld a,(gfram)        ; current frame number.
       add a,a             ; shift left.
       and 6               ; place in required range.
       ld e,a              ; low byte in e.
       ld d,0              ; no high byte.
       ld hl,hkeep         ; housekeeping routines.
       add hl,de           ; point to address of routine.
       ld e,(hl)           ; routine address low.
       inc hl              ; second byte.
       ld d,(hl)           ; routine address high.
       ex de,hl            ; flip address into hl.
       jp (hl)             ; carry out housekeeping routine.

; Housekeeping tasks.

hkeep  defw ccust,slh
       defw dscor,slh

; Slot housekeeping.

slh    call slh1           ; process a slot.

; Drop through to process second slot.

slh1   ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       ret nc              ; yes, no more processing please.
       ld a,(shk)          ; slot housekeeping.
       inc a               ; next slot.
       cp SLOTS            ; reached last slot?
       jr c,slh0           ; no, process this one.
       xor a               ; start at first slot again.
slh0   ld (shk),a          ; set new slot to process.
       jp dslot            ; display slot.

shk    defb 0              ; slot housekeeping number.

planim defb 0              ; player animated flag.
ammo   defb 6              ; ammunition.
firefl defb 0              ; fire pressed flag.

; Up the score.

uscor  ld a,(hl)           ; current value of digit.
       add a,b             ; add points to this digit.
       ld (hl),a           ; place new digit back in string.
       cp 42               ; more than value '9'?
       ret c               ; no - relax.
       sub 10              ; subtract 10.
       ld (hl),a           ; put new character back in string.
uscor0 dec hl              ; previous character in string.
       inc (hl)            ; up this by one.
       ld a,(hl)           ; what's the new value?
       cp 42               ; gone past nine?
       ret c               ; no, scoring done.
       sub 10              ; down by ten.
       ld (hl),a           ; put it back.
       jp uscor0           ; go round again.

dflag  defb 0              ; player dead flag.
dcount defb 0              ; dead counter.
spodds defb 3              ; spawning odds.

spodtb defb 4,5,5,6,6,7,7,8,8,9    ; Chances of spawning a new sprite.


; Debug routine - not used in the game.

debug  ld hl,22552
       ld de,sprtab
       ld b,24
debug3 push bc
       ld a,(de)
       ld b,8
debug0 rla
       jr c,debug1
       ld (hl),16
       jr debug2
debug1 ld (hl),24
debug2 inc hl
       djnz debug0
       push de
       ld de,24
       add hl,de
       pop de
       inc de
       pop bc
       djnz debug3
       ret


score  defb 32,32,32,32,32,32,32
       defb ENDCHR
convd  defb 0              ; conveyor direction.


; Display the screen.

chwinx ld hl,screen        ; point to this screen.
       ld a,0              ; top coordinate.
       ld (dispx),a        ; set coordinate.
       ld b,22             ; height in blocks.
chwin0 push bc             ; store row counter.
       xor a               ; left coordinate.
       ld (dispy),a        ; set horizontal starting point.
       ld b,32             ; width in blocks.
chwin1 push bc             ; store column counter.
       ld a,(hl)           ; get character.
       push hl             ; store character address.
       call chwch          ; draw the character.
       pop hl              ; restore pointer to address.
       inc hl              ; next block.
       pop bc              ; restore column count.
       djnz chwin1
       ld a,(dispx)        ; vertical position.
       inc a               ; move down.
       ld (dispx),a        ; next line starts here.
       pop bc              ; restore row count.
       djnz chwin0
       ret

; Draw character block in accumulator.

chwch  ld hl,chatts        ; block attributes.
       ld e,a              ; displacement to attribute.
       ld d,0              ; no high byte for attributes.
       add hl,de           ; point to attribute value.
       ex af,af'           ; store char number.
       call atadd          ; get attribute address.
       ldi                 ; transfer to screen.
       ex af,af'           ; restore char number.
       rlca                ; multiply by 8.
       rlca
       rlca
       ld d,a              ; store shift in d.
       and 248             ; mask away high byte bits.
       ld e,a              ; place result in e.
       ld a,d              ; restore shift.
       and 7               ; mask off the low byte bits.
       ld d,a              ; set high byte for displacement.
       ld hl,chgfx         ; character block graphics.
       add hl,de           ; point to character graphics.
       call chadd          ; get screen character address.

       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.
       dec de              ; back one char.
       inc d               ; next screen line down.
       ldi                 ; transfer to screen.

       ld hl,dispy         ; horizontal position.
       inc (hl)            ; move right one square.
       ret

screen defb 1,3,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,4,1
       defb 1,5,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,5,6,1
       defb 7,9,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,9,8
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0



; Calculate address of attribute for character at (dispx, dispy).

atadd  ld a,(dispx)        ; x position.
       rrca                ; multiply by 32.
       rrca
       rrca
       ld e,a              ; store away in e.
       and 3               ; mask bits for high byte.
scdst1 equ $+1             ; destination address for self-modifying code.
       add a,88            ; 88*256=22528, start of attributes.
       ld d,a              ; high byte done.
       ld a,e              ; get x*32 again.
       and 224             ; mask low byte.
       ld e,a              ; put in l.
       ld a,(dispy)        ; get y displacement.
       add a,e             ; add to low byte.
       ld e,a              ; hl=address of attributes.
       ret

chgfx  defb 0,0,0,0,0,0,0,0
       defb 42,24,42,24,42,24,42,24
       defb 51,56,76,30,138,2,68,16
;       defb 127,62,128,85,170,20,40,20
       defb 0,7,29,63,31,47,159,42
       defb 0,224,88,164,212,165,76,157
       defb 31,7,0,0,2,5,2,0
       defb 250,229,11,95,254,244,160,0
       defb 42,9,36,3,16,5,0,0
       defb 50,80,164,192,16,64,0,0
       defb 0,85,170,255,0,85,0,0
       defb 0,0,2,5,11,20,13,24
       defb 0,0,144,0,196,32,82,16
chatts defb 71,69,70,71,71,71,71,69,69,69,69,69


dswitl ld a,BUTSND         ; button sound.
       ld (sndtyp),a       ; set up sound.
       ld a,71             ; bright white on black.
       call colbut         ; colour the buttons.
       ld hl,16385         ; screen address.
       ld de,switp         ; pressed switch image.
       call dslot1         ; show image in slot.
       ld hl,16413         ; screen address.
       ld de,switu         ; unpressed switch image.
       jp dslot1           ; show image in slot.

dswitr ld a,BUTSND         ; button sound.
       ld (sndtyp),a       ; set up sound.
       ld a,71             ; bright white on black.
       call colbut         ; colour the buttons.
       ld hl,16385         ; screen address.
       ld de,switu         ; unpressed switch image.
       call dslot1         ; show image in slot.
       ld hl,16413         ; screen address.
       ld de,switp         ; pressed switch image.
       jp dslot1           ; show image in slot.

misch  xor a               ; zero in accumulator.
       ld (convd),a        ; no conveyor direction.
dswitn ld a,BUTSND         ; button sound.
       ld (sndtyp),a       ; set up sound.
       ld a,71             ; bright white on black.
       call colbut         ; colour the buttons.
       ld hl,16385         ; screen address.
       ld de,switu         ; unpressed switch image.
       call dslot1         ; show image in slot.
       ld hl,16413         ; screen address.
       ld de,switu         ; unpressed switch image.
       jp dslot1           ; show image in slot.

levno  defb 0                  ; level number.
qodds  defb 42,44,46,48,50     ; odds of adding to queue.
       defb 52,53,54,55,56
numlif defb 255                ; number of lives remaining.

; Customers on each level.

cuslev defb 1,2,3,7,5,6,7,1
       defb 2,3,4,5,6,7,2,4
       defb 3,8,7,1,4,5,6,5
       defb 8,2,3,4,1,6,8,7
       defb 6,5,2,3,4,8,1,8
       defb 1,2,3,7,5,6,7,1
       defb 2,3,4,5,6,7,2,4
       defb 3,8,7,1,4,5,6,5
       defb 8,2,3,4,1,6,8,7
       defb 6,5,2,3,4,8,1,8

; Sausage and burger panel text.

burtxt defb ATPOS,23,12
       defb 31,42,31,42,31,42,31,42,31,42,31,42
       defb ENDCHR
sostxt defb ATPOS,23,12
       defb 45,46,45,46,45,46,45,46,45,46,45,46
       defb ENDCHR

pantxt equ $
       defb ATPOS,23,27    ; player lives position.
       defb 43,43,43       ; unused lives.
cheatt defb ENDCHR
       defb 43,ENDCHR

dburgr ld a,66             ; colour.
       call amatt          ; set ammo attributes.
       ld hl,burtxt        ; panel image.
       jp dfont            ; display the panel.
dsausg ld a,67             ; colour.
       call amatt          ; set ammo attributes.
       ld hl,sostxt        ; panel image.
       jp dfont            ; display the panel.
dpanel ld hl,pantxt        ; panel image.
       jp dfont            ; display the panel.

dlife  ld a,(numlif)       ; number of lives.
       add a,26            ; add to first column.
       ld (dispy),a        ; set the column.
       ld a,23             ; line number of status panel.
       ld (dispx),a        ; set display position.
       ld a,44             ; dead player image.
       jp dfont1           ; display one character.
amatt  ld hl,23276         ; attribute start.
       ld b,12             ; cells to write.
amatt0 ld (hl),a           ; set the colour.
       inc l               ; next cell.
       djnz amatt0         ; repeat for all attributes in chain.
       ret

served defb 0              ; customers still to be served.
appear defb 0              ; customers yet to appear.

; Simple screen clearing routine.

cls    ld hl,16384         ; pixels.
       ld de,16385         ; second byte of screen.
       ld bc,6144          ; size of screen.
       ld (hl),l           ; blank the first byte.
       ldir                ; clear rest of screen.
       ld a,(23693)        ; attributes.
       ld (hl),a           ; set attribute.
       ld bc,767           ; bytes to write.
       ldir                ; set attributes for rest of screen.
       ret

lcount defb 0

nexlev ld hl,sedcnt        ; counter to next sedative.
       ld (hl),5           ; make sure it doesn't appear.
       call genbo0         ; speed up bonus generation.
       ld ix,sprtab        ; sprite table.
       ld b,NUMSPR         ; sprites to initialise.
       ld de,SPRSIZ        ; bytes per table entry.
       ld c,255            ; value for "off".
nexl1  ld a,(ix+6)         ; sprite type.
       cp BOWTYP           ; enemy?
       jp z,nexl2          ; yes, convert to something benign.
       cp FRLTYP           ; finger roll?
       jp z,nexl2          ; yes, convert it.
       cp BNCTYP           ; bouncing enemy?
       jp z,nexl2          ; yes, change sprite.
       cp SPWTYP           ; spawning enemy?
       jp z,nexl2          ; yes, change sprite.
nexl3  add ix,de           ; point to next one.
       djnz nexl1          ; repeat for all items.
       ld hl,lcount        ; level counter.
       inc (hl)            ; increment counter.
       ld a,(hl)           ; get counter value.
       cp 130              ; reached maximum value?
       jp c,mloop          ; not yet, return to game.
       ld hl,levno         ; level number.
       ld a,(hl)           ; get current level.
       inc a               ; next level.
       cp MAXLEV           ; reached the maximum one?
       jr c,nexl0          ; no, carry on.
       ld a,MAXLEV         ; repeat the last level.
nexl0  ld (hl),a           ; set the next level.
       ld hl,stage+1       ; units digit of stage number.
       ld b,1              ; amount to add.
       call uscor          ; up the digit(s).
       jp rstart           ; restart.
nexl2  call blowup         ; blow up this nasty.
       jp nexl3            ; continue.

; Kill the player.

kilp   ld ix,sprtab        ; sprite table.
       ld b,NUMSPR         ; sprites to initialise.
       ld de,SPRSIZ        ; bytes per table entry.
kilp0  ld a,(ix+6)         ; sprite type.
       cp PLYTYP           ; enemy?
       call z,blowup       ; yes, blow player up.
       add ix,de           ; point to next one.
       djnz kilp0          ; repeat for all items.
       ret


spwptr defw 0              ; spawn table pointer.

spwtab defw spawnb,spawnf,spawnb,spawnf,spawnb,spawnf,spawnb,spawnf
       defw spawnb,spawnf,spawnb,spawnt,spawnb,spawnf,spawnb,spawnf
       defw spawnb,spawnf,spawnb,spawnt,spawnb,spawnf,spawnb,spawnf
       defw spawnb,spawnf,spawnb,spawnt,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnf,spawnb,spawnh,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnt,spawnb,spawnh,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnt,spawnb,spawnh,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnt,spawnb,spawnh,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnt,spawnb,spawnh,spawnb,spawnf,spawnf,spawnf
       defw spawnb,spawnt,spawnb,spawnh,spawnb,spawnf,spawnh,spawnf

; Delay routine.

delay  halt                ; wait for an interrupt.
       djnz delay          ; repeat until b register reaches zero.
       ret

redtxt defb ATPOS,9,12
       defb 12,5,22,5,12,0
stage  defb 32,32
       defb ATPOS,11,11
       defb 7,5,20,0,18,5,1,4,25
       defb ENDCHR
remtxt defb ATPOS,9,12
       defb 0,0,0,0,0,0,0,0
       defb ATPOS,11,11
       defb 0,0,0,0,0,0,0,0,0
       defb ENDCHR


; Pause mode.

pause  ld a,(keys+5)       ; pause key.
       call ktest          ; has it been pressed?
       jr nc,pause         ; wait until pause key is released.
pause1 ld a,9              ; key J.
       call ktest          ; pressed?
       jr c,pause3         ; no, skip cheat.
       ld a,15             ; key C.
       call ktest          ; pressed?
       jr c,pause3         ; no, skip cheat.
       xor a               ; zero in accumulator.
       ld (served),a       ; none to serve.
       ld (appear),a       ; none to appear.
       ret
pause3 ld a,(keys+5)       ; pause button.
       call ktest          ; pressed it yet?
       jr c,pause1         ; pause game until pause key pressed.
pause2 ld a,(keys+5)       ; pause key.
       call ktest
       jr nc,pause2        ; wait until pause key is released.
       ret

sedcnt defb 1              ; counts down to next sedative bonus.
snooze defb 0              ; snooze period.

sleep  ld hl,snooze        ; snooze period.
       ld (hl),255         ; set to maximum.
       ret

snore  ld hl,snooze        ; snooze period.
       dec (hl)            ; decrement it.
       ret


logo   defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 15,254,1,192,28,112,0,1,255,128,31,255,131,224,62,248,0,7,255,224
       defb 31,255,195,224,62,248,0,15,255,240,31,255,195,224,62,248,0,15,255,240
       defb 31,255,227,224,62,255,255,31,255,248,31,7,227,224,62,255,255,159,129,248
       defb 31,3,227,224,62,255,255,159,0,248,31,7,195,224,62,255,255,159,0,248
       defb 31,255,195,240,126,255,255,31,1,248,31,255,227,255,254,248,0,31,255,248
       defb 31,255,241,255,254,248,0,31,255,240,31,255,241,255,254,248,3,159,255,240
       defb 31,3,248,255,254,248,7,223,255,224,31,1,248,63,254,248,7,223,255,128
       defb 31,1,248,0,62,248,7,223,0,0,31,3,248,0,126,252,15,223,128,0
       defb 31,255,240,255,254,255,255,223,255,224,31,255,241,255,252,127,255,143,255,240
       defb 31,255,225,255,252,127,255,143,255,240,31,255,193,255,248,63,255,7,255,240
       defb 15,255,0,255,224,15,252,1,255,224,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,240,120,3,255,0,0,0
       defb 0,0,3,253,254,15,255,192,0,0,0,0,7,255,255,31,255,224,0,0
       defb 0,0,7,255,255,31,255,224,0,0,0,0,15,255,255,191,255,240,0,0
       defb 0,0,15,223,223,191,3,240,0,0,0,0,15,143,143,190,1,240,0,0
       defb 0,0,15,143,143,190,1,240,0,0,0,0,15,143,143,190,3,240,0,0
       defb 0,0,15,143,143,191,255,240,0,0,0,0,15,143,143,191,255,240,0,0
       defb 0,0,15,143,143,191,255,224,0,0,0,0,15,143,143,191,255,192,0,0
       defb 0,0,15,143,143,191,255,0,0,0,0,0,15,143,143,190,0,0,0,0
       defb 0,0,15,143,143,191,0,0,0,0,0,0,15,143,143,191,255,192,0,0
       defb 0,0,15,143,143,159,255,224,0,0,0,0,15,143,143,159,255,224,0,0
       defb 0,0,15,143,143,143,255,224,0,0,0,0,7,7,7,3,255,192,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

dlogo  ld hl,16395         ; screen address to put logo.
       ld de,logo          ; logo image.
       ld b,48             ; height of image in pixels.
dlogo1 push bc             ; store pixel counter.
       ld c,l              ; low screen address.
       ld b,10             ; width of image in characters.
dlogo0 ld a,(de)           ; image byte.
       ld (hl),a           ; write to screen.
       inc de              ; next source byte.
       inc l               ; next screen address across.
       djnz dlogo0         ; repeat for all rows.
       ld l,c              ; restore screen address low byte.
       call nline          ; next screen address.
       pop bc              ; restore pixel counter.
       djnz dlogo1         ; repeat for all lines.
       ld hl,22539         ; address of attributes.
       ld de,22            ; 32 minus width of logo.
       ld a,6              ; height in characters.
dlogo3 ld b,10             ; width in characters.
dlogo2 ld (hl),2           ; set attributes.
       inc l               ; next cell.
       djnz dlogo2         ; repeat for rest of line.
       add hl,de           ; point to next attribute line.
       dec a               ; one less line.
       jr nz,dlogo3        ; repeat for all characters.
       ret

hstxt2 defb ATPOS,1,10
       defb 7,18,5,1,20,0,19,3,15,18,5
       defb ATPOS,3,8
       defb 5,14,20,5,18,0,25,15,21,18,0,14,1,13,5
       defb ATPOS,8,10,1,0,2,0,3,0,4,0,5,0,6
       defb ATPOS,10,10,7,0,8,0,9,0,10,0,11,0,12
       defb ATPOS,12,10,13,0,14,0,15,0,16,0,17,0,18
       defb ATPOS,14,10,19,0,20,0,21,0,22,0,23,0,24
       defb ATPOS,16,10,25,0,26,0,27,0,0,0,47,0,44
       defb ENDCHR

; Player attained a high score.

ent    ld hl,23693         ; attributes system variable.
       ld (hl),71          ; set permanent attributes.
       call cls            ; clear screen.
       ld hl,hstxt2        ; high score text.
       call dfont          ; prompt for name.

; High score name entry routine.

       ld hl,bufnum        ; buffer chars remaining.
       ld b,8              ; size of name.
       ld (hl),b           ; chars to write.
       ld hl,buffer        ; name buffer.
       ld (bufptr),hl      ; set pointer.
bufwip ld (hl),0           ; write space character to buffer.
       inc hl              ; next buffer character.
       djnz bufwip         ; wipe rest of buffer.
       defb 33,124,156     ; coordinates.
       ld (cursx),hl       ; set cursor position.
entx   call dcur           ; show cursor.
       call dhsnam         ; show name.
ent0   call joykey         ; controls.
       call haltf          ; delay.
       ld a,(joyval)       ; value returned.
       and 31              ; any movement?
       jr z,ent0           ; no.
       rra                 ; move right?
       jp c,entcr          ; cursor right.
       rra                 ; move left?
       jp c,entcl          ; cursor left.
       rra                 ; move left?
       jp c,entcd          ; cursor down.
       rra                 ; move left?
       jp c,entcu          ; cursor up.
       call hschr          ; find char number.
       cp 29               ; is it last char?
       ret z               ; yes, job done.
       cp 28               ; is it delete char?
       jr z,delhs          ; delete char.
       ld b,a              ; store in b.
       ld a,(bufnum)       ; buffer chars remaining.
       and a               ; any space left?
       jr z,ent1           ; no - cannot enter any more.
       ld a,b              ; restore char number.
       inc a               ; 'A' starts at one.
       ld hl,(bufptr)      ; pointer to buffer position.
       ld (hl),a           ; write character.
       inc hl              ; next char.
       ld (bufptr),hl      ; new buffer pointer position.
       call dhsnam         ; show name.
       ld hl,bufnum        ; buffer chars remaining.
       dec (hl)            ; one less.
ent1   call debfir         ; debounce fire.
       jr ent0

delhs  ld hl,bufnum        ; buffer chars left.
       ld a,(hl)           ; remaining.
       cp 8                ; completely empty?
       jp nc,ent0          ; yes, pull the other one.
       inc (hl)            ; one more character to enter.
       ld hl,(bufptr)      ; pointer to buffer position.
       dec hl              ; go back one.
       ld (hl),0           ; write character.
       ld (bufptr),hl      ; write new pointer.
       call dhsnam         ; display name.
       call debfir         ; release fire.
       jp ent0             ; back to loop again.

dhsnam ld hl,buffer-3      ; text buffer.
       jp dfont            ; display name.

bufptr defw buffer
bufnum defb 0


entcr  ld a,(cursy)        ; cursor position.
       cp MAXY             ; reached the edge?
       jp z,ent0           ; yes.
       call dcur           ; show cursor.
       defb 17,0,2         ; displacement coordinates.

entcm  ld b,8              ; number of iterations.
entcm0 push bc             ; store loop counter.
       push de             ; store displacement on stack.
       and a               ; clear carry.
       ld hl,(cursx)       ; cursor coords.
       add hl,de           ; displacement.
       ld (cursx),hl       ; new coordinates.
       call dcur           ; show at new position.
       call haltf          ; delay.
       call dcur           ; now delete cursor.
       pop de              ; displacement.
       pop bc              ; loop counter.
       djnz entcm0
       jp entx

entcl  ld a,(cursy)        ; cursor position.
       cp MINY             ; reached the edge?
       jp z,ent0           ; yes.
       call dcur           ; show cursor.
       defb 17,0,254       ; displacement coordinates.
       jp entcm

entcu  ld a,(cursx)        ; cursor position.
       cp MINX             ; reached the edge?
       jp z,ent0           ; yes.
       call dcur           ; show cursor.
       defb 17,254,255     ; displacement coordinates.
       jp entcm

entcd  ld a,(cursx)        ; cursor position.
       cp MAXX             ; reached the edge?
       jp z,ent0           ; yes.
       call dcur           ; show cursor.
       defb 17,2,0         ; displacement coordinates.
       jp entcm

; Find char based on coordinates of cursor.

hschr  ld a,(cursx)        ; cursor x.
       sub MINX            ; subtract x top.
       rrca                ; multiple of 8.
       rrca                ; multiple of 4.
       ld b,a              ; into b.
       rrca                ; multiple of 2.
       add a,b             ; 2 + 4 = 6.
       ld c,a              ; store line displacement.
       ld a,(cursy)        ; for row we take cursor y.
       sub MINY            ; subtract y left.
       rrca                ; divide by 16.
       rrca
       rrca
       rrca
       add a,c             ; add line displacement.
       cp 26               ; is it letters A-Z?
       ret c               ; yes, that's okay.
       jr z,hschr0         ; it's a full stop.
       cp 28               ; is it a space?
       ret nc              ; no, return delete or enter code.
       ld a,255            ; one below space.
       ret
hschr0 ld a,26             ; one below full stop.
       ret

cursx  defb 61
cursy  defb 36

       defb ATPOS,20,12
buffer defb 0,0,0,0,0,0,0,0,ENDCHR

; Frame synchronisation.

haltf  ld hl,pretim        ; previous clock setting.
       ld a,(23672)        ; current clock setting.
       sub (hl)            ; subtract last reading.
       cp 2                ; reached elapsed time already?
       jp nc,haltr0
       jp haltf

; Display cursor for high score name entry.

dcur   ld a,(cursy)        ; y position.
       and 6               ; position straddling x, multiples of 2.
       rlca                ; multiply by another 8 for multiple of 16.
       rlca
       rlca
       ld e,a              ; get low byte of image displacement.
       ld d,0              ; no high byte.
       ld hl,curimg        ; cursor images.
       add hl,de           ; 16 * position.
       add hl,de           ; 32 * position.
       add hl,de           ; 48 * position.
       ex de,hl            ; flip address into de pair.
       ld bc,(cursx)       ; get coordinates for sprite.
       call scadd          ; get screen address.
       ld a,16             ; sprite height.
dcur0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dcur0
       ret

curimg defb 16,8,0,48,12,0,112,14,0,240,15,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,240,15,0,112,14,0,48,12,0,16,8,0
       defb 4,2,0,12,3,0,28,3,128,60,3,192,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,60,3,192,28,3,128,12,3,0,4,2,0
       defb 1,0,128,3,0,192,7,0,224,15,0,240,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,15,0,240,7,0,224,3,0,192,1,0,128
       defb 0,64,32,0,192,48,1,192,56,3,192,60,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,3,192,60,1,192,56,0,192,48,0,64,32

debfir call haltf          ; delay.
       call joykey         ; control routine.
       and 16              ; fire pressed?
       jr nz,debfir        ; yes, debounce it.
       ret

timoff defb 0              ; time the conveyor has been off.

conoff ld hl,timoff        ; time off.
       ld a,(hl)           ; get downtime.
       inc a               ; reached maximum time?
       jr z,conof0         ; yes, flash the buttons.
       ld (hl),a           ; set new downtime.
       ret
conof0 ld a,(gfram)        ; game frame.
       and 3               ; four ink colours.
       add a,66            ; bright red through to cyan.

; Set button colours to value in accumulator.

colbut ld hl,22529         ; first cell.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ld de,27            ; distance to next button.
       add hl,de           ; point to second button.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       inc l               ; right border.
       inc l               ; left border on next line.
       inc l               ; first button, second row.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       add hl,de           ; point to second button.
       ld (hl),a           ; set colour.
       inc l               ; next cell.
       ld (hl),a           ; set colour.
       ret

; Game over.

gameov call tryhs          ; check if we have achieved a high score.
       xor a               ; first menu screen.
       ld (menuno),a       ; set up so displays second menu (high scores) next.
       jp menu

; Palette entry amendment routine.
; ULAplus-enabled hardware/emulators only.

setpal ld bc,48955         ; register select.
       ld a,64             ; mode select.
       out (c),a           ; set ULAplus mode.
       ld b,255            ; data write.
       ld a,1              ; mode on.
       out (c),a           ; switch on ULAplus.

       ld b,64             ; number of palette table entries to write.
       ld hl,palett        ; palette we want.
       ld e,0              ; register number.
setpa0 push bc             ; store counter.
       ld b,191            ; register select.
       ld a,e              ; register number to write.
       out (c),a           ; write to port.
       ld b,255            ; data select.
       ld a,(hl)           ; get colour data from table.
       out (c),a           ; write to port.
       inc e               ; next clut entry.
       inc hl              ; next table entry.
       pop bc              ; restore counter from stack.
       djnz setpa0         ; set rest of palette.

       ld b,191            ; register select.
       ld a,1              ; first register.
       out (c),a           ; select the register.
       ld b,255            ; data port.
       in a,(c)            ; read register.
       ld hl,palett+1      ; second palette entry.
       cp (hl)             ; are they the same?
       ret nz              ; no, so use default settings.

; ULAplus is enabled.

       ret

; Palette data.

; 80 = brown.
; gggrrrbb b
; 11011001
; 01100011

palett defb 0,2,28,31,224,248,152,191
       defb 0,2,28,31,224,99,152,127
       defb 0,3,28,80,224,67,217,251
       defb 0,3,28,31,224,67,217,251
       defb 0,2,28,31,224,251,152,255      ; flashing.
       defb 0,2,28,31,224,99,152,127
       defb 0,3,29,80,224,67,217,251       ; flashing bright.
       defb 0,3,29,31,224,67,217,251


; Uncontended memory.

       org 32768

; Main loop.

mloop  call haltr          ; synch with display.
       call dspr           ; update sprite display.
       call cpspr          ; copy sprites.
       call pspr           ; process sprites.
       call genam          ; generate ammo sprites.
       call genbo          ; generate bonus sprites.
       call gensp          ; generate spawning sprites.
       call mconv          ; move conveyor belt.
       call queue          ; add to queues.
       call wcust          ; customer waiting code.
       ld a,(keys+5)       ; pause key.
       call ktest          ; test the key.
       call nc,pause       ; pause mode.
       ld hl,gfram         ; frame.
       inc (hl)            ; move frame along.

; Level complete / player dead checks and back to start of main loop.

       ld a,(served)       ; customers still to serve.
       and a               ; are there any left?
       jp z,nexlev         ; no, finished the level.
       ld a,(dflag)        ; dead flag.
       and a               ; is player dead?
       jp z,mloop          ; no, he isn't.
       ld hl,dcount        ; dead timer.
       dec (hl)            ; decrement timer.
       jp nz,mloop         ; not done, back to main loop.
       call dlife          ; display life extinct.
       ld hl,numlif        ; number of lives.
       dec (hl)            ; decrement counter.
       jp nz,rstart        ; restart if lives are left.
       jp gameov           ; game over.


; Process sprites.

pspr   ld ix,sprtab        ; sprite table.
       ld b,NUMSPR         ; number of slots to process.
pspr1  push bc             ; store counter.
       ld a,(ix+6)         ; get sprite type.
       cp 15               ; is it within range?
       call c,pspr2        ; yes, process the sprite.
       ld de,SPRSIZ        ; distance to next entry.
       add ix,de           ; next sprite.
       pop bc              ; retrieve loop counter.
       djnz pspr1          ; repeat for remaining sprites.
       ret
pspr2  call pspr3          ; do the routine.
       ret
pspr3  ld hl,evtyp0        ; sprite type events list.
       rlca                ; multiply type by 2.
       ld e,a              ; displacement in de.
       ld d,0              ; no high byte.
       add hl,de           ; point to address of routine.
       ld e,(hl)           ; address low.
       inc hl              ; next byte of address.
       ld d,(hl)           ; address high.
       ex de,hl            ; swap address into hl.
       jp (hl)             ; go there.

evtyp0 defw play           ; player controlled sprite.
       defw proj           ; projectile fired by player.
       defw dsupp          ; dropping supplies.
       defw boun           ; bouncing enemy.
       defw spawn          ; spawning sprites.
       defw burg           ; burger.
       defw expl           ; explosion.
       defw bowk           ; bounce and walk combo.
       defw bivw           ; bounce inhibited vertically, wait.
       defw dsed           ; dropping customer sedative.
       defw hunt           ; hunter-killer enemy.

play   ld e,71             ; bright white.
       call cspr           ; set sprite colour.
       call colp           ; player collision.
       ld a,(ix+6)         ; get new type if changed.
       cp PLYTYP           ; still a player sprite?
       ret nz              ; no, exit.
       xor a               ; zero in accumulator.
       ld (planim),a       ; reset animation flag.
       call joykey         ; joystick/keyboard routines.
       and 3               ; get horizontal control.
       jr z,playv          ; nothing pressed, do vertical.
       cp 3                ; both pressed?
       jr z,playv          ; yes, do vertical.
       dec a               ; test direction.
       call z,playr        ; direction pressed, so move if we can.
       ld a,(joyval)       ; control reading.
       ld (joylas),a       ; set up direction for firing.
       and 2               ; test direction.
       call nz,playl       ; okay, move player.
playv  ld a,(joyval)       ; control reading.
       and 12              ; get vertical controls.
       jr z,playf          ; nothing pressed.
       cp 12               ; both pressed?
       jr z,playf          ; nothing pressed.
       and 4               ; test direction.
       call nz,playd       ; okay, move player.
       ld a,(joyval)       ; control reading.
       ld (joylas),a       ; set up direction for firing.
       and 8               ; test direction.
       call nz,playu       ; okay, move player.
playf  ld a,(joyval)       ; joystick/keyboard reading.
       and 16              ; was fire pressed?
       jp z,playnf         ; no.
       ld a,(firefl)       ; fire flag.
       and a               ; has fire been released?
       ret nz              ; no, debounce it first.
       ld a,(ammo)         ; ammunition.
       and a               ; is there any left?
       jp z,playa          ; no, do animation.
       call eslot          ; find empty slot.
       jp z,playa          ; can't find a slot, do animation now.

; Set up projectile.

       ld (iy+6),AMMTYP    ; projectile type.
       ld (iy+10),0        ; set up frame.
       ld a,(ix+8)         ; x coord.
       ld (iy+8),a         ; set coordinate.
       ld a,(ix+9)         ; y coord.
       ld (iy+9),a         ; set coordinate.
       ld a,(joylas)       ; joystick reading.
       ld (iy+5),a         ; set up direction.
       ld a,(amotyp)       ; type of ammunition.
       add a,PRJIMG        ; add to basic projectile type.
       ld (iy+7),a         ; set image.
       ld hl,ammo          ; ammo.
       dec (hl)            ; one less.
       ld (firefl),a       ; set fire flag.
       ld a,(hl)           ; ammo left.
       rlca                ; multiply by 2.
       add a,236           ; start of ammo attributes, low byte.
       ld l,a              ; set attributes address low.
       ld h,90             ; attributes address high.
       ld (hl),0           ; black out the ink.
       inc l               ; second cell of the two.
       ld (hl),0           ; black out the ink.
       ld a,FIRSND         ; firing sound.
       ld (sndtyp),a       ; set up to fire.
       jp playa            ; animate player.

playnf ld (firefl),a       ; reset fire flag.
playa  ld a,(planim)       ; player animated flag.
       and a               ; set to non-zero?
       ret z               ; no, don't animate.
       call animsp         ; animate sprite.
       ret
playr  ld a,(ix+9)         ; test y coordinate.
       cp 216              ; at right edge?
       ret nc              ; yes, cannot go further.
       add a,2             ; move 2 pixels.
       ld (ix+9),a         ; set new position.
       ld (planim),a       ; set animation flag.
       ld (ix+7),PLRIMG    ; player right image.
       cp 215              ; approaching switch position?
       ret c               ; no.
       ld a,(ix+8)         ; x coordinate.
       cp 72               ; at switch height?
       ret nc              ; no, missed it.
       ld a,(convd)        ; conveyor direction.
       dec a               ; already moving this way?
       ret z               ; yes, no change required.
       ld a,1              ; new direction = right.
       ld (convd),a        ; set new conveyor direction.
       jp dswitr
playl  ld a,(ix+9)         ; test y coordinate.
       cp 25               ; at left edge?
       ret c               ; yes, cannot go further.
       sub 2               ; move 2 pixels.
       ld (ix+9),a         ; set new position.
       ld (planim),a       ; set animation flag.
       ld (ix+7),PLLIMG    ; player left image.
       cp 25               ; approaching switch position?
       ret nc              ; no.
       ld a,(ix+8)         ; x coordinate.
       cp 72               ; at switch height?
       ret nc              ; no, missed it.
       ld a,(convd)        ; conveyor direction.
       inc a               ; already moving this way?
       ret z               ; yes, no change required.
       ld a,255            ; new direction is left.
       ld (convd),a        ; conveyor direction.
       jp dswitl


playd  ld a,(ix+8)         ; test x coordinate.
       cp 214              ; at bottom edge?
       ret nc              ; yes, cannot go further.
       add a,2             ; move 2 pixels.
       ld (ix+8),a         ; set new position.
       ld (planim),a       ; set animation flag.
       ret
playu  ld a,(ix+8)         ; test y coordinate.
       cp 66               ; at top edge?
       ret c               ; yes, cannot go further.
       sub 2               ; move 2 pixels.
       ld (ix+8),a         ; set new position.
       ld (planim),a       ; set animation flag.
       ret
proj   call animsp         ; animation.
       ld e,67             ; bright magenta.
       call cspr           ; set sprite colour.
       ld b,8              ; default to straight 8 pixel steps.
       ld a,(ix+5)         ; counter.
       add a,16            ; add to distance travelled.
       cp 208              ; reached end of range?
       jp nc,kilspr        ; travelled too far, kill off the sprite.
       ld (ix+5),a         ; set new value.
       and 3               ; test horizontal direction flags.
       jr z,proj0          ; definitely straight.
       cp 3                ; both directions?
       jr z,proj0          ; still straight.
       ld a,(ix+5)         ; direction flags.
       and 12              ; test vertical.
       jr z,proj0          ; definitely straight.
       cp 12               ; both directions?
       jr z,proj0          ; still straight.
       ld b,6              ; use 6-pixel diagonal steps.
proj0  ld a,(ix+5)         ; direction flags.
       and 1               ; left/right directions.
       call nz,projr       ; direction pressed, so move if we can.
       ld a,(ix+5)         ; direction flags.
       and 2               ; test direction.
       call nz,projl       ; okay, move ammo.
       ld a,(ix+5)         ; direction flags.
       and 4               ; test direction.
       call nz,projd       ; okay, move projectile.
       ld a,(ix+5)         ; direction flags.
       and 8               ; test direction.
       call nz,proju       ; okay, move it.
       call colf           ; fired projectile collision.
       ret
projr  ld a,(ix+9)         ; test y coordinate.
       add a,b             ; move several pixels.
       cp 214              ; at right edge?
       jp nc,projch        ; yes, change horizontal.
       ld (ix+9),a         ; set new position.
       ret
projl  ld a,(ix+9)         ; test y coordinate.
       sub b               ; move a few pixels.
       cp 25               ; at left edge?
       jp c,projh0         ; yes, change horizontal.
       ld (ix+9),a         ; set new position.
       ret
projd  ld a,(ix+8)         ; test x coordinate.
       cp 208              ; at bottom edge?
       jp nc,kilspr        ; yes, cannot go further.
       add a,b             ; move a few pixels.
       ld (ix+8),a         ; set new position.
       ret
proju  ld a,(ix+8)         ; test y coordinate.
       cp 49               ; past top edge?
       jp c,kilspr         ; yes, cannot go further.
       sub b               ; move some pixels.
       ld (ix+8),a         ; set new position.
       ret
projh1 ld (ix+9),216       ; set y position.
projch ld a,(ix+5)         ; direction flags.
       xor 3               ; toggle horizontal bits.
       ld (ix+5),a         ; store new direction.
       ld a,WALSND         ; hit wall sound.
       ld (sndtyp),a       ; play sound.
       ret
projh0 ld (ix+9),24        ; set y position.
       jp projch           ; change horizontal direction.

; Dropping supplies.

dsupp  ld e,5              ; bright cyan.
dsupp0 call cspr           ; set sprite colour.
       inc (ix+8)          ; move down.
       ld a,(ix+8)         ; read position from memory.
       cp 214              ; reached bottom?
       ret c               ; no, not yet.
blowup ld (ix+5),0         ; reset counter.
       ld (ix+6),EXPTYP    ; set explosion type.
       ld (ix+7),EXPIMG    ; set explosion image.
       ld (ix+10),0        ; reset frame number.
       ret
dsed   ld e,67             ; bright magenta.
       jp dsupp0


; Bouncing enemy tomato.

boun   call animsp         ; animate sprite.
       ld e,66             ; bright red.
       call cspr           ; colour the sprite.
       ld a,(ix+5)         ; direction flags.
       rla                 ; left/right direction.
       jr c,bounr          ; move right.
       ld a,(ix+9)         ; horizontal position.
       cp 26               ; at left edge?
       jr c,bounch         ; yes, change horizontal.
       sub 2               ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),TMLIMG    ; set up sprite image.
bounv  ld a,(ix+5)         ; direction flags.
       and 64              ; up/down direction.
       jr nz,bound         ; move down.
       ld a,(ix+8)         ; vertical position.
       cp 65               ; at top edge?
       jr c,bouncv         ; yes, change vertical.
       sub 2               ; move sprite.
       ld (ix+8),a         ; set new coordinate.
bounok ret
bounr  ld a,(ix+9)         ; get the horizontal position.
       cp 214              ; can we go right?
       jr nc,bounch        ; no, change direction.
       add a,2             ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),TMRIMG    ; set up sprite image.
       jr bounv            ; move vertically next.
bounch ld a,(ix+5)         ; direction flags.
       xor 128             ; toggle left/right bit.
       ld (ix+5),a         ; set new direction.
       jr bounv            ; move vertically next.
bound  ld a,(ix+8)         ; get the vertical position.
       cp 212              ; can we go down?
       jr nc,bouncv        ; no, change direction.
       add a,2             ; move sprite.
       ld (ix+8),a         ; set new coordinate.
       jr bounok           ; moves done.
bouncv ld a,(ix+5)         ; direction flags.
       xor 64              ; toggle the vertical bit.
       ld (ix+5),a         ; set new direction.
       jr bounok           ; moves complete.

; Bounce a bit, walk a bit.

bowk   ld e,70             ; bright yellow.
       call cspr           ; set sprite colour.
       call animsp         ; animate sprite.
       ld a,(ix+5)         ; direction flags.
       rla                 ; left/right direction.
       jr c,bowkr          ; move right.
       ld a,(ix+9)         ; horizontal position.
       cp 26               ; at left edge?
       jr c,bowkch         ; yes, change horizontal.
       sub 2               ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),BBLIMG    ; left bap.
bowkv  ld a,(ix+5)         ; get counter.
       ld b,a              ; store in b.
       and 192             ; direction bits.
       ld c,a              ; store those in c register.
       ld a,b              ; get counter and direction.
       inc a               ; increment it.
       and 63              ; only want counter bits.
       or c                ; combine with direction.
       ld (ix+5),a         ; new value.
       and 32              ; look again at counter.
       jp z,ranver         ; yes, no vertical movement - but choose next vertical direction.
       ld a,(ix+5)         ; direction flags.
       and 64              ; up/down direction.
       jr nz,bowkd         ; move down.
       ld a,(ix+8)         ; vertical position.
       cp 65               ; at top edge?
       jr c,bowkcv         ; yes, change vertical.
       sub 2               ; move sprite.
       ld (ix+8),a         ; set new coordinate.
       ret
bowkr  ld a,(ix+9)         ; get the horizontal position.
       cp 214              ; can we go right?
       jr nc,bowkch        ; no, change direction.
       add a,2             ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),BBRIMG    ; right-facing bap.
       jr bowkv            ; move vertically next.
bowkch ld a,(ix+5)         ; direction flags.
       xor 128             ; toggle left/right bit.
       ld (ix+5),a         ; set new direction.
       jr bowkv            ; move vertically next.
bowkd  ld a,(ix+8)         ; get the vertical position.
       cp 212              ; can we go down?
       jr nc,bowkcv        ; no, change direction.
       add a,2             ; move sprite.
       ld (ix+8),a         ; set new coordinate.
       ret
bowkcv ld a,(ix+5)         ; direction flags.
       xor 64              ; toggle the vertical bit.
       ld (ix+5),a         ; set new direction.
       ret
ranver ld a,r              ; random enough number over a period.
       and 64              ; vertical direction bit.
       xor (ix+5)          ; combine with vertical direction.
       ld (ix+5),a         ; set new vertical direction.
       ret

; Finger roll - bounce inhibited vertically, wait periodically.

bivw   ld e,70             ; bright yellow.
       call cspr           ; set sprite colour.
       ld a,(gfram)        ; game frame.
       rra                 ; test low bit counter status.
       call c,animsp       ; animate sprite every other frame.
       ld a,(ix+5)         ; get counter.
       ld b,a              ; store in b.
       and 192             ; direction bits.
       ld c,a              ; store those in c register.
       ld a,b              ; get counter and direction.
       inc a               ; increment it.
       and 63              ; only want counter bits.
       or c                ; combine with direction.
       ld (ix+5),a         ; new value.
       ld b,a              ; copy to b register.
       and 48              ; look again at counter.
;       ret z               ; don't move this time.
       jr z,bivwv          ; don't move horizontally this time.
       ld a,b              ; restore direction flags.
       rla                 ; left/right direction.
       jr c,bivwr          ; move right.
       ld a,(ix+9)         ; horizontal position.
       cp 26               ; at left edge?
       jr c,bivwch         ; yes, change horizontal.
       sub 2               ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),FRLIMG    ; set left image.
bivwv  ld a,(ix+5)         ; direction flags.
       and 64              ; up/down direction.
       jr nz,bivwd         ; move down.
       ld a,(ix+8)         ; vertical position.
       cp 65               ; at top edge?
       jr c,bivwcv         ; yes, change vertical.
       dec a               ; move sprite up slowly.
       ld (ix+8),a         ; set new coordinate.
       ret
bivwr  ld a,(ix+9)         ; get the horizontal position.
       cp 214              ; can we go right?
       jr nc,bivwch        ; no, change direction.
       add a,2             ; move sprite.
       ld (ix+9),a         ; set new coordinate.
       ld (ix+7),FRRIMG    ; set right image.
       jr bivwv            ; move vertically next.
bivwch ld a,(ix+5)         ; direction flags.
       xor 128             ; toggle left/right bit.
       ld (ix+5),a         ; set new direction.
       jr bivwv            ; move vertically next.
bivwd  ld a,(ix+8)         ; get the vertical position.
       cp 212              ; can we go down?
       jr nc,bivwcv        ; no, change direction.
       inc a               ; move sprite down slowly.
       ld (ix+8),a         ; set new coordinate.
       ret
bivwcv ld a,(ix+5)         ; direction flags.
       xor 64              ; toggle the vertical bit.
       ld (ix+5),a         ; set new direction.
       ret                 ; moves complete.


; Spawning sprites.

spawn  ld a,(sndtyp)       ; current sound type.
       and a               ; has a sound been defined?
       jr nz,spawns        ; yes, don't add this sound.
       ld a,SPWSND         ; spawning sound.
       ld (sndtyp),a       ; set up the sound.
spawns ld a,(ix+5)         ; spawn time.
       and 7               ; zero to seven.
       jp nz,spawnc        ; colour not black, it's okay.
       ld a,4              ; default to green.
spawnc ld e,a              ; put colour in e register.
       call cspr           ; colour this sprite.
       ld a,(ix+5)         ; spawn time.
       and 1               ; time to change frame?
       call z,animsp       ; animate the sprite first.
       ld a,(ix+5)         ; spawn time.
       inc a               ; increment time.
       ld (ix+5),a         ; set new time.
       cp 75               ; finished spawning?
       ret c               ; no, we're done for now.
       call random         ; random number.
       and 14              ; eight multiples of two.
       ld e,a              ; displacement in de.
       ld d,0              ; no high byte.
       ld hl,(spwptr)      ; pointer to table of spawn routines.
       add hl,de           ; point to pointer to spawn routine.
       ld e,(hl)           ; spawn address low.
       inc hl              ; second byte of pointer.
       ld d,(hl)           ; spawn address high.
       ex de,hl            ; flip into hl.
       jp (hl)             ; executre spawn routine.

; Spawn a tomato.

spawnt ld (ix+6),BNCTYP    ; set up a bouncer.
       call random         ; get random number.
       ld (ix+5),a         ; randomise flags for this sprite.
       ld (ix+7),TMLIMG    ; set up sprite image.
       ld (ix+10),0        ; reset frame.
       ret

; Spawn a hunter.

spawnh ld a,(ix+8)         ; x coordinate.
       cp 100              ; near top of screen?
       jp c,spawnt         ; yes, spawn a tomato instead.
       ld a,(ix+9)         ; y coordinate.
       cp 64               ; near left edge?
       jp c,spawnt         ; spawn a tomato.
       cp 174              ; near right edge?
       jp nc,spawnt        ; spawn tomato then.
       ld (ix+6),HUNTYP    ; set up a hunter.
       ld (ix+7),BRLIMG    ; set up sprite image.
       ld (ix+10),0        ; reset frame.
       ret

noesc  ld a,(levno)        ; current level.
       cp 2                ; first two levels?
       jp c,burg1          ; yes, back to burger routine.

; Spawn a burger bun.

spawnb ld (ix+6),BOWTYP    ; set up a bouncing/walking sprite.
       call random         ; get random number.
       ld (ix+5),a         ; randomise flags for this sprite.
       ld (ix+7),BBLIMG    ; set up sprite image.
       ld (ix+10),0        ; reset frame.
       ret

; Spawn a finger roll for a hot dog.

spawnf ld (ix+6),FRLTYP    ; set up a bouncer.
       call random         ; get random number.
       ld (ix+5),a         ; randomise flags for this sprite.
       ld (ix+7),FRLIMG    ; set up sprite image.
       ld (ix+10),0        ; reset frame.
       ret

; Burger.

burg   ld e,70             ; bright yellow.
       call cspr           ; set sprite colour.
       inc (ix+5)          ; increment counter.
       jp z,noesc          ; escape conveyor, turn back into enemy.
burg1  ld a,(ix+8)         ; x coordinate.
       cp 214              ; reached bottom yet?
       jr nc,burg0         ; no, move left/right according to conveyor.
       inc a               ; move down a pixel.
       ld (ix+8),a         ; new coordinate.
       ret
burg0  ld a,(convd)        ; conveyor direction.
       and a               ; is it on?
       ret z               ; no, don't move.
       add a,(ix+9)        ; add to y position.
       ld (ix+9),a         ; set new coordinate.
       cp 16               ; reached left edge?
       jr c,servl          ; serve left customer.
       cp 224              ; reached right edge?
       ret c               ; no, it's okay.
servr  ld (ix+6),255       ; switch off burger.
       ld de,cslotr        ; first slot.
       ld a,(de)           ; get slot contents.
       and a               ; was anyone there?
       ret z               ; no, food was wasted.
       ld hl,cslotr+1      ; second of right slots.
       ld bc,NQUEUE - 1    ; queue length.
       ldir                ; shuffle up.
       xor a               ; put zero in accumulator.
       ld (de),a           ; clear last slot.
       jp bscor            ; add score for burger.
servl  ld (ix+6),255       ; switch off burger.
       ld de,cslotl        ; first slot.
       ld a,(de)           ; get slot contents.
       and a               ; was anyone there?
       ret z               ; no, food was wasted.
       ld hl,cslotl+1      ; second of left slots.
       ld bc,NQUEUE - 1    ; queue length.
       ldir                ; shuffle up.
       xor a               ; clear the accumulator.
       ld (de),a           ; clear last slot.
bscor  ld hl,served        ; number of customers left to serve.
       dec (hl)            ; decrement counter.
       ld a,FEDSND         ; customer fed sound.
       ld (sndtyp),a       ; play sound.s
       ld hl,score+4       ; hundreds column.
       ld b,1              ; amount to add.
       jp uscor            ; up score by that amount.

expl   ld e,68             ; bright green.
       call cspr           ; set sprite colour.
       ld a,EXPSND         ; explosion sound.
       ld (sndtyp),a       ; play the sound.
       inc (ix+5)          ; increment counter.
       ld a,(ix+5)         ; counter number.
       rra                 ; every other frame.
       ret c               ; don't change frame yet.
       call animsp         ; animate sprite.
       ret nz              ; not looped yet.

; Drop into routine to kill off this sprite.

kilspr ld (ix+6),255
       ret

; Hunter-killer sprites.

hunt   ld a,(gfram)        ; get counter.
       rra                 ; time to animate?
       call c,animsp       ; animate sprite.
       ld e,7              ; dull white.
       call cspr           ; set sprite colours.
       ld a,(convd)        ; conveyor direction.
       and a               ; is conveyor off?
       jp nz,misc          ; no, create mischief by switching it off.
       ld b,NUMSPR         ; slots to check.
       ld iy,sprtab        ; sprite table.
       ld de,SPRSIZ        ; size of each entry.
hunt0  ld a,(iy+6)         ; fetch next sprite type.
       cp PLYTYP           ; is it the player?
       jp z,hunt1          ; yes, found him.
       add iy,de           ; look to next slot.
       djnz hunt0          ; repeat.
       ret                 ; can't find player.
hunt1  ld a,(ix+8)         ; x coordinate.
       cp (iy+8)           ; test against player's coordinate.
       jr z,hunth          ; vertical is okay, do horizontal now.
       jr c,huntd          ; move down.
       dec (ix+8)          ; move up.
       jp hunth            ; now do horizontal.
huntd  inc (ix+8)          ; move down.
       jp hunth            ; now do horizontal.
hunth  ld a,(ix+9)         ; sprite's y position.
       cp (iy+9)           ; compare with player's coordinate.
       ret z               ; where we should be.
       jr c,huntr          ; move right.
       dec (ix+9)          ; move left.
       ret
huntr  inc (ix+9)          ; move right.
       ret

; Mischievous sprite heads straight for conveyor button.

misc   ld a,(ix+8)         ; sprite's vertical position.
       cp 64               ; at top of screen?
       jr z,misc0          ; yes, left or right.
       dec (ix+8)          ; move up one pixel.
       ret
misc0  ld a,(ix+9)         ; horizontal position of sprite.
       cp 120              ; on left side of screen?
       jr c,miscl          ; yes, move left.
       cp 214              ; reached right side?
       jp nc,misch         ; yes, switch off the conveyor.
       inc (ix+9)          ; move sprite towards button.
       ld (ix+7),BRRIMG    ; bread right image.
       ret
miscl  cp 26               ; reached left side?
       jp c,misch          ; yes, switch off the conveyor.
       dec (ix+9)          ; move sprite towards button.
       ld (ix+7),BRLIMG    ; bread left image.
       ret

; Find an empty slot in the sprite table.

eslot  ld b,NUMSPR-1       ; slots to check.
eslotn ld iy,sprtab+SPRSIZ ; sprite table.
       ld de,SPRSIZ        ; size of each entry.
eslot0 ld a,(iy+6)         ; fetch next sprite type.
       inc a               ; is it enabled?
       jp z,eslot1         ; no, it's free.
eslot2 add iy,de           ; look to next slot.
       djnz eslot0         ; repeat.
       ret
eslot1 ld a,(iy+0)         ; get current status.
       inc a               ; is it enabled?
       jp nz,eslot2        ; yes, can't use this one.
       inc a               ; set flag state to non-zero when found slot.
       ret

; Player collision detection code.

colp   ld iy,sprtab+SPRSIZ ; sprite table.
       ld de,SPRSIZ        ; size of each entry.
       ld b,NUMSPR-1       ; slots to check.
       ld l,(ix+8)         ; x coord of original.
       ld h,(ix+9)         ; y coordinate.
colp0  ld a,(iy+6)         ; fetch next sprite type.
       cp 255              ; is it enabled?
       jp nz,colp1         ; yes, type determines action.
colp2  add iy,de           ; look to next slot.
       djnz colp0          ; repeat.
       ret
colp1  cp SUPTYP           ; supplies?
       jp z,cols           ; supplies test.
       cp SEDTYP           ; sedative?
       jp z,cols           ; supplies test.
       cp BNCTYP           ; bouncing enemy?
       jp z,colb           ; test for bouncer.
       cp HUNTYP           ; hunter killer enemy?
       jp z,colb           ; test for bouncer.
       cp BOWTYP           ; bouncing and walking enemy?
       jp z,colb           ; test for bouncer.
       cp FRLTYP           ; finger roll enemy?
       jp nz,colp2         ; no, carry on.

; Enemy collision tests.
; Hot dog finger rolls.

colh   ld a,(iy+8)         ; get current position.
       sub l               ; subtract x.
       jr nc,colb0         ; positive result.
       neg                 ; make result positive.
colh0  cp 16               ; within distance?
       jp nc,colp2         ; no, so no collision.
       ld c,a              ; distance in c register.
       ld a,(iy+9)         ; get current position.

       add a,4             ; distance to left edge of sprite.
       sub h               ; subtract y.
       cp 16               ; within range of projectile?
       jr c,colh1          ; yes, we have a collision.
       cp 248              ; check other side.
       jp c,colp2          ; no collision.
       cpl                 ; complement result.
       add a,9             ; range of zero to 8.
colh1  add a,c             ; add distance.
       cp 21               ; outside corner area?
       jp nc,colp2         ; yes, no collision.
       ld (dflag),a        ; set player dead flag.
       ld a,75             ; timer before restart.
       ld (dcount),a       ; set dead timer.
       jp blowup           ; explode.

; Burgers/tomatoes.

colb   ld a,(iy+8)         ; get current position.
       sub l               ; subtract x.
       jr nc,colb0         ; positive result.
       neg                 ; make result positive.
colb0  cp 16               ; within distance?
       jp nc,colp2         ; no, so no collision.
       ld c,a              ; distance in c register.
       ld a,(iy+9)         ; get current position.
       sub h               ; subtract y.
       jr nc,colb1         ; positive result.
       neg                 ; make result positive.
colb1  cp 16               ; within distance?
       jp nc,colp2         ; no, so no collision.
       add a,c             ; add distance.
       cp 25               ; outside corner area?
       jp nc,colp2         ; yes, no collision.
       ld (dflag),a        ; set player dead flag.
       ld a,75             ; timer before restart.
       ld (dcount),a       ; set dead timer.
       jp blowup           ; explode.

cols   ld a,(iy+8)         ; get current position.
       sub l               ; subtract x.
       jr nc,cols0         ; positive result.
       neg                 ; make result positive.
cols0  cp 16               ; within distance?
       jp nc,colp2         ; no, so no collision.
       ld a,(iy+9)         ; get current position.
       sub h               ; subtract y.
       jr nc,cols1         ; positive result.
       neg                 ; make result positive.
cols1  cp 16               ; within distance?
       jp nc,colp2         ; no, so no collision.
       ld a,GETSND         ; collection sound.
       ld (sndtyp),a       ; play sound.
       ld (iy+6),255       ; collect sprite.
       ld a,(iy+7)         ; image type.
       cp CANIMG           ; is it a bonus?
       jr z,cols4          ; yes, increase score.
       cp SEDIMG           ; is it a sedative?
       jp z,sleep          ; yes, sedate customers.
       cp SP2IMG           ; is it a burger?
       jr nz,cols3         ; no, sausage is okay.
       call dburgr         ; show burgers in panel.
       ld b,1              ; ammo type is burger.
       jp cols2            ; add to ammunition.
cols3  call dsausg         ; display new sausage ammo in panel.
       ld b,0              ; default is sausage.
cols2  ld a,6              ; maximum ammunition.
       ld (ammo),a         ; set ammunition.
       ld hl,amotyp        ; type of ammunition.
       ld (hl),b           ; set ammunition type.
sco010 ld hl,score+5       ; tens column.
       ld b,1              ; amount to add.
       jp uscor            ; increase player's score.
cols4  ld hl,sedcnt        ; sedative counter.
       dec (hl)            ; decrement so extra bonus edges closer.
sco200 ld hl,score+4       ; hundreds column.
       ld b,2              ; total to add.
       jp uscor            ; increase the score.
amotyp defb 0              ; sausage or burger.


; Fired projectile collision detection code.

colf   ld iy,sprtab+SPRSIZ ; sprite table.
       ld de,SPRSIZ        ; size of each entry.
       ld b,NUMSPR-1       ; slots to check.
       ld l,(ix+8)         ; x coord of original.
       ld h,(ix+9)         ; y coordinate.
colf0  ld a,(iy+6)         ; fetch next sprite type.
       cp 255              ; is it enabled?
       jp nz,colf1         ; yes, type determines action.
colf2  add iy,de           ; look to next slot.
       djnz colf0          ; repeat.
       ret
colf1  cp BOWTYP           ; bouncing/walking enemy?
       jp z,colf5          ; yes, process that.
       cp FRLTYP           ; hot dog finger roll?
       jp z,colf6          ; yes, process that.
       cp HUNTYP           ; hunter killer?
       jp z,colf5          ; yes, process that.
       cp BNCTYP           ; bouncing enemy?
       jp nz,colf2         ; no, carry on.
colf5  ld a,(iy+8)         ; get current position.
       sub l               ; subtract x.
       jr nc,colf3         ; positive result.
       neg                 ; make result positive.
colf3  cp 16               ; within distance?
       jp nc,colf2         ; no, so no collision.
       ld c,a              ; distance in c register.
       ld a,(iy+9)         ; get current position.
       sub h               ; subtract y.
       jr nc,colf4         ; positive result.
       neg                 ; make result positive.
colf4  cp 16               ; within distance?
       jp nc,colf2         ; no, so no collision.
       add a,c             ; add distance.
       cp 27               ; outside corner area?
       jp nc,colf2         ; yes, no collision.

; Is projectile a burger or a sausage?

       ld a,(ix+7)         ; get projectile sprite image.
       cp PRJIMG           ; is it a sausage?
       jp z,kilnme         ; yes, can't form a burger - destroy enemy.

; Has this burger projectile hit a burger bun?

       ld a,(iy+7)         ; get sprite type.
       cp BBRIMG           ; is it an enemy burger?
       jp z,burfal         ; yes, make it fall.
       cp BBLIMG           ; is it an enemy burger?
       jp z,burfal         ; yes, make sprite fall.

kilnme ld (iy+6),EXPTYP    ; explosion type.
       ld (iy+7),EXPIMG    ; explosion image.
       ld (iy+10),0        ; reset the frame.
       ld hl,score+5       ; score digit.
       ld b,3              ; amount to add.
       call uscor          ; increase player's score.
       jp kilspr           ; kill off the projectile.


; Specific collision test for hot dogs.

colf6  ld a,(iy+8)         ; get current position.
       sub l               ; subtract x.
       jr nc,colf7         ; positive result.
       neg                 ; make result positive.
colf7  cp 16               ; within distance?
       jp nc,colf2         ; no, so no collision.
       ld c,a              ; distance in c register.
       ld a,(iy+9)         ; get current position.
       add a,4             ; distance to left edge of sprite.
       sub h               ; subtract y.
       cp 16               ; within range of projectile?
       jr c,colf8          ; yes, we have a collision.
       cp 248              ; check other side.
       jp c,colf2          ; no collision.
       cpl                 ; complement result.
       add a,9             ; range of zero to 8.
colf8  add a,c             ; add distance.
       cp 23               ; outside corner area?
       jp nc,colf2         ; yes, no collision.

; Is this a sausage hitting the finger roll?

       ld a,(ix+7)         ; get ammunition image.
       cp PRJIMG           ; is it a sausage projectile?
       jp z,dogfal         ; yes, make it fall.

; It was a burger - these can't fill hot dog buns.

       jp kilnme           ; just kill the enemy instead.

burfal ld (iy+7),FBGIMG    ; falling burger image.
       jp mkfal1
dogfal ld (iy+7),DOGIMG    ; falling burger image.
mkfal1 ld (iy+5),0         ; reset the counter.
       ld (iy+6),FBGTYP    ; falling burger sprite.
       ld (iy+10),0        ; reset the frame.
       ld hl,score+5       ; score digit.
       ld b,5              ; amount to add.
       call uscor          ; increase player's score.
       ld a,TRASND         ; transformation sound.
       ld (sndtyp),a       ; play next time.
       jp kilspr           ; kill off the projectile.


; Generate ammunition.

genam  ld a,(gfram)        ; game frame.
       and 15              ; time to decrement counter?
       ret nz              ; no, not yet.
       ld hl,amtim         ; ammunition time delay.
       dec (hl)            ; one less loop.
       ret nz              ; not time to appear yet.
       call eslot          ; find empty slot.
       jp z,genam1         ; no slot available yet.
       call random         ; random time until next appearance.
       and 7               ; range of time delay.
       add a,15            ; minimum time.
       ld (amtim),a        ; set new timer.
       call random         ; random number for position.
       ld b,a              ; copy to b register.
       and 126             ; range of positions.
       add a,60            ; leftmost edge of range.
       ld (iy+6),SUPTYP    ; set supply type.
       ld (iy+8),48        ; just above screen.
       ld (iy+9),a         ; set coordinate.
       ld (iy+10),0        ; reset the frame.
       rr b                ; get low byte of random number.
       jr c,genam0         ; generate burger.
       ld (iy+7),SUPIMG    ; sausage crate image.
       ret
genam0 ld (iy+7),SP2IMG    ; burger crate image.
       ret
genam1 inc (hl)            ; go round for one more iteration.
       ret

; Generate bonus.

genbo  ld a,(gfram)        ; game frame.
       and 63              ; every 64 frames we do the check.
       cp 12               ; time to generate a can?
       ret nz              ; no, not yet.
genbo0 ld hl,botim         ; bonus time delay.
       dec (hl)            ; one less loop.
       ret nz              ; not time to appear yet.
       call eslot          ; find empty slot.
       jp z,genam1         ; no slot available yet.
       call random         ; random time until next appearance.
       and 7               ; range of time delay.
       add a,14            ; minimum time.
       ld (botim),a        ; set new timer.
       call random         ; random number for position.
       and 126             ; range of positions.
       add a,60            ; leftmost edge of range.
       ld (iy+8),49        ; just above screen.
       ld (iy+9),a         ; set coordinate.
       ld (iy+10),0        ; reset the frame.
       ld hl,sedcnt        ; sedative counter.
       ld a,(hl)           ; fetch value.
       and 7               ; 1 in 8.
       jr z,genbo1         ; generate sedative bonus.
       ld (iy+6),SUPTYP    ; set supply type.
       ld (iy+7),CANIMG    ; can image.
       ret
genbo1 ld (iy+6),SEDTYP    ; set supply type.
       ld (iy+7),SEDIMG    ; can image.
       dec (hl)            ; don't generate this next time.
       ret


; Generate spawning sprites.

gensp  ld a,(served)       ; number to serve.
       and a               ; are any left?
       ret z               ; no, don't bother generating any more enemies.
       call random         ; random number.
       ld hl,spodds        ; spawning odds.
       cp (hl)             ; spawning okay?
       ret nc              ; no, don't spawn.
       ld a,(levno)        ; current level.
       rra                 ; divide by 2.
       and 127             ; remove rubbish in bit 7.
       add a,5             ; add 5 as minimum.
       cp MAXSPW           ; reached maximum?
       jr c,gensp0         ; not yet.
       ld a,MAXSPW         ; set to maximum number.
gensp0 ld b,a              ; slots to check.
       call eslotn         ; find empty slot.
       ret z               ; no empty slots, don't spawn.
       ld a,b     
       and a
       ret z
       call random         ; random number.
       and 63              ; in range 0 - 63.
       add a,92            ; minimum extent.
       ld (iy+8),a         ; set x position.
       call random         ; random number.
       and 127             ; in range 0 - 127.
       add a,56            ; minimum extent.
       ld (iy+9),a         ; set x position.
       ld (iy+6),SPWTYP    ; set spawn sprite type.
       ld (iy+7),SPWIMG    ; set sprite image.
       ld (iy+10),0        ; reset frame number.
       ld (iy+5),0         ; reset spawning counter.
       ret


; Add customers to queues.

queue  ld a,(gfram)        ; frame.
       and 31              ; time to add to queue?
       ret nz              ; not this time.
       ld a,(snooze)       ; snooze counter.
       and a               ; is it on?
       ret nz              ; yes, so don't add to queue.
       ld a,(appear)       ; customers still to appear.
       and a               ; have they all joined a queue or been served?
       ret z               ; yes, nothing to do then.
       ld a,(levno)        ; current level.
       ld e,a              ; copy to de.
       ld d,0              ; no high byte.
       ld hl,qodds         ; odds of adding to queue.
       add hl,de           ; point to queue odds.
       ld c,(hl)           ; put chance in c register.
       call random         ; get a random number.
       cp c                ; add to a queue?
       ret nc              ; not this time.
       rra                 ; fifty-fifty chance of either side.
       jr c,queue0         ; go left.
       ld hl,cslotr        ; right customer slots.
       jp queue1
queue0 ld hl,cslotl        ; right customer slots.
queue1 ld b,NQUEUE         ; slots to test.
queue2 ld a,(hl)           ; get customer in slot.
       and a               ; anybody there?
       jr z,queue3         ; no, fill that slot.
       inc hl              ; point to next slot.
       djnz queue2         ; repeat for all slots.
       ret
queue3 push hl             ; store slot address.
       ld a,(levno)        ; current level.
       rlca                ; multiply by 8.
       rlca
       rlca
       ld e,a              ; copy to de.
       ld d,0              ; no high byte.
       call random         ; random displacement.
       ld hl,cuslev        ; customers for each level.
       and 7               ; zero to 7.
       add a,e             ; add to level.
       ld e,a              ; displacement to customer.
       add hl,de           ; point to customer.
       ld a,(hl)           ; get customer.
       pop hl              ; retrieve slot address from stack.
       ld (hl),a           ; put customer in the slot.
       ld hl,appear        ; customers left to appear.
       dec (hl)            ; one less to appear.
       ret

; Customer waiting code.

wcust  ld a,(snooze)       ; snooze counter.
       and a               ; has it been set?
       jp nz,snore         ; yes, decrement counter.
       ld a,(gfram)        ; frame.
       and 127             ; roughly every five seconds.
       cp 8                ; time to process customers?
       ret nz              ; not yet.
       ld hl,cslotl        ; customer slots.
       ld b,SLOTS          ; number of slots.
wcust0 push bc             ; store loop counter on stack.
       ld a,(hl)           ; get customer information.
       and a               ; is slot empty?
       jp z,wcust2         ; yes, skip this slot.
       add a,16            ; increment time spent waiting.
       jp c,wcust1         ; too long - lose a life.
       ld (hl),a           ; set new period.
wcust2 inc hl              ; next customer.
       pop bc              ; retrieve loop counter from stack.
       djnz wcust0         ; repeat for them all.
       ret
wcust1 ld a,b              ; non-zero value in accumulator.
       ld (dflag),a        ; time up.
       jp kilp             ; kill the player.

; Customer colours code.

ccust  ld hl,slotat        ; slot attribute pointers.
       ld de,cslotl        ; customer slots.
       ld b,SLOTS          ; number of slots.
ccust0 push bc             ; store loop counter on stack.
       ld c,7              ; set white on black attributes.
       ld a,(de)           ; get customer information.
       cp 112              ; is slot empty or recently filled?
       jp c,ccust1         ; yes, just want white here.
       ld c,194            ; set flashing red attributes.
       cp 224              ; very angry?
       jr nc,ccust1        ; set attributes.
       ld c,66             ; getting angry - bright red.
       cp 160              ; getting angry?
       jr nc,ccust1        ; set attributes.
       ld c,6              ; getting impatient.
ccust1 ld a,(hl)           ; get next attribute address.
       inc hl              ; second byte.
       push hl             ; store pointer address on stack.
       ld h,(hl)           ; high byte of address.
       ld l,a              ; low byte of attribute address.
       ld (hl),c           ; set colour.
       inc l               ; next cell along.
       ld (hl),c           ; set cell colour.
       ld a,c              ; copy colour to accumulator.
       ld bc,31            ; distance to cell on next line.
       add hl,bc           ; point to third attribute cell.
       ld (hl),a           ; set that cell's colour.
       inc l               ; next cell along.
       ld (hl),a           ; set last cell's colour.
       pop hl              ; retrieve address of attributes pointer.
       inc hl              ; next attribute pointer.
       inc de              ; next customer.
       pop bc              ; retrieve loop counter from stack.
       djnz ccust0         ; repeat for them all.
       jp slh

slotat defw 23136,23040,22944,22848,22752,22656
       defw 23166,23070,22974,22878,22782,22686


; Copy sprites.

cpspr  ld hl,sprtab+6      ; sprite table.
       ld de,sprtab        ; current sprite table.
       ld a,NUMSPR         ; table positions to search.
cpspr0 ldi                 ; copy type across.
       ldi                 ; image.
       ldi                 ; x.
       ldi                 ; y.
       ldi                 ; frame.
       ld bc,6             ; bytes to skip.
       ex de,hl            ; flip source and target.
       add hl,bc           ; add to target.
       ex de,hl            ; flip source and target.
       add hl,bc           ; add to source.
       dec a               ; reduce counter.
       jp nz,cpspr0        ; repeat for all slots.
donowt ret


dscor  defb 33,23,2        ; line 23, column 2.
       ld (dispx),hl       ; set up the coordinates.
       ld hl,score         ; score text.
       call dfont          ; show font.
       jp slh              ; now do slot housekeeping if time permits.

; Moving conveyor routine.

mconv  ld a,(convd)        ; conveyor direction.
       and a               ; is it on?
       jp z,conoff         ; no, it's off.
       ld hl,20642         ; screen address.
       ld b,28             ; width of conveyor.
       dec a               ; moving right?
       jr z,mconr          ; move right.
mconl  ld a,h              ; store top position of conveyor.
       rlc (hl)            ; rotate left.
       ld h,a              ; back to top of conveyor.
       inc l               ; move right.
       djnz mconl          ; repeat for remainder of belt.
mconv0 xor a               ; put zero in accumulator.
       ld (timoff),a       ; reset conveyor downtime.
       ret
mconr  ld a,h              ; store top position of conveyor.
       rrc (hl)            ; rotate right.
       ld h,a              ; back to top of conveyor.
       inc l               ; move right.
       djnz mconr          ; repeat for remainder of belt.
       jp mconv0           ; reset downtime.

; Standard sprite colouring routine.

cspr   ld a,(ix+8)         ; x coordinate.
       cp 48               ; out of range?
;       cp 64               ; out of range?
       ret c               ; yes, ignore it.
       ld b,(ix+9)         ; horizontal.
       ld c,a              ; x coordinate.
       cp 56               ; two cells above screen?
       jr c,csprsg         ; just write a single line.
       cp 64               ; one cell above screen?
       jr c,csprdb         ; just write a single line.
       call ataddp         ; get pixel attribute address.
       ld c,e              ; colour in c register now.
cspr3  ld a,b              ; y coordinate.
       and 7               ; is sprite straddling character cells?
       jp z,cspr2          ; no, just 2 cells wide.

       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld de,30            ; distance to next line down.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld a,(ix+2)         ; x position.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ret
cspr2  ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld de,31            ; distance to next line down.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ld a,(ix+2)         ; x position.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       add hl,de           ; next row.
       ld (hl),c           ; set the colour.
       inc l               ; next cell.
       ld (hl),c           ; set the colour.
       ret
csprsg ld a,c              ; coordinate.
       add a,16            ; look 16 pixels down.
       ld c,a              ; new coordinate.
       call ataddp         ; get pixel attribute address.
       ld (hl),e           ; set the colour.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       ld a,b              ; y coordinate.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       ret
csprdb ld a,c              ; coordinate.
       add a,8             ; look 8 pixels down.
       ld c,a              ; new coordinate.
       call ataddp         ; get pixel attribute address.
       ld (hl),e           ; set the colour.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       ld a,b              ; y coordinate.
       and 7               ; is sprite straddling character cells?
       jr z,csprd0         ; no, skip a cell.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       dec l               ; back one.
csprd0 ld bc,31            ; distance to next attributes row.
       add hl,bc           ; point to attributes address.
       ld (hl),e           ; set the colour.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       ld a,b              ; y coordinate.
       and 7               ; is sprite straddling character cells?
       ret z               ; no, job done.
       inc l               ; next cell.
       ld (hl),e           ; set the colour.
       ret


; Pseudo-random number generator.
; Steps a pointer through the ROM (held in seed), returning the contents
; of the byte at that location.

random ld hl,(seed)        ; pointer to ROM.
       ld a,(seed2)        ; previous seed.
       add a,(hl)          ; combine with number from location.
       ld (seed2),a        ; second seed.
       res 5,h             ; stay within first 8K of ROM.
       xor l               ; more randomness.
       inc hl              ; increment pointer.
       ld (seed),hl        ; new position.
       ret
seed   defw 999            ; random number seed.
seed2  defb 0


; Standard font, 8 x 8 pixels.

font   defb 0,0,0,0,0,0,0,0
       defb 28,34,65,73,129,129,137,118
       defb 56,68,146,130,145,129,130,124
       defb 28,34,66,142,153,129,66,60
       defb 60,66,65,137,153,129,130,124
       defb 60,66,138,134,137,129,130,124
       defb 30,33,65,134,130,156,144,96
       defb 30,33,65,142,145,129,65,62
       defb 54,73,73,129,129,137,137,118
       defb 12,18,28,36,36,68,68,56
       defb 6,9,9,9,113,130,132,120
       defb 48,78,73,129,130,129,153,102
       defb 48,72,136,144,156,130,66,60
       defb 54,73,129,129,129,137,137,118
       defb 54,73,129,129,145,137,137,118
       defb 28,34,65,137,129,129,66,60
       defb 62,65,129,145,130,140,136,112
       defb 62,65,129,137,65,50,18,12
       defb 60,66,129,137,130,129,137,118
       defb 62,33,69,98,145,129,66,60
       defb 62,129,129,70,136,136,136,112
       defb 54,73,137,153,145,129,130,124
       defb 110,153,153,145,129,130,76,48
       defb 110,153,153,145,129,129,146,108
       defb 54,73,65,34,73,137,145,110
       defb 110,145,129,65,34,66,68,56
       defb 124,130,130,100,78,129,129,126

       defb 0,0,0,0,24,36,36,24                ; 27 = .
       defb 0,0,0,0,24,36,20,8                 ; 28 = ,
       defb 108,146,180,72,0,0,0,0             ; 29 = "
       defb 48,72,40,16,0,0,0,0                ; 30 = '
       defb 7,30,55,61,31,39,16,5              ; burger left = 31.

       defb 0,30,33,65,73,66,66,60
       defb 12,18,34,36,68,68,68,56
       defb 120,132,130,98,70,129,129,126
       defb 126,129,145,98,145,129,66,60
       defb 14,49,65,146,129,129,102,24
       defb 62,65,129,134,98,130,132,120
       defb 28,34,66,132,146,130,132,120
       defb 126,129,129,98,68,136,136,112
       defb 30,33,73,65,146,130,132,120
       defb 30,33,65,73,97,130,132,120

       defb 224,248,188,236,124,244,184,224    ; burger right = 42.
       defb 62,64,128,154,128,128,98,28        ; life = 43.
       defb 0,0,26,37,26,0,4,0                 ; extinct life = 44.
       defb 0,112,220,243,124,31,5,0           ; sausage left = 45.
       defb 0,0,0,128,126,197,255,86           ; sausage right = 46.
       defb 0,6,25,97,129,97,25,6              ; backspace = 47.


; Text routines.

; Display an ENDCHR-terminated string at hl.
; Note that this version is for a bespoke 8*5 font, and
; additionally lacks attribute support.

dfont  ld a,(hl)           ; get char from hl.
       inc hl              ; point to next char.
       cp ENDCHR           ; is it a null terminator?
       ret z               ; yes, job done.
       call dfont1         ; display char.
       jp dfont            ; repeat.
dfont1 cp ATPOS            ; is it the AT code?
       jr z,dfont0         ; yes, set x and y coordinates.
       push hl             ; store pointer to string.
       rlca                ; multiply char number by 8.
       rlca
       rlca
       ld e,a              ; store shift in e.
       and 1               ; high bit.
       ld d,a              ; get high byte for this font.
       ld a,e              ; restore shift.
       and 248             ; mask away unwanted bits.
       ld e,a              ; low byte of displacement.
       ld hl,font          ; point to font.
       add hl,de           ; address of char image.
       call chadd          ; get screen address in de.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       inc d               ; next screen line down.
       dec de              ; back one char as ldi moved us on.
       ldi                 ; transfer byte to screen.
       ld hl,dispy         ; y position.
       inc (hl)            ; move to next column.
       pop hl              ; restore string pointer.
       ret
dfont0 ld e,(hl)           ; get first control code.
       inc hl              ; point to next one.
       ld d,(hl)           ; get that code too.
       inc hl              ; point to next valid character.
       ld (dispx),de       ; set coordinates.
       ret                 ; back to main loop.


; Display sprites.

dspr   ld ix,sprtab        ; sprite table.
       ld b,NUMSPR         ; table positions to search.
dspr0  push bc             ; store sprite counter.
       ld a,(ix+0)         ; switched on/off.
       inc a               ; is it on?
       jp nz,dspr1         ; yes, old is on.
       ld a,(ix+6)         ; new sprite.
       inc a               ; is it on?
       jp nz,dnew          ; yes, new is on.

; Neither sprite slot enabled, nothing to do.

dspr2  ld de,SPRSIZ        ; size of sprite table.
       add ix,de           ; point to next slot.
       pop bc              ; restore counter.
       djnz dspr0          ; repeat for all slots.
       ret

; Old sprite is on - what about corresponding position in new table?

dspr1  ld a,(ix+6)         ; new sprite.
       inc a               ; is it on?
       jp nz,dboth         ; yes, both are on.
dold   ld c,(ix+2)         ; old x.
       ld b,(ix+3)         ; old y.
       ld a,(ix+1)         ; old image type.
       ld d,(ix+4)         ; frame number.
       call image          ; get image address.
       call scadd          ; get screen address.
       ld a,18             ; sprite height.
dold0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dold0
       jp dspr2
dnew   ld c,(ix+8)         ; new x.
       ld b,(ix+9)         ; new y.
       ld a,(ix+7)         ; new type.
       ld d,(ix+10)        ; frame number.
dnew1  call image          ; get image address.
       call scadd          ; get screen address.
       ld a,18             ; sprite height.
dnew0  ex af,af'           ; store away.
       call dline          ; draw line of sprite.
       ex af,af'           ; restore loop counter.
       dec a               ; one less iteration.
       jp nz,dnew0
       jp dspr2
dboth  ld c,(ix+2)         ; old x.
       ld b,(ix+3)         ; old y.
       ld a,(ix+1)         ; old image.
       ld d,(ix+4)         ; frame number.
       call image          ; get image address.
       call scadd          ; get screen address.
       exx                 ; old sprite to alternate registers.
       ld c,(ix+8)         ; new x.
       ld b,(ix+9)         ; new y.
       ld a,(ix+7)         ; new image type.
       ld d,(ix+10)        ; frame number.
       call image          ; get image address.
       call scadd          ; get screen address.
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       call dline2
       jp dspr2



; Return character cell address of block at (dispx, dispy).
; This version skips top two lines of character cell.

chadd  ld a,(dispx)        ; vertical position.
       ld e,a              ; store in e.
       and 24              ; which segment, 1 to 3?
       add a,64            ; 64*256 = 16384, Spectrum's screen memory.
       ld d,a              ; this is our high byte.
       ld a,e              ; what was that vertical position again?
       and 7               ; which row within segment?
       rrca                ; multiply row by 32.
       rrca
       rrca
       ld e,a              ; low byte.
       ld a,(dispy)        ; add on y coordinate.
       add a,e             ; mix with low byte.
       ld e,a              ; address of screen position in de.
       ret

; Calculate screen address of (c, b), returning result in hl.
; Only corrupts hl and accumulator.

scadd  ld a,c              ; vertical position.
       and 7               ; get position within char cell.
       add a,56            ; add to screen start address.
       ld h,a              ; place that in high byte.
       ld a,c              ; restore the vertical.
       rrca                ; divide by 8 to find segment.
       rrca
       rrca
       and 24              ; 0, 8 and 16 for segment displacement.
       add a,h             ; add to high byte.
       ld h,a              ; store in high byte.
       ld a,c              ; restore vertical again.
       rlca                ; multiply by 4.
       rlca
       and 224             ; multiples of 32 wanted.
       ld l,a              ; put in low byte.
       ld a,b              ; fetch horizontal position.
       rrca
       rrca
       rrca
       and 31
       add a,l
       ld l,a
       ret                 ; hl = address of screen.


; Calculate address of attribute for pixel at (c, b).

ataddp ld a,c              ; look at the vertical first.
       rlca                ; divide by 64.
       rlca                ; quicker than 6 rrca operations.
       ld l,a              ; store in e register for now.
       and 3               ; mask to find segment.
       add a,87            ; attributes start at 88*256=22528.
       ld h,a              ; that's our high byte sorted.
       ld a,l              ; vertical/64 - same as vertical*4.
       and 224             ; want a multiple of 32.
       ld l,a              ; vertical element calculated.
       ld a,b              ; get horizontal position.
       rra                 ; divide by 8.
       rra
       rra
       and 31              ; want result in range 0-31.
       add a,l             ; add to existing low byte.
       ld l,a              ; that's the low byte done.
       ret                 ; attribute address now in de.

dline  ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc de              ; next byte of source image.
       inc hl              ; next screen byte.
       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc hl              ; next screen byte.
       inc de              ; next byte of source image.
       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc de              ; next byte of source image.
       dec hl              ; next screen byte.
       dec hl              ; next screen byte.

; Line drawn, now work out next target address.

nline  inc h               ; increment pixel.
       ld a,h              ; get pixel address.
       and 7               ; straddling character position?
       ret nz              ; no, we're on next line already.
       ld a,h              ; get pixel address.
       sub 8               ; subtract 8 for start of segment.
       ld h,a              ; new high byte of address.
       ld a,l              ; get low byte of address.
       add a,32            ; one line down.
       ld l,a              ; new low byte.
       ret nc              ; not reached next segment yet.
       ld a,h              ; address high.
       add a,8             ; add 8 to next segment.
       ld h,a              ; new high byte.
       ret

dline2 ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       exx
       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc l               ; next screen byte.
       inc de              ; next byte of source image.
       exx
       inc de              ; next byte of source image.
       inc l               ; next screen byte.

       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       exx
       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc l               ; next screen byte.
       inc de              ; next byte of source image.
       exx
       inc l               ; next screen byte.
       inc de              ; next byte of source image.

       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       exx
       ld a,(de)           ; byte of sprite.
       xor (hl)            ; merge with screen.
       ld (hl),a           ; write to screen.
       inc de              ; next byte of source image.
       dec l               ; next screen byte.
       dec l               ; next screen byte.
       call nline
       exx
       inc de              ; next byte of source image.
       dec l               ; next screen byte.
       dec l               ; next screen byte.
       jp nline

dispx  defb 0
dispy  defb 0


; Animate sprite.

animsp ld a,(ix+7)         ; get sprite number.
       call gfrm           ; get frame image number.
       ld a,(ix+10)        ; old frame.
       inc a               ; next frame.
       cp (hl)             ; reached the maximum frame?
       jr c,anims0         ; not yet.
       xor a               ; restart at first frame.
anims0 ld (ix+10),a        ; write new frame.
       ret


; Get frame image number from sprite.
; On entry: a = sprite number.
; On exit:  a = sprite image number.
;          hl = ptr to number of frames.

gfrm   rlca                ; double sprite number.
       ld e,a              ; store in e.
       ld d,0              ; no high byte.
       ld hl,frmlst        ; frame list.
       add hl,de           ; point to frame.
       ld a,(hl)           ; frame image number.
       inc hl              ; point to frames.
       ret

; Get address of image.
; On entry: a = sprite image number.
;           b = position straddling cells.
; On exit: de = address.

image  push de             ; store frame.
       call gfrm           ; get frame image number for sprite.
       pop de              ; retrieve frame number.
       add a,d             ; add frame number.
       rlca                ; double sprite number.
       rlca                ; quadruple image number = shift positions.
       rlca                ; 8 times image number for first of the four.
       ld e,a              ; store in e.
       and 7               ; high byte mask.
       ld d,a              ; displacement high byte.
       ld a,e              ; restore shift result.
       and 248             ; low byte mask.
       ld e,a              ; store low byte of de.
       ld a,b              ; position straddling cells on screen.
       and 6               ; 0, 2, 4 or 6.
       add a,e             ; add to displacement.
       ld e,a              ; store low byte of de.
       ld hl,frmdat        ; pointer to frames list.
       add hl,de           ; point to number.
       ld e,(hl)           ; address low.
       inc hl              ; second byte.
       ld d,(hl)           ; address high.
       ret

dslot  ld hl,cslotl        ; slots.
       ld e,a              ; displacement in de.
       ld d,0              ; no high byte.
       add hl,de           ; point to slot.
       ld a,(hl)           ; fetch slot image type.
       and a               ; is it on?
       call nz,tslot       ; test slot for sleepiness.
       rrca                ; multiply image by 32.
       rrca
       rrca
       ld c,a              ; copy to c.
       and 1               ; high byte bitmask.
       ld b,a              ; high byte of displacement.
       ld a,c              ; restore shift result.
       and 224             ; mask away what we don't want.
       ld c,a              ; high byte of bc.

       ld hl,cusimg        ; customer images.
       add hl,bc           ; point to image data for slot.
       ex de,hl            ; flip into de.
       rl l                ; double slot number (flipped e to l above).
       ld h,0              ; no high byte.
       ld bc,csal          ; address of screen.
       add hl,bc           ; point to address.
       ld a,(hl)           ; get address low in accumulator.
       inc hl              ; point to next byte.
       ld h,(hl)           ; get high byte in h.
       ld l,a              ; put screen address low in l.

dslot1 ld b,16             ; lines to write.
dslot0 ld a,(de)           ; fetch image data.
       inc de              ; next image byte.
       ld (hl),a           ; copy to screen.
       inc l               ; next screen address.
       ld a,(de)           ; fetch image data.
       inc de              ; next image byte.
       ld (hl),a           ; copy to screen.
       dec l               ; back to start of line.
       call nline          ; find next line.
       djnz dslot0         ; repeat for rest of slot.
       ret

tslot  ld a,(snooze)       ; snooze counter.
       and a               ; is it on?
       ld a,(hl)           ; default to customer in slot.
       ret z               ; snooze mode not on.
       ld a,9              ; sleepy face.
       ret


; Customer slots.

cslotl defb 7,1,6,3,5,0
cslotr defb 8,4,2,1,0,0

; Screen addresses of each slot.

csal   defw 20576
       defw 20480
       defw 18592
       defw 18496
       defw 16608
       defw 16512
;       defw 16416

csar   defw 20606
       defw 20510
       defw 18622
       defw 18526
       defw 16638
       defw 16542
;       defw 16446


; Customer images.

cusimg defw 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0    ; empty slot.

       defb 42,89,28,210,87,164,39,31,55,253,79,249,79,253,95,115
       defb 248,243,255,247,135,255,184,255,191,31,183,231,94,247,95,239
       defb 255,255,253,247,255,254,251,247,231,207,94,158,219,244,191,255
       defb 160,80,252,114,176,248,255,166,255,254,127,198,191,236,79,248
       defb 195,255,128,255,84,111,250,63,252,55,253,159,253,154,253,76
       defb 254,13,95,22,238,252,63,90,95,184,47,120,55,188,27,124
       defb 250,248,255,148,247,19,220,80,244,165,163,204,15,56,205,252
       defb 143,248,143,248,223,93,255,255,255,255,254,31,255,62,127,252
       defb 199,249,159,252,127,255,207,253,167,250,134,232,207,157,255,255
       defb 80,2,255,255,111,123,6,48,0,1,144,9,195,243,120,14
       defb 127,254,223,255,255,179,238,229,255,61,253,94,253,143,221,207
       defb 245,255,211,255,215,255,103,127,107,159,145,254,236,120,255,129
       defb 0,12,0,30,0,26,0,12,48,0,120,0,104,2,48,4
       defb 0,28,0,248,47,224,5,4,0,8,1,96,0,0,0,0

; Storm Trooper.

       defb 63,143,252,227,236,56,236,14,230,7,227,252,247,252,127,254
       defb 255,254,184,12,223,4,255,244,255,252,255,248,243,184,101,56

; Sleeping customer.

sleepy defb 127,254,255,255,199,227,191,253,255,255,255,255,255,255,255,255
       defb 187,221,199,227,255,255,255,255,239,255,243,207,252,63,127,254

; Switch pressed and unpressed.

switp  defb 0,0,0,0,0,0,7,224,29,88,63,165,159,212,47,165
       defb 159,74,74,133,160,11,85,95,42,254,21,244,2,160,0,0
switu  defb 0,0,7,224,29,88,63,164,31,212,47,165,159,76,42,157
       defb 31,250,7,229,0,11,0,95,2,254,5,244,2,160,0,0



; Start, and number of frames per sprite.

frmlst defb 0,2            ; player right sprite, frames.
       defb 2,2            ; player left sprite.
       defb 4,2            ; ammunition.
       defb 6,1            ; ammunition.
       defb 7,5            ; spawning.
       defb 12,4           ; cob, right.
       defb 16,1           ; crate of sausages.
       defb 17,6           ; explosion.
       defb 23,4           ; tomato, left.
       defb 27,1           ; completed hot dog.
       defb 28,1           ; completed burger.
       defb 29,1           ; crate of burgers.
       defb 30,1           ; soft drink can.
       defb 31,4           ; right-facing bap.
       defb 35,4           ; left-facing finger roll.
       defb 39,4           ; right-facing finger roll.
       defb 43,4           ; right-facing tomato.
       defb 47,1           ; Tango can.
       defb 48,4           ; bread left.
       defb 52,4           ; bread right.

frmdat defw rob0ra,rob0rb,rob0rc,rob0rd    ; robot 0 right.
       defw rob1ra,rob1rb,rob1rc,rob1rd    ; robot 1 right.
       defw rob0la,rob0lb,rob0lc,rob0ld    ; robot 0 left.
       defw rob1la,rob1lb,rob1lc,rob1ld    ; robot 1 left.
       defw ammo0a,ammo0b,ammo0c,ammo0d    ; fired sausage.
       defw ammo1a,ammo1b,ammo1c,ammo1d    ; fired sausage.
       defw ammo2a,ammo2b,ammo2c,ammo2d    ; fired burger.
       defw spaw0a,spaw0b,spaw0c,spaw0d    ; spawning.
       defw spaw1a,spaw1b,spaw1c,spaw1d
       defw spaw2a,spaw2b,spaw2c,spaw2d
       defw spaw3a,spaw3b,spaw3c,spaw3d
       defw spaw4a,spaw4b,spaw4c,spaw4d
       defw nme0ra,nme0rb,nme0rc,nme0rd    ; mutant cob, left.
       defw nme1ra,nme1rb,nme1rc,nme1rd    ; mutant cob, left.
       defw nme2ra,nme2rb,nme2rc,nme2rd    ; mutant cob, left.
       defw nme3ra,nme3rb,nme3rc,nme3rd    ; mutant cob, left.
       defw crate0,crate1,crate2,crate3    ; sausage crates.
       defw expl0a,expl0b,expl0c,expl0d    ; explosions.
       defw expl1a,expl1b,expl1c,expl1d
       defw expl2a,expl2b,expl2c,expl2d
       defw expl3a,expl3b,expl3c,expl3d
       defw expl4a,expl4b,expl4c,expl4d
       defw expl5a,expl5b,expl5c,expl5d
       defw toml0a,toml0b,toml0c,toml0d    ; Tomato, left.
       defw toml1a,toml1b,toml1c,toml1d
       defw toml2a,toml2b,toml2c,toml2d
       defw toml3a,toml3b,toml3c,toml3d
       defw hdog0,hdog1,hdog2,hdog3        ; hot dog, complete.
       defw cburg0,cburg1,cburg2,cburg3    ; completed burger.
       defw crate4,crate5,crate6,crate7    ; burger crate.
       defw coke0,coke1,coke2,coke3        ; soft drink.
       defw nme0la,nme0lb,nme0lc,nme0ld    ; mutant cob, right.
       defw nme1la,nme1lb,nme1lc,nme1ld    ; mutant cob, right.
       defw nme2la,nme2lb,nme2lc,nme2ld    ; mutant cob, right.
       defw nme3la,nme3lb,nme3lc,nme3ld    ; mutant cob, right.
       defw finl0a,finl0b,finl0c,finl0d    ; finger roll, left.
       defw finl1a,finl1b,finl1c,finl1d
       defw finl2a,finl2b,finl2c,finl2d
       defw finl3a,finl3b,finl3c,finl3d
       defw finr0a,finr0b,finr0c,finr0d    ; finger roll, left.
       defw finr1a,finr1b,finr1c,finr1d
       defw finr2a,finr2b,finr2c,finr2d
       defw finr3a,finr3b,finr3c,finr3d
       defw tomr0a,tomr0b,tomr0c,tomr0d    ; Tomato, right.
       defw tomr1a,tomr1b,tomr1c,tomr1d
       defw tomr2a,tomr2b,tomr2c,tomr2d
       defw tomr3a,tomr3b,tomr3c,tomr3d
       defw tango0,tango1,tango2,tango3    ; second soft drink.
       defw brel0a,brel0b,brel0c,brel0d    ; bread, left.
       defw brel1a,brel1b,brel1c,brel1d
       defw brel2a,brel2b,brel2c,brel2d
       defw brel3a,brel3b,brel3c,brel3d
       defw brer0a,brer0b,brer0c,brer0d    ; bread, right.
       defw brer1a,brer1b,brer1c,brer1d
       defw brer2a,brer2b,brer2c,brer2d
       defw brer3a,brer3b,brer3c,brer3d


; Sprite graphics start here.

rob0ra defb 0,0,0,64,224,0,17,176,0,11,216,0
       defb 11,252,0,23,252,0,43,130,0,59,0,0
       defb 27,52,0,7,0,0,15,0,0,0,196,0
       defb 31,58,0,35,151,0,73,195,128,29,203,128
       defb 65,195,128,43,151,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0rb defb 0,0,0,16,56,0,4,108,0,2,246,0
       defb 2,255,0,5,255,0,10,224,128,14,192,0
       defb 6,205,0,1,192,0,3,192,0,0,49,0
       defb 7,206,128,8,229,192,18,112,224,7,114,224
       defb 16,112,224,10,229,192,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0rc defb 0,0,0,4,14,0,1,27,0,0,189,128
       defb 0,191,192,1,127,192,2,184,32,3,176,0
       defb 1,179,64,0,112,0,0,240,0,0,12,64
       defb 1,243,160,2,57,112,4,156,56,1,220,184
       defb 4,28,56,2,185,112,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0rd defb 0,0,0,1,3,128,0,70,192,0,47,96
       defb 0,47,240,0,95,240,0,174,8,0,236,0
       defb 0,108,208,0,28,0,0,60,0,0,3,16
       defb 0,124,232,0,142,92,1,39,14,0,119,46
       defb 1,7,14,0,174,92,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

rob1ra defb 0,0,0,64,224,0,17,176,0,11,216,0
       defb 11,252,0,23,252,0,43,130,0,59,0,0
       defb 27,52,0,7,0,0,15,0,0,0,196,0
       defb 31,58,0,35,151,0,9,195,128,93,203,128
       defb 1,195,128,87,175,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1rb defb 0,0,0,16,56,0,4,108,0,2,246,0
       defb 2,255,0,5,255,0,10,224,128,14,192,0
       defb 6,205,0,1,192,0,3,192,0,0,49,0
       defb 7,206,128,8,229,192,2,112,224,23,114,224
       defb 0,112,224,21,235,192,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1rc defb 0,0,0,4,14,0,1,27,0,0,189,128
       defb 0,191,192,1,127,192,2,184,32,3,176,0
       defb 1,179,64,0,112,0,0,240,0,0,12,64
       defb 1,243,160,2,57,112,0,156,56,5,220,184
       defb 0,28,56,5,122,240,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1rd defb 0,0,0,1,3,128,0,70,192,0,47,96
       defb 0,47,240,0,95,240,0,174,8,0,236,0
       defb 0,108,208,0,28,0,0,60,0,0,3,16
       defb 0,124,232,0,142,92,0,39,14,1,119,46
       defb 0,7,14,1,94,188,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

rob0la defb 0,0,0,1,192,128,3,98,0,6,244,0
       defb 15,244,0,15,250,0,16,117,0,0,55,0
       defb 11,54,0,0,56,0,0,60,0,8,192,0
       defb 23,62,0,58,113,0,112,228,128,116,238,0
       defb 112,224,128,58,117,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0lb defb 0,0,0,0,112,32,0,216,128,1,189,0
       defb 3,253,0,3,254,128,4,29,64,0,13,192
       defb 2,205,128,0,14,0,0,15,0,2,48,0
       defb 5,207,128,14,156,64,28,57,32,29,59,128
       defb 28,56,32,14,157,64,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0lc defb 0,0,0,0,28,8,0,54,32,0,111,64
       defb 0,255,64,0,255,160,1,7,80,0,3,112
       defb 0,179,96,0,3,128,0,3,192,0,140,0
       defb 1,115,224,3,167,16,7,14,72,7,78,224
       defb 7,14,8,3,167,80,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob0ld defb 0,0,0,0,7,2,0,13,136,0,27,208
       defb 0,63,208,0,63,232,0,65,212,0,0,220
       defb 0,44,216,0,0,224,0,0,240,0,35,0
       defb 0,92,248,0,233,196,1,195,146,1,211,184
       defb 1,195,130,0,233,212,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

rob1la defb 0,0,0,1,192,128,3,98,0,6,244,0
       defb 15,244,0,15,250,0,16,117,0,0,55,0
       defb 11,54,0,0,56,0,0,60,0,8,192,0
       defb 23,62,0,58,113,0,112,228,0,116,238,128
       defb 112,224,0,61,122,128,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1lb defb 0,0,0,0,112,32,0,216,128,1,189,0
       defb 3,253,0,3,254,128,4,29,64,0,13,192
       defb 2,205,128,0,14,0,0,15,0,2,48,0
       defb 5,207,128,14,156,64,28,57,0,29,59,160
       defb 28,56,0,15,94,160,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1lc defb 0,0,0,0,28,8,0,54,32,0,111,64
       defb 0,255,64,0,255,160,1,7,80,0,3,112
       defb 0,179,96,0,3,128,0,3,192,0,140,0
       defb 1,115,224,3,167,16,7,14,64,7,78,232
       defb 7,14,0,3,215,168,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
rob1ld defb 0,0,0,0,7,2,0,13,136,0,27,208
       defb 0,63,208,0,63,232,0,65,212,0,0,220
       defb 0,44,216,0,0,224,0,0,240,0,35,0
       defb 0,92,248,0,233,196,1,195,144,1,211,186
       defb 1,195,128,0,245,234,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

; Fired sausage, 2 frames.

ammo0a defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,112,0,0,220,0,0,243,128,0
       defb 124,126,0,31,197,0,5,255,0,0,86,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo0b defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,28,0,0,55,0,0,60,224,0
       defb 31,31,128,7,241,64,1,127,192,0,21,128
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo0c defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,7,0,0,13,192,0,15,56,0
       defb 7,199,224,1,252,80,0,95,240,0,5,96
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo0d defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,1,192,0,3,112,0,3,206,0
       defb 1,241,248,0,127,20,0,23,252,0,1,88
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo1a defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,7,0,0,29,128,0,231,128
       defb 63,31,0,81,252,0,127,208,0,53,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo1b defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,1,192,0,7,96,0,57,224
       defb 15,199,192,20,127,0,31,244,0,13,64,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo1c defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,112,0,1,216,0,14,120
       defb 3,241,240,5,31,192,7,253,0,3,80,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
ammo1d defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,28,0,0,118,0,3,158
       defb 0,252,124,1,71,240,1,255,64,0,212,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

; fired burger, 1 frame.

ammo2a defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,240,0,15,124,0,27,222,0
       defb 30,246,0,15,190,0,19,250,0,8,92,0,2,240,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
ammo2b defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,0,3,223,0,6,247,128
       defb 7,189,128,3,239,128,4,254,128,2,23,0,0,188,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
ammo2c defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,63,0,0,247,192,1,189,224
       defb 1,239,96,0,251,224,1,63,160,0,133,192,0,47,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
ammo2d defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,192,0,61,240,0,111,120
       defb 0,123,216,0,62,248,0,79,232,0,33,112,0,11,192,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

spaw0a defb 0,0,0,2,68,0,64,31,0,9,14,0,0,91,0,0,0,0,4,128,0,14,33,0
       defb 4,136,0,10,0,0,0,0,0,2,64,0,32,224,0,4,64,0,0,0,0,64,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw0b defb 0,0,0,0,145,0,16,7,192,2,67,128,0,22,192,0,0,0,1,32,0,3,136,64
       defb 1,34,0,2,128,0,0,0,0,0,144,0,8,56,0,1,16,0,0,0,0,16,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw0c defb 0,0,0,0,36,64,4,1,240,0,144,224,0,5,176,0,0,0,0,72,0,0,226,16
       defb 0,72,128,0,160,0,0,0,0,0,36,0,2,14,0,0,68,0,0,0,0,4,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw0d defb 0,0,0,0,9,16,1,0,124,0,36,56,0,1,108,0,0,0,0,18,0,0,56,132
       defb 0,18,32,0,40,0,0,0,0,0,9,0,0,131,128,0,17,0,0,0,0,1,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

spaw1a defb 0,0,0,0,5,0,0,65,128,0,10,128,0,1,128,0,0,0,40,0,0,36,0,0
       defb 32,128,0,52,0,0,0,0,0,0,18,0,2,70,0,0,10,0,0,0,0,8,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw1b defb 0,0,0,0,1,64,0,16,96,0,2,160,0,0,96,0,0,0,10,0,0,9,0,0
       defb 8,32,0,13,0,0,0,0,0,0,4,128,0,145,128,0,2,128,0,0,0,2,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw1c defb 0,0,0,0,0,80,0,4,24,0,0,168,0,0,24,0,0,0,2,128,0,2,64,0
       defb 2,8,0,3,64,0,0,0,0,0,1,32,0,36,96,0,0,160,0,0,0,0,128,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw1d defb 0,0,0,0,0,20,0,1,6,0,0,42,0,0,6,0,0,0,0,160,0,0,144,0
       defb 0,130,0,0,208,0,0,0,0,0,0,72,0,9,24,0,0,40,0,0,0,0,32,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

spaw2a defb 0,0,0,0,10,0,0,60,0,0,29,0,0,52,0,0,0,0,20,0,0,14,0,0
       defb 36,0,0,10,0,0,0,0,0,0,40,0,0,116,128,0,32,0,0,0,0,1,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw2b defb 0,0,0,0,2,128,0,15,0,0,7,64,0,13,0,0,0,0,5,0,0,3,128,0
       defb 9,0,0,2,128,0,0,0,0,0,10,0,0,29,32,0,8,0,0,0,0,0,64,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw2c defb 0,0,0,0,0,160,0,3,192,0,1,208,0,3,64,0,0,0,1,64,0,0,224,0
       defb 2,64,0,0,160,0,0,0,0,0,2,128,0,7,72,0,2,0,0,0,0,0,16,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw2d defb 0,0,0,0,0,40,0,0,240,0,0,116,0,0,208,0,0,0,0,80,0,0,56,0
       defb 0,144,0,0,40,0,0,0,0,0,0,160,0,1,210,0,0,128,0,0,0,0,4,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

spaw3a defb 0,0,0,4,128,0,31,18,0,14,64,0,27,16,0,0,0,0,0,72,0,8,28,0
       defb 1,40,0,0,20,0,0,0,0,9,0,0,28,68,0,10,0,0,0,0,0,0,8,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw3b defb 0,0,0,1,32,0,7,196,128,3,144,0,6,196,0,0,0,0,0,18,0,2,7,0
       defb 0,74,0,0,5,0,0,0,0,2,64,0,7,17,0,2,128,0,0,0,0,0,2,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw3c defb 0,0,0,0,72,0,1,241,32,0,228,0,1,177,0,0,0,0,0,4,128,0,129,192
       defb 0,18,128,0,1,64,0,0,0,0,144,0,1,196,64,0,160,0,0,0,0,0,0,128
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw3d defb 0,0,0,0,18,0,0,124,72,0,57,0,0,108,64,0,0,0,0,1,32,0,32,112
       defb 0,4,160,0,0,80,0,0,0,0,36,0,0,113,16,0,40,0,0,0,0,0,0,32
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

spaw4a defb 0,0,0,18,0,0,15,128,0,38,0,0,11,0,0,0,0,0,0,160,0,1,196,0
       defb 0,145,0,1,64,0,0,0,0,20,0,0,78,0,0,4,0,0,0,0,0,0,64,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw4b defb 0,0,0,4,128,0,3,224,0,9,128,0,2,192,0,0,0,0,0,40,0,0,113,0
       defb 0,36,64,0,80,0,0,0,0,5,0,0,19,128,0,1,0,0,0,0,0,0,16,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw4c defb 0,0,0,1,32,0,0,248,0,2,96,0,0,176,0,0,0,0,0,10,0,0,28,64
       defb 0,9,16,0,20,0,0,0,0,1,64,0,4,224,0,0,64,0,0,0,0,0,4,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
spaw4d defb 0,0,0,0,72,0,0,62,0,0,152,0,0,44,0,0,0,0,0,2,128,0,7,16
       defb 0,2,68,0,5,0,0,0,0,0,80,0,1,56,0,0,16,0,0,0,0,0,1,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


; Enemy burger cob, left.

nme0ra defb 0,0,0,0,0,0,0,0,0,3,240,0
       defb 14,220,0,27,250,0,46,191,0,63,247,0
       defb 92,221,128,105,255,128,105,255,128,63,255,0
       defb 71,248,128,104,5,128,58,175,0,31,252,0
       defb 3,224,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme0rb defb 0,0,0,0,0,0,0,0,0,0,252,0
       defb 3,183,0,6,254,128,11,175,192,15,253,192
       defb 23,55,96,26,127,224,26,127,224,15,255,192
       defb 17,254,32,26,1,96,14,171,192,7,255,0
       defb 0,248,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme0rc defb 0,0,0,0,0,0,0,0,0,0,63,0
       defb 0,237,192,1,191,160,2,235,240,3,255,112
       defb 5,205,216,6,159,248,6,159,248,3,255,240
       defb 4,127,136,6,128,88,3,170,240,1,255,192
       defb 0,62,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme0rd defb 0,0,0,0,0,0,0,0,0,0,15,192
       defb 0,59,112,0,111,232,0,186,252,0,255,220
       defb 1,115,118,1,167,254,1,167,254,0,255,252
       defb 1,31,226,1,160,22,0,234,188,0,127,240
       defb 0,15,128,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme1ra defb 3,240,0,14,220,0,27,250,0,46,191,0
       defb 63,247,0,92,221,128,105,255,128,105,255,128
       defb 63,255,0,7,248,0,32,1,0,64,0,128
       defb 104,5,128,58,175,0,31,252,0,3,224,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme1rb defb 0,252,0,3,183,0,6,254,128,11,175,192
       defb 15,253,192,23,55,96,26,127,224,26,127,224
       defb 15,255,192,1,254,0,8,0,64,16,0,32
       defb 26,1,96,14,171,192,7,255,0,0,248,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme1rc defb 0,63,0,0,237,192,1,191,160,2,235,240
       defb 3,255,112,5,205,216,6,159,248,6,159,248
       defb 3,255,240,0,127,128,2,0,16,4,0,8
       defb 6,128,88,3,170,240,1,255,192,0,62,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme1rd defb 0,15,192,0,59,112,0,111,232,0,186,252
       defb 0,255,220,1,115,118,1,167,254,1,167,254
       defb 0,255,252,0,31,224,0,128,4,1,0,2
       defb 1,160,22,0,234,188,0,127,240,0,15,128
       defb 0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme2ra defb 0,0,0,3,240,0,14,220,0,27,250,0
       defb 46,191,0,63,247,0,92,221,128,105,255,128
       defb 105,255,128,63,255,0,7,248,0,32,1,0
       defb 64,0,128,104,5,128,58,175,0,31,252,0
       defb 3,224,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme2rb defb 0,0,0,0,252,0,3,183,0,6,254,128
       defb 11,175,192,15,253,192,23,55,96,26,127,224
       defb 26,127,224,15,255,192,1,254,0,8,0,64
       defb 16,0,32,26,1,96,14,171,192,7,255,0
       defb 0,248,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme2rc defb 0,0,0,0,63,0,0,237,192,1,191,160
       defb 2,235,240,3,255,112,5,205,216,6,159,248
       defb 6,159,248,3,255,240,0,127,128,2,0,16
       defb 4,0,8,6,128,88,3,170,240,1,255,192
       defb 0,62,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme2rd defb 0,0,0,0,15,192,0,59,112,0,111,232
       defb 0,186,252,0,255,220,1,115,118,1,167,254
       defb 1,167,254,0,255,252,0,31,224,0,128,4
       defb 1,0,2,1,160,22,0,234,188,0,127,240
       defb 0,15,128,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme3ra defb 0,0,0,0,0,0,3,240,0,14,220,0
       defb 27,250,0,46,191,0,63,247,0,92,221,128
       defb 105,255,128,105,255,128,63,255,0,7,248,0
       defb 64,0,128,104,5,128,58,175,0,31,252,0
       defb 3,224,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme3rb defb 0,0,0,0,0,0,0,252,0,3,183,0
       defb 6,254,128,11,175,192,15,253,192,23,55,96
       defb 26,127,224,26,127,224,15,255,192,1,254,0
       defb 16,0,32,26,1,96,14,171,192,7,255,0
       defb 0,248,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme3rc defb 0,0,0,0,0,0,0,63,0,0,237,192
       defb 1,191,160,2,235,240,3,255,112,5,205,216
       defb 6,159,248,6,159,248,3,255,240,0,127,128
       defb 4,0,8,6,128,88,3,170,240,1,255,192
       defb 0,62,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0
nme3rd defb 0,0,0,0,0,0,0,15,192,0,59,112
       defb 0,111,232,0,186,252,0,255,220,1,115,118
       defb 1,167,254,1,167,254,0,255,252,0,31,224
       defb 1,0,2,1,160,22,0,234,188,0,127,240
       defb 0,15,128,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0

; Enemy burger cob, right.

nme0lb defb 0,0,0,0,0,0,0,0,0,0,252,0,3,183,0,5,253,128,15,215,64,14,255,192
       defb 27,179,160,31,249,96,31,249,96,15,255,192,17,254,32,26,1,96,15,85,192,3,255,128
       defb 0,124,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme0lc defb 0,0,0,0,0,0,0,0,0,0,63,0,0,237,192,1,127,96,3,245,208,3,191,240
       defb 6,236,232,7,254,88,7,254,88,3,255,240,4,127,136,6,128,88,3,213,112,0,255,224
       defb 0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme0ld defb 0,0,0,0,0,0,0,0,0,0,15,192,0,59,112,0,95,216,0,253,116,0,239,252
       defb 1,187,58,1,255,150,1,255,150,0,255,252,1,31,226,1,160,22,0,245,92,0,63,248
       defb 0,7,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme0la defb 0,0,0,0,0,0,0,0,0,3,240,0,14,220,0,23,246,0,63,93,0,59,255,0
       defb 110,206,128,127,229,128,127,229,128,63,255,0,71,248,128,104,5,128,61,87,0,15,254,0
       defb 1,240,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

nme1la defb 3,240,0,14,220,0,23,246,0,63,93,0,59,255,0,110,206,128,127,229,128,127,229,128
       defb 63,255,0,7,248,0,32,1,0,64,0,128,104,5,128,61,87,0,15,254,0,1,240,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme1lb defb 0,252,0,3,183,0,5,253,128,15,215,64,14,255,192,27,179,160,31,249,96,31,249,96
       defb 15,255,192,1,254,0,8,0,64,16,0,32,26,1,96,15,85,192,3,255,128,0,124,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme1lc defb 0,63,0,0,237,192,1,127,96,3,245,208,3,191,240,6,236,232,7,254,88,7,254,88
       defb 3,255,240,0,127,128,2,0,16,4,0,8,6,128,88,3,213,112,0,255,224,0,31,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme1ld defb 0,15,192,0,59,112,0,95,216,0,253,116,0,239,252,1,187,58,1,255,150,1,255,150
       defb 0,255,252,0,31,224,0,128,4,1,0,2,1,160,22,0,245,92,0,63,248,0,7,192
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

nme2la defb 0,0,0,3,240,0,14,220,0,23,246,0,63,93,0,59,255,0,110,206,128,127,229,128
       defb 127,229,128,63,255,0,7,248,0,32,1,0,64,0,128,104,5,128,61,87,0,15,254,0
       defb 1,240,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme2lb defb 0,0,0,0,252,0,3,183,0,5,253,128,15,215,64,14,255,192,27,179,160,31,249,96
       defb 31,249,96,15,255,192,1,254,0,8,0,64,16,0,32,26,1,96,15,85,192,3,255,128
       defb 0,124,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme2lc defb 0,0,0,0,63,0,0,237,192,1,127,96,3,245,208,3,191,240,6,236,232,7,254,88
       defb 7,254,88,3,255,240,0,127,128,2,0,16,4,0,8,6,128,88,3,213,112,0,255,224
       defb 0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme2ld defb 0,0,0,0,15,192,0,59,112,0,95,216,0,253,116,0,239,252,1,187,58,1,255,150
       defb 1,255,150,0,255,252,0,31,224,0,128,4,1,0,2,1,160,22,0,245,92,0,63,248
       defb 0,7,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

nme3la defb 0,0,0,0,0,0,3,240,0,14,220,0,23,246,0,63,93,0,59,255,0,110,206,128
       defb 127,229,128,127,229,128,63,255,0,7,248,0,64,0,128,104,5,128,61,87,0,15,254,0
       defb 1,240,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme3lb defb 0,0,0,0,0,0,0,252,0,3,183,0,5,253,128,15,215,64,14,255,192,27,179,160
       defb 31,249,96,31,249,96,15,255,192,1,254,0,16,0,32,26,1,96,15,85,192,3,255,128
       defb 0,124,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme3lc defb 0,0,0,0,0,0,0,63,0,0,237,192,1,127,96,3,245,208,3,191,240,6,236,232
       defb 7,254,88,7,254,88,3,255,240,0,127,128,4,0,8,6,128,88,3,213,112,0,255,224
       defb 0,31,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
nme3ld defb 0,0,0,0,0,0,0,15,192,0,59,112,0,95,216,0,253,116,0,239,252,1,187,58
       defb 1,255,150,1,255,150,0,255,252,0,31,224,1,0,2,1,160,22,0,245,92,0,63,248
       defb 0,7,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


crate0 defb 6,24,0,13,244,0,7,24,0,1,224,0,0,0,0,85,239,128,47,239,128,85,106,128
       defb 0,0,0,2,193,0,23,213,0,54,199,0,62,215,0,54,197,0,0,0,0,85,239,128
       defb 47,239,128,85,106,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate1 defb 1,134,0,3,125,0,1,198,0,0,120,0,0,0,0,21,123,224,11,251,224,21,90,160
       defb 0,0,0,0,176,64,5,245,64,13,177,192,15,181,192,13,177,64,0,0,0,21,123,224
       defb 11,251,224,21,90,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate2 defb 0,97,128,0,223,64,0,113,128,0,30,0,0,0,0,5,94,248,2,254,248,5,86,168
       defb 0,0,0,0,44,16,1,125,80,3,108,112,3,237,112,3,108,80,0,0,0,5,94,248
       defb 2,254,248,5,86,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate3 defb 0,24,96,0,55,208,0,28,96,0,7,128,0,0,0,1,87,190,0,191,190,1,85,170
       defb 0,0,0,0,11,4,0,95,84,0,219,28,0,251,92,0,219,20,0,0,0,1,87,190
       defb 0,191,190,1,85,170,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

crate4 defb 1,224,0,3,240,0,0,176,0,1,64,0,0,0,0,85,239,128,47,239,128,85,106,128
       defb 0,0,0,2,193,0,23,213,0,54,199,0,62,215,0,54,197,0,0,0,0,85,239,128
       defb 47,239,128,85,106,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate5 defb 0,120,0,0,252,0,0,44,0,0,80,0,0,0,0,21,123,224,11,251,224,21,90,160
       defb 0,0,0,0,176,64,5,245,64,13,177,192,15,181,192,13,177,64,0,0,0,21,123,224
       defb 11,251,224,21,90,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate6 defb 0,30,0,0,63,0,0,11,0,0,20,0,0,0,0,5,94,248,2,254,248,5,86,168
       defb 0,0,0,0,44,16,1,125,80,3,108,112,3,237,112,3,108,80,0,0,0,5,94,248
       defb 2,254,248,5,86,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
crate7 defb 0,7,128,0,15,192,0,2,192,0,5,0,0,0,0,1,87,190,0,191,190,1,85,170
       defb 0,0,0,0,11,4,0,95,84,0,219,28,0,251,92,0,219,20,0,0,0,1,87,190
       defb 0,191,190,1,85,170,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl0a defb 0,0,0,0,0,0,0,0,0,0,24,0,0,60,0,14,24,0,31,34,0,31,7,0
       defb 15,194,0,1,224,0,1,224,0,1,220,0,15,30,0,31,30,0,31,12,0,14,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl0b defb 0,0,0,0,0,0,0,0,0,0,6,0,0,15,0,3,134,0,7,200,128,7,193,192
       defb 3,240,128,0,120,0,0,120,0,0,119,0,3,199,128,7,199,128,7,195,0,3,128,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl0c defb 0,0,0,0,0,0,0,0,0,0,1,128,0,3,192,0,225,128,1,242,32,1,240,112
       defb 0,252,32,0,30,0,0,30,0,0,29,192,0,241,224,1,241,224,1,240,192,0,224,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl0d defb 0,0,0,0,0,0,0,0,0,0,0,96,0,0,240,0,56,96,0,124,136,0,124,28
       defb 0,63,8,0,7,128,0,7,128,0,7,112,0,60,120,0,124,120,0,124,48,0,56,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl1a defb 0,0,0,0,0,0,0,60,0,0,126,0,31,126,0,63,127,0,63,255,0,63,255,0
       defb 31,231,0,15,238,0,1,254,0,15,191,0,63,191,0,63,158,0,31,12,0,14,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl1b defb 0,0,0,0,0,0,0,15,0,0,31,128,7,223,128,15,223,192,15,255,192,15,255,192
       defb 7,249,192,3,251,128,0,127,128,3,239,192,15,239,192,15,231,128,7,195,0,3,128,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl1c defb 0,0,0,0,0,0,0,3,192,0,7,224,1,247,224,3,247,240,3,255,240,3,255,240
       defb 1,254,112,0,254,224,0,31,224,0,251,240,3,251,240,3,249,224,1,240,192,0,224,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl1d defb 0,0,0,0,0,0,0,0,240,0,1,248,0,125,248,0,253,252,0,255,252,0,255,252
       defb 0,127,156,0,63,184,0,7,248,0,62,252,0,254,252,0,254,120,0,124,48,0,56,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl2a defb 0,0,0,0,24,0,31,126,0,63,255,0,127,255,128,127,255,128,127,255,128,127,255,0
       defb 127,247,0,63,239,0,31,255,0,31,255,128,63,255,128,127,255,128,127,255,0,63,190,0
       defb 14,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl2b defb 0,0,0,0,6,0,7,223,128,15,255,192,31,255,224,31,255,224,31,255,224,31,255,192
       defb 31,253,192,15,251,192,7,255,192,7,255,224,15,255,224,31,255,224,31,255,192,15,239,128
       defb 3,131,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl2c defb 0,0,0,0,1,128,1,247,224,3,255,240,7,255,248,7,255,248,7,255,248,7,255,240
       defb 7,255,112,3,254,240,1,255,240,1,255,248,3,255,248,7,255,248,7,255,240,3,251,224
       defb 0,224,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl2d defb 0,0,0,0,0,96,0,125,248,0,255,252,1,255,254,1,255,254,1,255,254,1,255,252
       defb 1,255,220,0,255,188,0,127,252,0,127,254,0,255,254,1,255,254,1,255,252,0,254,248
       defb 0,56,56,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl3a defb 0,0,0,14,60,0,63,127,0,127,255,128,127,255,128,115,223,128,119,239,128,127,243,128
       defb 127,227,0,62,239,0,31,239,128,63,239,128,127,255,128,115,255,128,123,255,128,127,159,0
       defb 63,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl3b defb 0,0,0,3,143,0,15,223,192,31,255,224,31,255,224,28,247,224,29,251,224,31,252,224
       defb 31,248,192,15,187,192,7,251,224,15,251,224,31,255,224,28,255,224,30,255,224,31,231,192
       defb 15,195,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl3c defb 0,0,0,0,227,192,3,247,240,7,255,248,7,255,248,7,61,248,7,126,248,7,255,56
       defb 7,254,48,3,238,240,1,254,248,3,254,248,7,255,248,7,63,248,7,191,248,7,249,240
       defb 3,240,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl3d defb 0,0,0,0,56,240,0,253,252,1,255,254,1,255,254,1,207,126,1,223,190,1,255,206
       defb 1,255,140,0,251,188,0,127,190,0,255,190,1,255,254,1,207,254,1,239,254,1,254,124
       defb 0,252,56,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl4a defb 0,0,0,14,60,0,62,127,0,126,123,128,124,57,128,51,159,128,7,207,128,55,225,128
       defb 123,192,0,60,7,0,31,15,128,59,159,128,123,221,128,115,223,128,113,223,128,111,159,0
       defb 63,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl4b defb 0,0,0,3,143,0,15,159,192,31,158,224,31,14,96,12,231,224,1,243,224,13,248,96
       defb 30,240,0,15,1,192,7,195,224,14,231,224,30,247,96,28,247,224,28,119,224,27,231,192
       defb 15,195,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl4c defb 0,0,0,0,227,192,3,231,240,7,231,184,7,195,152,3,57,248,0,124,248,3,126,24
       defb 7,188,0,3,192,112,1,240,248,3,185,248,7,189,216,7,61,248,7,29,248,6,249,240
       defb 3,240,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl4d defb 0,0,0,0,56,240,0,249,252,1,249,238,1,240,230,0,206,126,0,31,62,0,223,134
       defb 1,239,0,0,240,28,0,124,62,0,238,126,1,239,118,1,207,126,1,199,126,1,190,124
       defb 0,252,56,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

expl5a defb 0,0,0,4,56,0,30,123,0,60,123,128,56,57,128,32,16,0,1,128,128,51,193,128
       defb 121,128,0,56,3,0,3,15,128,59,141,128,59,216,128,113,217,128,96,25,0,7,15,0
       defb 15,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl5b defb 0,0,0,1,14,0,7,158,192,15,30,224,14,14,96,8,4,0,0,96,32,12,240,96
       defb 30,96,0,14,0,192,0,195,224,14,227,96,14,246,32,28,118,96,24,6,64,1,195,192
       defb 3,193,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl5c defb 0,0,0,0,67,128,1,231,176,3,199,184,3,131,152,2,1,0,0,24,8,3,60,24
       defb 7,152,0,3,128,48,0,48,248,3,184,216,3,189,136,7,29,152,6,1,144,0,112,240
       defb 0,240,96,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
expl5d defb 0,0,0,0,16,224,0,121,236,0,241,238,0,224,230,0,128,64,0,6,2,0,207,6
       defb 1,230,0,0,224,12,0,12,62,0,238,54,0,239,98,1,199,102,1,128,100,0,28,60
       defb 0,60,24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; Tomato, left.

toml0a defb 0,0,0,14,80,0,1,176,0,0,64,0,7,176,0,8,252,0,16,62,0,43,31,0
       defb 47,223,0,91,207,128,95,239,128,78,79,128,84,239,128,20,207,0,47,223,0,11,158,0
       defb 16,60,0,5,112,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml0b defb 0,0,0,3,148,0,0,108,0,0,16,0,1,236,0,2,63,0,4,15,128,10,199,192
       defb 11,247,192,22,243,224,23,251,224,19,147,224,21,59,224,5,51,192,11,247,192,2,231,128
       defb 4,15,0,1,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml0c defb 0,0,0,0,229,0,0,27,0,0,4,0,0,123,0,0,143,192,1,3,224,2,177,240
       defb 2,253,240,5,188,248,5,254,248,4,228,248,5,78,248,1,76,240,2,253,240,0,185,224
       defb 1,3,192,0,87,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml0d defb 0,0,0,0,57,64,0,6,192,0,1,0,0,30,192,0,35,240,0,64,248,0,172,124
       defb 0,191,124,1,111,62,1,127,190,1,57,62,1,83,190,0,83,60,0,191,124,0,46,120
       defb 0,64,240,0,21,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

toml1a defb 0,0,0,12,64,0,6,80,0,1,160,0,0,64,0,7,176,0,8,252,0,16,62,0
       defb 43,31,0,47,223,0,91,207,128,95,239,128,78,79,128,84,239,128,20,207,0,11,159,0
       defb 16,62,0,5,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml1b defb 0,0,0,3,16,0,1,148,0,0,104,0,0,16,0,1,236,0,2,63,0,4,15,128
       defb 10,199,192,11,247,192,22,243,224,23,251,224,19,147,224,21,59,224,5,51,192,2,231,192
       defb 4,15,128,1,94,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml1c defb 0,0,0,0,196,0,0,101,0,0,26,0,0,4,0,0,123,0,0,143,192,1,3,224
       defb 2,177,240,2,253,240,5,188,248,5,254,248,4,228,248,5,78,248,1,76,240,0,185,240
       defb 1,3,224,0,87,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml1d defb 0,0,0,0,49,0,0,25,64,0,6,128,0,1,0,0,30,192,0,35,240,0,64,248
       defb 0,172,124,0,191,124,1,111,62,1,127,190,1,57,62,1,83,190,0,83,60,0,46,124
       defb 0,64,248,0,21,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

toml2a defb 0,0,0,7,176,0,12,96,0,7,176,0,8,252,0,16,62,0,43,31,0,47,223,0
       defb 91,207,128,95,239,128,78,79,128,84,239,128,20,207,0,47,223,0,11,158,0,16,60,0
       defb 5,112,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml2b defb 0,0,0,1,236,0,3,24,0,1,236,0,2,63,0,4,15,128,10,199,192,11,247,192
       defb 22,243,224,23,251,224,19,147,224,21,59,224,5,51,192,11,247,192,2,231,128,4,15,0
       defb 1,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml2c defb 0,0,0,0,123,0,0,198,0,0,123,0,0,143,192,1,3,224,2,177,240,2,253,240
       defb 5,188,248,5,254,248,4,228,248,5,78,248,1,76,240,2,253,240,0,185,224,1,3,192
       defb 0,87,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml2d defb 0,0,0,0,30,192,0,49,128,0,30,192,0,35,240,0,64,248,0,172,124,0,191,124
       defb 1,111,62,1,127,190,1,57,62,1,83,190,0,83,60,0,191,124,0,46,120,0,64,240
       defb 0,21,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

toml3a defb 6,64,0,13,176,0,0,64,0,7,176,0,8,252,0,16,62,0,43,31,0,47,223,0
       defb 91,207,128,95,239,128,78,79,128,84,239,128,84,239,128,31,207,0,47,223,0,11,158,0
       defb 16,60,0,5,112,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml3b defb 1,144,0,3,108,0,0,16,0,1,236,0,2,63,0,4,15,128,10,199,192,11,247,192
       defb 22,243,224,23,251,224,19,147,224,21,59,224,21,59,224,7,243,192,11,247,192,2,231,128
       defb 4,15,0,1,92,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml3c defb 0,100,0,0,219,0,0,4,0,0,123,0,0,143,192,1,3,224,2,177,240,2,253,240
       defb 5,188,248,5,254,248,4,228,248,5,78,248,5,78,248,1,252,240,2,253,240,0,185,224
       defb 1,3,192,0,87,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
toml3d defb 0,25,0,0,54,192,0,1,0,0,30,192,0,35,240,0,64,248,0,172,124,0,191,124
       defb 1,111,62,1,127,190,1,57,62,1,83,190,1,83,190,0,127,60,0,191,124,0,46,120
       defb 0,64,240,0,21,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; Tomato, right.

tomr0a defb 0,0,0,2,156,0,3,96,0,0,128,0,3,120,0,15,196,0,31,2,0,62,53,0
       defb 62,253,0,124,246,128,125,254,128,124,156,128,125,202,128,60,202,0,62,253,0,30,116,0
       defb 15,2,0,3,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr0b defb 0,0,0,0,167,0,0,216,0,0,32,0,0,222,0,3,241,0,7,192,128,15,141,64
       defb 15,191,64,31,61,160,31,127,160,31,39,32,31,114,160,15,50,128,15,191,64,7,157,0
       defb 3,192,128,0,234,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr0c defb 0,0,0,0,41,192,0,54,0,0,8,0,0,55,128,0,252,64,1,240,32,3,227,80
       defb 3,239,208,7,207,104,7,223,232,7,201,200,7,220,168,3,204,160,3,239,208,1,231,64
       defb 0,240,32,0,58,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr0d defb 0,0,0,0,10,112,0,13,128,0,2,0,0,13,224,0,63,16,0,124,8,0,248,212
       defb 0,251,244,1,243,218,1,247,250,1,242,114,1,247,42,0,243,40,0,251,244,0,121,208
       defb 0,60,8,0,14,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

tomr1a defb 0,0,0,0,140,0,2,152,0,1,96,0,0,128,0,3,120,0,15,196,0,31,2,0
       defb 62,53,0,62,253,0,124,246,128,125,254,128,124,156,128,125,202,128,60,202,0,62,116,0
       defb 31,2,0,7,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr1b defb 0,0,0,0,35,0,0,166,0,0,88,0,0,32,0,0,222,0,3,241,0,7,192,128
       defb 15,141,64,15,191,64,31,61,160,31,127,160,31,39,32,31,114,160,15,50,128,15,157,0
       defb 7,192,128,1,234,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr1c defb 0,0,0,0,8,192,0,41,128,0,22,0,0,8,0,0,55,128,0,252,64,1,240,32
       defb 3,227,80,3,239,208,7,207,104,7,223,232,7,201,200,7,220,168,3,204,160,3,231,64
       defb 1,240,32,0,122,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr1d defb 0,0,0,0,2,48,0,10,96,0,5,128,0,2,0,0,13,224,0,63,16,0,124,8
       defb 0,248,212,0,251,244,1,243,218,1,247,250,1,242,114,1,247,42,0,243,40,0,249,208
       defb 0,124,8,0,30,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

tomr2a defb 0,0,0,3,120,0,1,140,0,3,120,0,15,196,0,31,2,0,62,53,0,62,253,0
       defb 124,246,128,125,254,128,124,156,128,125,202,128,60,202,0,62,253,0,30,116,0,15,2,0
       defb 3,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr2b defb 0,0,0,0,222,0,0,99,0,0,222,0,3,241,0,7,192,128,15,141,64,15,191,64
       defb 31,61,160,31,127,160,31,39,32,31,114,160,15,50,128,15,191,64,7,157,0,3,192,128
       defb 0,234,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr2c defb 0,0,0,0,55,128,0,24,192,0,55,128,0,252,64,1,240,32,3,227,80,3,239,208
       defb 7,207,104,7,223,232,7,201,200,7,220,168,3,204,160,3,239,208,1,231,64,0,240,32
       defb 0,58,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr2d defb 0,0,0,0,13,224,0,6,48,0,13,224,0,63,16,0,124,8,0,248,212,0,251,244
       defb 1,243,218,1,247,250,1,242,114,1,247,42,0,243,40,0,251,244,0,121,208,0,60,8
       defb 0,14,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

tomr3a defb 0,152,0,3,108,0,0,128,0,3,120,0,15,196,0,31,2,0,62,53,0,62,253,0
       defb 124,246,128,125,254,128,124,156,128,125,202,128,125,202,128,60,254,0,62,253,0,30,116,0
       defb 15,2,0,3,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr3b defb 0,38,0,0,219,0,0,32,0,0,222,0,3,241,0,7,192,128,15,141,64,15,191,64
       defb 31,61,160,31,127,160,31,39,32,31,114,160,31,114,160,15,63,128,15,191,64,7,157,0
       defb 3,192,128,0,234,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr3c defb 0,9,128,0,54,192,0,8,0,0,55,128,0,252,64,1,240,32,3,227,80,3,239,208
       defb 7,207,104,7,223,232,7,201,200,7,220,168,7,220,168,3,207,224,3,239,208,1,231,64
       defb 0,240,32,0,58,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tomr3d defb 0,2,96,0,13,176,0,2,0,0,13,224,0,63,16,0,124,8,0,248,212,0,251,244
       defb 1,243,218,1,247,250,1,242,114,1,247,42,1,247,42,0,243,248,0,251,244,0,121,208
       defb 0,60,8,0,14,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; Completed hot dog.

hdog0  defb 0,0,0,0,0,0,0,0,0,112,0,0,140,0,0,23,0,0,131,192,0,69,224,0
       defb 162,248,0,81,60,0,56,206,0,94,51,0,47,132,128,21,240,128,2,191,0,0,84,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
hdog1  defb 0,0,0,0,0,0,0,0,0,28,0,0,35,0,0,5,192,0,32,240,0,17,120,0
       defb 40,190,0,20,79,0,14,51,128,23,140,192,11,225,32,5,124,32,0,175,192,0,21,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
hdog2  defb 0,0,0,0,0,0,0,0,0,7,0,0,8,192,0,1,112,0,8,60,0,4,94,0
       defb 10,47,128,5,19,192,3,140,224,5,227,48,2,248,72,1,95,8,0,43,240,0,5,64
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
hdog3  defb 0,0,0,0,0,0,0,0,0,1,192,0,2,48,0,0,92,0,2,15,0,1,23,128
       defb 2,139,224,1,68,240,0,227,56,1,120,204,0,190,18,0,87,194,0,10,252,0,1,80
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

cburg0 defb 0,0,0,0,0,0,0,0,0,3,224,0,30,252,0,59,190,0,110,239,0,123,251,0
       defb 127,191,0,63,254,128,143,248,128,66,64,128,0,1,0,80,15,0,42,254,0,5,176,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
cburg1 defb 0,0,0,0,0,0,0,0,0,0,248,0,7,191,0,14,239,128,27,187,192,30,254,192
       defb 31,239,192,15,255,160,35,254,32,16,144,32,0,0,64,20,3,192,10,191,128,1,108,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
cburg2 defb 0,0,0,0,0,0,0,0,0,0,62,0,1,239,192,3,187,224,6,238,240,7,191,176
       defb 7,251,240,3,255,232,8,255,136,4,36,8,0,0,16,5,0,240,2,175,224,0,91,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
cburg3 defb 0,0,0,0,0,0,0,0,0,0,15,128,0,123,240,0,238,248,1,187,188,1,239,236
       defb 1,254,252,0,255,250,2,63,226,1,9,2,0,0,4,1,64,60,0,171,248,0,22,192
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; Hot dog finger, left.

finl0a defb 0,0,0,2,32,0,7,112,0,7,48,0,15,152,0,15,184,0,15,184,0,15,152,0
       defb 15,184,0,15,152,0,14,184,0,4,152,0,4,184,0,15,152,0,7,48,0,7,64,0
       defb 2,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl0b defb 0,0,0,0,136,0,1,220,0,1,204,0,3,230,0,3,238,0,3,238,0,3,230,0
       defb 3,238,0,3,230,0,3,174,0,1,38,0,1,46,0,3,230,0,1,204,0,1,208,0
       defb 0,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl0c defb 0,0,0,0,34,0,0,119,0,0,115,0,0,249,128,0,251,128,0,251,128,0,249,128
       defb 0,251,128,0,249,128,0,235,128,0,73,128,0,75,128,0,249,128,0,115,0,0,116,0
       defb 0,42,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl0d defb 0,0,0,0,8,128,0,29,192,0,28,192,0,62,96,0,62,224,0,62,224,0,62,96
       defb 0,62,224,0,62,96,0,58,224,0,18,96,0,18,224,0,62,96,0,28,192,0,29,0
       defb 0,10,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finl1a defb 0,0,0,3,64,0,7,160,0,7,176,0,15,208,0,15,216,0,15,200,0,15,216,0
       defb 15,200,0,15,216,0,7,8,0,10,88,0,10,72,0,15,208,0,7,144,0,7,160,0
       defb 3,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl1b defb 0,0,0,0,208,0,1,232,0,1,236,0,3,244,0,3,246,0,3,242,0,3,246,0
       defb 3,242,0,3,246,0,1,194,0,2,150,0,2,146,0,3,244,0,1,228,0,1,232,0
       defb 0,208,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl1c defb 0,0,0,0,52,0,0,122,0,0,123,0,0,253,0,0,253,128,0,252,128,0,253,128
       defb 0,252,128,0,253,128,0,112,128,0,165,128,0,164,128,0,253,0,0,121,0,0,122,0
       defb 0,52,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl1d defb 0,0,0,0,13,0,0,30,128,0,30,192,0,63,64,0,63,96,0,63,32,0,63,96
       defb 0,63,32,0,63,96,0,28,32,0,41,96,0,41,32,0,63,64,0,30,64,0,30,128
       defb 0,13,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finl2a defb 0,0,0,2,32,0,7,112,0,7,48,0,15,152,0,15,184,0,15,184,0,15,152,0
       defb 15,184,0,15,152,0,14,56,0,4,152,0,4,184,0,15,152,0,7,48,0,7,16,0
       defb 2,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl2b defb 0,0,0,0,136,0,1,220,0,1,204,0,3,230,0,3,238,0,3,238,0,3,230,0
       defb 3,238,0,3,230,0,3,142,0,1,38,0,1,46,0,3,230,0,1,204,0,1,196,0
       defb 0,144,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl2c defb 0,0,0,0,34,0,0,119,0,0,115,0,0,249,128,0,251,128,0,251,128,0,249,128
       defb 0,251,128,0,249,128,0,227,128,0,73,128,0,75,128,0,249,128,0,115,0,0,113,0
       defb 0,36,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl2d defb 0,0,0,0,8,128,0,29,192,0,28,192,0,62,96,0,62,224,0,62,224,0,62,96
       defb 0,62,224,0,62,96,0,56,224,0,18,96,0,18,224,0,62,96,0,28,192,0,28,64
       defb 0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finl3a defb 0,0,0,2,96,0,6,176,0,7,112,0,15,56,0,15,120,0,15,56,0,15,88,0
       defb 15,56,0,15,88,0,13,56,0,9,88,0,9,56,0,15,24,0,7,80,0,6,0,0
       defb 2,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl3b defb 0,0,0,0,152,0,1,172,0,1,220,0,3,206,0,3,222,0,3,206,0,3,214,0
       defb 3,206,0,3,214,0,3,78,0,2,86,0,2,78,0,3,198,0,1,212,0,1,128,0
       defb 0,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl3c defb 0,0,0,0,38,0,0,107,0,0,119,0,0,243,128,0,247,128,0,243,128,0,245,128
       defb 0,243,128,0,245,128,0,211,128,0,149,128,0,147,128,0,241,128,0,117,0,0,96,0
       defb 0,42,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finl3d defb 0,0,0,0,9,128,0,26,192,0,29,192,0,60,224,0,61,224,0,60,224,0,61,96
       defb 0,60,224,0,61,96,0,52,224,0,37,96,0,36,224,0,60,96,0,29,64,0,24,0
       defb 0,10,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finr0a defb 0,0,0,1,16,0,3,184,0,3,56,0,6,124,0,7,124,0,7,124,0,6,124,0
       defb 7,124,0,6,124,0,7,92,0,6,72,0,7,72,0,6,124,0,3,56,0,0,184,0
       defb 1,80,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr0b defb 0,0,0,0,68,0,0,238,0,0,206,0,1,159,0,1,223,0,1,223,0,1,159,0
       defb 1,223,0,1,159,0,1,215,0,1,146,0,1,210,0,1,159,0,0,206,0,0,46,0
       defb 0,84,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr0c defb 0,0,0,0,17,0,0,59,128,0,51,128,0,103,192,0,119,192,0,119,192,0,103,192
       defb 0,119,192,0,103,192,0,117,192,0,100,128,0,116,128,0,103,192,0,51,128,0,11,128
       defb 0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr0d defb 0,0,0,0,4,64,0,14,224,0,12,224,0,25,240,0,29,240,0,29,240,0,25,240
       defb 0,29,240,0,25,240,0,29,112,0,25,32,0,29,32,0,25,240,0,12,224,0,2,224
       defb 0,5,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finr1a defb 0,0,0,0,176,0,1,120,0,3,120,0,2,252,0,6,252,0,4,252,0,6,252,0
       defb 4,252,0,6,252,0,4,56,0,6,148,0,4,148,0,2,252,0,2,120,0,1,120,0
       defb 0,176,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr1b defb 0,0,0,0,44,0,0,94,0,0,222,0,0,191,0,1,191,0,1,63,0,1,191,0
       defb 1,63,0,1,191,0,1,14,0,1,165,0,1,37,0,0,191,0,0,158,0,0,94,0
       defb 0,44,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr1c defb 0,0,0,0,11,0,0,23,128,0,55,128,0,47,192,0,111,192,0,79,192,0,111,192
       defb 0,79,192,0,111,192,0,67,128,0,105,64,0,73,64,0,47,192,0,39,128,0,23,128
       defb 0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr1d defb 0,0,0,0,2,192,0,5,224,0,13,224,0,11,240,0,27,240,0,19,240,0,27,240
       defb 0,19,240,0,27,240,0,16,224,0,26,80,0,18,80,0,11,240,0,9,224,0,5,224
       defb 0,2,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finr2a defb 0,0,0,1,16,0,3,184,0,3,56,0,6,124,0,7,124,0,7,124,0,6,124,0
       defb 7,124,0,6,124,0,7,28,0,6,72,0,7,72,0,6,124,0,3,56,0,2,56,0
       defb 0,144,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr2b defb 0,0,0,0,68,0,0,238,0,0,206,0,1,159,0,1,223,0,1,223,0,1,159,0
       defb 1,223,0,1,159,0,1,199,0,1,146,0,1,210,0,1,159,0,0,206,0,0,142,0
       defb 0,36,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr2c defb 0,0,0,0,17,0,0,59,128,0,51,128,0,103,192,0,119,192,0,119,192,0,103,192
       defb 0,119,192,0,103,192,0,113,192,0,100,128,0,116,128,0,103,192,0,51,128,0,35,128
       defb 0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr2d defb 0,0,0,0,4,64,0,14,224,0,12,224,0,25,240,0,29,240,0,29,240,0,25,240
       defb 0,29,240,0,25,240,0,28,112,0,25,32,0,29,32,0,25,240,0,12,224,0,8,224
       defb 0,2,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

finr3a  defb 0,0,0,1,144,0,3,88,0,3,184,0,7,60,0,7,188,0,7,60,0,6,188,0
        defb 7,60,0,6,188,0,7,44,0,6,164,0,7,36,0,6,60,0,2,184,0,0,24,0
        defb 1,80,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr3b  defb 0,0,0,0,100,0,0,214,0,0,238,0,1,207,0,1,239,0,1,207,0,1,175,0
        defb 1,207,0,1,175,0,1,203,0,1,169,0,1,201,0,1,143,0,0,174,0,0,6,0
        defb 0,84,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr3c  defb 0,0,0,0,25,0,0,53,128,0,59,128,0,115,192,0,123,192,0,115,192,0,107,192
        defb 0,115,192,0,107,192,0,114,192,0,106,64,0,114,64,0,99,192,0,43,128,0,1,128
        defb 0,21,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
finr3d  defb 0,0,0,0,6,64,0,13,96,0,14,224,0,28,240,0,30,240,0,28,240,0,26,240
        defb 0,28,240,0,26,240,0,28,176,0,26,144,0,28,144,0,24,240,0,10,224,0,0,96
        defb 0,5,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


; Soft drink can.

coke0  defb 0,0,0,7,0,0,8,160,0,22,176,0,46,152,0,81,44,0,83,86,0,70,239,0
       defb 57,247,0,3,251,128,35,247,128,17,251,128,9,253,128,4,250,0,2,61,0,0,134,0
       defb 0,40,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
coke1  defb 0,0,0,1,192,0,2,40,0,5,172,0,11,166,0,20,75,0,20,213,128,17,187,192
       defb 14,125,192,0,254,224,8,253,224,4,126,224,2,127,96,1,62,128,0,143,64,0,33,128
       defb 0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
coke2  defb 0,0,0,0,112,0,0,138,0,1,107,0,2,233,128,5,18,192,5,53,96,4,110,240
       defb 3,159,112,0,63,184,2,63,120,1,31,184,0,159,216,0,79,160,0,35,208,0,8,96
       defb 0,2,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
coke3  defb 0,0,0,0,28,0,0,34,128,0,90,192,0,186,96,1,68,176,1,77,88,1,27,188
       defb 0,231,220,0,15,238,0,143,222,0,71,238,0,39,246,0,19,232,0,8,244,0,2,24
       defb 0,0,160,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

tango0 defb 0,0,0,0,56,0,1,68,0,3,90,0,7,93,0,11,162,128,31,178,128,47,216,128
       defb 63,231,0,95,248,0,95,255,0,31,250,0,79,244,0,3,200,0,32,16,0,0,96,0
       defb 5,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tango1 defb 0,0,0,0,14,0,0,81,0,0,214,128,1,215,64,2,232,160,7,236,160,11,246,32
       defb 15,249,192,23,254,0,23,255,192,7,254,128,19,253,0,0,242,0,8,4,0,0,24,0
       defb 1,96,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tango2 defb 0,0,0,0,3,128,0,20,64,0,53,160,0,117,208,0,186,40,1,251,40,2,253,136
       defb 3,254,112,5,255,128,5,255,240,1,255,160,4,255,64,0,60,128,2,1,0,0,6,0
       defb 0,88,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
tango3 defb 0,0,0,0,0,224,0,5,16,0,13,104,0,29,116,0,46,138,0,126,202,0,191,98
       defb 0,255,156,1,127,224,1,127,252,0,127,232,1,63,208,0,15,32,0,128,64,0,1,128
       defb 0,22,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


; Bread, right.

brer0a defb 0,0,0,1,0,0,4,112,0,9,252,0,19,254,0,11,252,0,23,254,0,39,254,0
       defb 19,254,0,39,254,0,23,58,0,39,150,0,23,150,0,39,254,0,23,252,0,39,224,0
       defb 19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer0b defb 0,0,0,0,64,0,1,28,0,2,127,0,4,255,128,2,255,0,5,255,128,9,255,128
       defb 4,255,128,9,255,128,5,206,128,9,229,128,5,229,128,9,255,128,5,255,0,9,248,0
       defb 4,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer0c defb 0,0,0,0,16,0,0,71,0,0,159,192,1,63,224,0,191,192,1,127,224,2,127,224
       defb 1,63,224,2,127,224,1,115,160,2,121,96,1,121,96,2,127,224,1,127,192,2,126,0
       defb 1,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer0d defb 0,0,0,0,4,0,0,17,192,0,39,240,0,79,248,0,47,240,0,95,248,0,159,248
       defb 0,79,248,0,159,248,0,92,232,0,158,88,0,94,88,0,159,248,0,95,240,0,159,128
       defb 0,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brer1a defb 0,0,0,0,0,0,9,248,0,7,254,0,47,255,0,15,254,0,95,255,0,31,255,0
       defb 47,255,0,95,255,0,30,115,0,95,39,0,31,39,0,95,255,0,31,252,0,95,224,0
       defb 46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer1b defb 0,0,0,0,0,0,2,126,0,1,255,128,11,255,192,3,255,128,23,255,192,7,255,192
       defb 11,255,192,23,255,192,7,156,192,23,201,192,7,201,192,23,255,192,7,255,0,23,248,0
       defb 11,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer1c defb 0,0,0,0,0,0,0,159,128,0,127,224,2,255,240,0,255,224,5,255,240,1,255,240
       defb 2,255,240,5,255,240,1,231,48,5,242,112,1,242,112,5,255,240,1,255,192,5,254,0
       defb 2,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer1d defb 0,0,0,0,0,0,0,39,224,0,31,248,0,191,252,0,63,248,1,127,252,0,127,252
       defb 0,191,252,1,127,252,0,121,204,1,124,156,0,124,156,1,127,252,0,127,240,1,127,128
       defb 0,184,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brer2a defb 0,0,0,1,0,0,4,112,0,9,252,0,19,254,0,11,252,0,23,254,0,39,254,0
       defb 19,254,0,39,254,0,23,58,0,39,150,0,23,150,0,39,254,0,23,252,0,39,224,0
       defb 19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer2b defb 0,0,0,0,64,0,1,28,0,2,127,0,4,255,128,2,255,0,5,255,128,9,255,128
       defb 4,255,128,9,255,128,5,206,128,9,229,128,5,229,128,9,255,128,5,255,0,9,248,0
       defb 4,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer2c defb 0,0,0,0,16,0,0,71,0,0,159,192,1,63,224,0,191,192,1,127,224,2,127,224
       defb 1,63,224,2,127,224,1,115,160,2,121,96,1,121,96,2,127,224,1,127,192,2,126,0
       defb 1,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer2d defb 0,0,0,0,4,0,0,17,192,0,39,240,0,79,248,0,47,240,0,95,248,0,159,248
       defb 0,79,248,0,159,248,0,92,232,0,158,88,0,94,88,0,159,248,0,95,240,0,159,128
       defb 0,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brer3a defb 0,0,0,0,0,0,2,112,0,4,248,0,9,248,0,5,252,0,11,252,0,19,252,0
       defb 9,252,0,19,252,0,11,148,0,19,196,0,11,196,0,19,252,0,11,248,0,19,224,0
       defb 9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer3b defb 0,0,0,0,0,0,0,156,0,1,62,0,2,126,0,1,127,0,2,255,0,4,255,0
       defb 2,127,0,4,255,0,2,229,0,4,241,0,2,241,0,4,255,0,2,254,0,4,248,0
       defb 2,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer3c defb 0,0,0,0,0,0,0,39,0,0,79,128,0,159,128,0,95,192,0,191,192,1,63,192
       defb 0,159,192,1,63,192,0,185,64,1,60,64,0,188,64,1,63,192,0,191,128,1,62,0
       defb 0,144,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brer3d defb 0,0,0,0,0,0,0,9,192,0,19,224,0,39,224,0,23,240,0,47,240,0,79,240
       defb 0,39,240,0,79,240,0,46,80,0,79,16,0,47,16,0,79,240,0,47,224,0,79,128
       defb 0,36,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

; Bread, left.  Copied from bread right, needs doing.

brel0a defb 0,0,0,1,0,0,4,112,0,9,252,0,19,254,0,11,252,0,23,254,0,39,254,0
       defb 19,254,0,39,254,0,23,58,0,39,150,0,23,150,0,39,254,0,23,252,0,39,224,0
       defb 19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel0b defb 0,0,0,0,64,0,1,28,0,2,127,0,4,255,128,2,255,0,5,255,128,9,255,128
       defb 4,255,128,9,255,128,5,206,128,9,229,128,5,229,128,9,255,128,5,255,0,9,248,0
       defb 4,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel0c defb 0,0,0,0,16,0,0,71,0,0,159,192,1,63,224,0,191,192,1,127,224,2,127,224
       defb 1,63,224,2,127,224,1,115,160,2,121,96,1,121,96,2,127,224,1,127,192,2,126,0
       defb 1,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel0d defb 0,0,0,0,4,0,0,17,192,0,39,240,0,79,248,0,47,240,0,95,248,0,159,248
       defb 0,79,248,0,159,248,0,92,232,0,158,88,0,94,88,0,159,248,0,95,240,0,159,128
       defb 0,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brel1a defb 0,0,0,0,0,0,9,248,0,7,254,0,47,255,0,15,254,0,95,255,0,31,255,0
       defb 47,255,0,95,255,0,30,115,0,95,39,0,31,39,0,95,255,0,31,252,0,95,224,0
       defb 46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel1b defb 0,0,0,0,0,0,2,126,0,1,255,128,11,255,192,3,255,128,23,255,192,7,255,192
       defb 11,255,192,23,255,192,7,156,192,23,201,192,7,201,192,23,255,192,7,255,0,23,248,0
       defb 11,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel1c defb 0,0,0,0,0,0,0,159,128,0,127,224,2,255,240,0,255,224,5,255,240,1,255,240
       defb 2,255,240,5,255,240,1,231,48,5,242,112,1,242,112,5,255,240,1,255,192,5,254,0
       defb 2,224,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel1d defb 0,0,0,0,0,0,0,39,224,0,31,248,0,191,252,0,63,248,1,127,252,0,127,252
       defb 0,191,252,1,127,252,0,121,204,1,124,156,0,124,156,1,127,252,0,127,240,1,127,128
       defb 0,184,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brel2a defb 0,0,0,1,0,0,4,112,0,9,252,0,19,254,0,11,252,0,23,254,0,39,254,0
       defb 19,254,0,39,254,0,23,58,0,39,150,0,23,150,0,39,254,0,23,252,0,39,224,0
       defb 19,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel2b defb 0,0,0,0,64,0,1,28,0,2,127,0,4,255,128,2,255,0,5,255,128,9,255,128
       defb 4,255,128,9,255,128,5,206,128,9,229,128,5,229,128,9,255,128,5,255,0,9,248,0
       defb 4,192,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel2c defb 0,0,0,0,16,0,0,71,0,0,159,192,1,63,224,0,191,192,1,127,224,2,127,224
       defb 1,63,224,2,127,224,1,115,160,2,121,96,1,121,96,2,127,224,1,127,192,2,126,0
       defb 1,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel2d defb 0,0,0,0,4,0,0,17,192,0,39,240,0,79,248,0,47,240,0,95,248,0,159,248
       defb 0,79,248,0,159,248,0,92,232,0,158,88,0,94,88,0,159,248,0,95,240,0,159,128
       defb 0,76,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

brel3a defb 0,0,0,0,0,0,2,112,0,4,248,0,9,248,0,5,252,0,11,252,0,19,252,0
       defb 9,252,0,19,252,0,11,148,0,19,196,0,11,196,0,19,252,0,11,248,0,19,224,0
       defb 9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel3b defb 0,0,0,0,0,0,0,156,0,1,62,0,2,126,0,1,127,0,2,255,0,4,255,0
       defb 2,127,0,4,255,0,2,229,0,4,241,0,2,241,0,4,255,0,2,254,0,4,248,0
       defb 2,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel3c defb 0,0,0,0,0,0,0,39,0,0,79,128,0,159,128,0,95,192,0,191,192,1,63,192
       defb 0,159,192,1,63,192,0,185,64,1,60,64,0,188,64,1,63,192,0,191,128,1,62,0
       defb 0,144,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
brel3d defb 0,0,0,0,0,0,0,9,192,0,19,224,0,39,224,0,23,240,0,47,240,0,79,240
       defb 0,39,240,0,79,240,0,46,80,0,79,16,0,47,16,0,79,240,0,47,224,0,79,128
       defb 0,36,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0


       org 53248

; Yerzmyey's music.

imusic defb 33,60,212,195,9,208
music  defb 195,68,209,243,126,50,120,208,34,187,208,35,205,181,208,26,19,60
       defb 50,122,208,237,83,112,208,205,181,208,237,83,114,208,213,205,181,208,237,83,116,208,33,27
       defb 0,205,186,208,235,34,118,208,33,129,208,34,123,208,33,130,208,17,131,208,1,44,0,112
       defb 237,176,225,1,33,0,175,205,175,208,61,50,139,208,50,149,208,50,159,208,62,1,50,121
       defb 208,35,34,137,208,34,147,208,34,157,208,205,31,212,251,201,119,241,143,241,19,242,94,238
       defb 6,1,12,69,231,77,243,207,242,255,0,0,0,0,0,0,0,144,241,255,0,0,0,0
       defb 0,0,0,144,241,255,0,0,0,0,0,0,0,144,241,255,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,190,200,9,195,175,208,94,35,86,35,235,1,67,238,9,235,201
       defb 22,0,95,135,131,95,221,25,221,126,1,203,127,14,16,194,211,208,74,203,119,6,2,194
       defb 219,208,66,230,31,103,221,94,2,221,126,0,245,230,240,15,15,15,15,87,241,230,15,111
       defb 221,203,1,110,200,203,226,201,58,160,208,79,33,122,208,190,218,5,209,175,79,60,50,160
       defb 208,105,38,0,41,237,91,112,208,25,78,35,126,50,68,211,121,42,116,208,1,7,0,205
       defb 175,208,35,205,181,208,237,83,123,208,205,181,208,237,83,125,208,205,181,208,237,83,127,208
       defb 201,221,53,2,240,221,126,255,221,119,2,201,58,121,208,61,50,121,208,194,142,210,58,120
       defb 208,50,121,208,221,33,132,208,205,57,209,242,111,209,42,123,208,126,60,204,248,208,42,123
       defb 208,205,152,209,34,123,208,221,33,142,208,205,57,209,242,130,209,42,125,208,205,152,209,34
       defb 125,208,221,33,152,208,205,57,209,242,142,210,42,127,208,205,152,209,34,127,208,195,142,210
       defb 126,254,96,218,198,209,254,112,218,211,209,254,128,218,244,209,202,235,209,254,129,202,209,209
       defb 254,130,202,241,209,254,143,218,16,210,214,161,221,119,2,221,119,255,35,195,152,209,221,119
       defb 1,221,54,0,0,221,54,7,32,35,201,214,96,229,1,99,0,42,118,208,205,175,208,35
       defb 221,117,3,221,116,4,225,35,195,152,209,35,221,54,7,255,201,175,24,2,214,112,229,1
       defb 33,0,42,114,208,205,175,208,35,221,117,5,221,116,6,221,54,254,0,225,35,195,152,209
       defb 214,128,50,174,208,35,126,35,50,172,208,221,54,254,1,229,175,1,33,0,42,114,208,205
       defb 175,208,35,221,117,5,221,116,6,225,195,152,209,221,126,7,60,200,61,61,221,119,7,245
       defb 221,126,0,79,60,230,31,221,119,0,241,192,221,94,3,221,86,4,33,96,0,25,126,61
       defb 250,236,209,79,60,230,31,221,119,0,35,126,60,221,119,7,201,121,183,192,124,50,167,208
       defb 201,221,126,7,60,200,221,126,254,183,200,254,2,202,135,210,221,54,254,2,195,139,210,175
       defb 50,174,208,203,230,201,221,33,132,208,205,53,210,121,50,60,211,221,42,135,208,205,192,208
       defb 121,176,15,50,168,208,221,33,132,208,221,126,7,60,202,186,210,205,105,210,205,50,211,34
       defb 161,208,33,169,208,119,205,113,210,221,33,142,208,205,53,210,221,126,7,60,202,239,210,121
       defb 50,60,211,221,42,145,208,205,192,208,58,168,208,177,176,50,168,208,205,105,210,221,33,142
       defb 208,205,50,211,34,163,208,33,170,208,119,205,113,210,221,33,152,208,205,53,210,221,126,7
       defb 60,202,40,211,121,50,60,211,221,42,155,208,205,192,208,58,168,208,203,1,203,0,176,177
       defb 50,168,208,205,105,210,221,33,152,208,205,50,211,34,165,208,33,171,208,119,205,113,210,195
       defb 31,212,125,245,213,221,110,5,221,102,6,17,1,0,25,221,126,1,134,198,0,135,95,22
       defb 0,33,95,211,25,94,35,86,235,209,241,203,98,40,4,203,162,25,201,167,237,82,201,248
       defb 14,16,14,96,13,128,12,216,11,40,11,136,10,240,9,96,9,224,8,88,8,224,7,124
       defb 7,8,7,176,6,64,6,236,5,148,5,68,5,248,4,176,4,112,4,44,4,240,3,190
       defb 3,132,3,88,3,32,3,246,2,202,2,162,2,124,2,88,2,56,2,22,2,248,1,223
       defb 1,194,1,172,1,144,1,123,1,101,1,81,1,62,1,44,1,28,1,11,1,252,0,239
       defb 0,225,0,214,0,200,0,189,0,178,0,168,0,159,0,150,0,142,0,133,0,126,0,119
       defb 0,112,0,107,0,100,0,94,0,89,0,84,0,79,0,75,0,71,0,66,0,63,0,59
       defb 0,56,0,53,0,50,0,47,0,44,0,42,0,39,0,37,0,35,0,33,0,31,0,29
       defb 0,28,0,26,0,25,0,23,0,22,0,21,0,19,0,18,0,17,0,16,0,15,0,33
       defb 174,208,175,182,62,13,32,5,214,3,43,43,43,14,253,6,255,237,121,6,191,237,171,61
       defb 242,47,212,201,2,34,5,193,5,36,6,83,79,78,71,32,66,89,32,83,84,32,67,79
       defb 77,80,73,76,69,158,19,2,12,64,0,8,64,0,4,64,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,3,15,0,0,15,0,0,15,0,0,15,0,0,15
       defb 0,0,14,0,0,14,0,0,14,0,0,13,0,0,12,0,0,12,0,0,12,0,0,12
       defb 0,0,12,0,0,12,0,0,12,0,0,12,0,0,11,0,0,10,0,0,10,0,0,10
       defb 0,0,9,0,0,9,0,0,8,0,0,7,0,0,7,0,0,7,0,0,7,0,0,7
       defb 0,0,7,0,0,6,0,0,4,0,0,0,0,4,47,160,0,79,160,0,111,160,0,143
       defb 160,0,174,160,0,204,160,0,234,160,0,232,160,151,230,160,186,228,160,216,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,0,5,15,32,0,15,38,47,13
       defb 44,94,12,50,141,11,56,188,10,62,235,25,36,26,24,42,73,22,48,120,21,54,167,20
       defb 60,214,35,34,5,34,40,52,33,46,99,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,15,64,0,15
       defb 64,0,15,64,0,15,64,0,14,64,0,14,64,0,13,64,0,13,64,0,12,64,0,12
       defb 64,0,11,64,0,11,64,0,10,64,0,10,64,0,9,64,0,8,64,0,7,64,0,7
       defb 64,0,7,64,0,7,64,0,6,64,0,6,64,0,6,64,0,5,64,0,5,64,0,5
       defb 64,0,5,64,0,4,64,0,3,64,0,3,64,0,3,64,0,2,64,0,0,0,7,15
       defb 32,0,14,32,0,14,32,0,14,32,0,12,32,0,12,32,0,11,32,0,10,32,0,9
       defb 32,0,9,32,0,8,32,0,8,32,0,7,32,0,6,32,0,6,32,0,6,32,0,6
       defb 32,0,5,32,0,5,32,0,5,32,0,4,32,0,4,32,0,4,32,0,4,32,0,4
       defb 32,0,3,32,0,3,32,0,2,32,0,1,32,0,0,32,0,0,32,0,0,32,0,0
       defb 0,8,15,128,0,15,128,2,14,128,4,14,128,2,14,160,0,14,160,2,14,160,4,14
       defb 160,2,14,128,0,13,128,2,13,128,4,13,128,2,13,160,0,13,160,2,12,160,4,11
       defb 160,2,10,128,0,8,128,2,8,128,4,8,128,2,8,160,0,8,160,2,8,160,4,8
       defb 160,2,8,128,0,8,128,2,8,128,4,8,128,2,7,128,0,7,160,2,6,160,4,5
       defb 160,2,0,29,9,13,128,0,13,128,2,13,128,1,12,128,0,12,128,0,12,160,2,11
       defb 160,1,11,160,0,11,160,0,11,128,2,11,128,1,11,128,1,10,128,0,9,160,2,9
       defb 160,1,8,160,0,8,128,0,8,128,0,7,128,0,6,128,0,6,128,0,6,128,0,5
       defb 128,0,5,128,0,5,128,0,5,128,0,4,128,0,4,128,0,4,128,0,3,128,0,3
       defb 128,0,2,128,0,0,0,10,10,128,0,10,128,1,10,128,1,10,128,0,10,128,0,10
       defb 128,1,10,128,0,10,160,1,10,160,1,10,160,0,10,160,0,10,160,1,10,128,0,10
       defb 128,1,10,128,1,10,128,0,10,128,0,10,128,1,10,128,0,10,160,1,10,160,1,10
       defb 160,0,10,160,0,10,160,1,10,128,0,10,128,1,10,128,1,10,128,0,10,128,0,10
       defb 128,1,10,128,0,10,128,0,3,5,11,14,0,0,11,128,0,11,128,0,11,128,0,11
       defb 128,0,10,128,0,9,128,0,8,128,0,6,128,0,5,128,0,5,128,0,4,128,0,4
       defb 128,0,3,128,0,2,128,0,2,128,0,1,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,0,12,12,128,0,12,128,0,12,128,0,12
       defb 128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12
       defb 128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12,128,0,12
       defb 128,0,12,128,0,12,128,1,12,128,1,12,128,2,12,128,2,12,128,3,12,128,2,12
       defb 128,2,12,128,1,12,128,1,12,128,0,12,128,0,15,16,13,11,128,0,11,128,0,9
       defb 128,0,8,128,0,7,128,0,6,128,0,6,128,0,5,128,0,5,128,0,4,128,0,3
       defb 128,0,3,128,0,2,128,0,1,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,0,14,12,128,0,12
       defb 128,0,12,128,0,11,128,0,10,128,0,9,128,0,8,128,0,7,128,0,6,128,0,6
       defb 128,0,5,128,0,5,128,0,4,128,0,4,128,0,3,128,0,3,128,0,2,128,0,1
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0
       defb 128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,128,0,0,0,78,2
       defb 0,2,0,3,0,4,0,3,0,4,0,30,0,29,0,28,0,31,0,5,0,6,0,5
       defb 0,7,0,8,0,9,0,8,0,9,0,8,0,9,0,8,0,9,0,1,0,10,0,11
       defb 0,12,0,13,0,14,0,13,0,15,0,13,0,14,0,13,0,15,0,16,0,17,0,18
       defb 0,20,0,16,0,17,0,18,0,19,0,23,0,23,0,24,0,24,0,21,0,22,0,21
       defb 0,22,0,21,0,22,0,21,0,25,0,25,0,26,0,27,0,26,0,27,0,26,0,27
       defb 0,26,0,27,0,8,0,9,0,8,0,9,0,8,0,9,0,8,0,9,0,16,0,17
       defb 0,16,0,18,0,16,0,17,0,16,0,19,0,0,0,0,0,0,0,0,0,0,0,0
       defb 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0
       defb 3,7,0,3,7,0,3,7,0,3,7,0,3,7,0,3,7,0,3,7,0,3,7,0
       defb 3,7,0,3,7,0,3,4,0,4,7,0,4,7,0,4,7,0,4,7,0,4,7,0
       defb 4,7,0,4,7,0,4,7,0,4,7,0,4,7,0,4,1,254,6,73,7,80,7,2
       defb 144,7,239,7,246,7,3,144,7,3,8,23,8,4,56,8,143,8,163,8,5,195,8,248
       defb 8,69,9,6,79,9,127,9,206,9,7,79,9,127,9,215,9,8,233,9,40,10,115,10
       defb 9,148,10,211,10,35,11,10,254,6,76,11,89,11,11,254,6,149,11,89,11,12,254,6
       defb 163,11,89,11,13,189,11,248,11,68,12,14,173,12,227,12,49,13,15,173,12,227,12,89
       defb 13,16,129,13,139,13,145,13,17,212,13,212,13,215,13,18,26,14,32,14,145,13,19,38
       defb 14,44,14,215,13,20,86,14,92,14,215,13,21,98,14,145,14,225,14,22,36,15,77,15
       defb 159,15,23,239,7,206,15,219,15,24,24,16,80,16,219,15,25,153,16,194,16,19,17,26
       defb 70,17,123,17,200,17,27,28,18,86,18,161,18,28,245,18,1,19,69,9,29,57,19,66
       defb 19,206,9,30,245,18,72,19,69,9,31,78,19,91,19,215,9,255,161,109,142,30,84,72
       defb 60,48,36,24,12,12,84,72,60,48,36,24,12,12,84,72,60,48,84,72,60,48,36,24
       defb 12,12,142,25,87,75,63,51,39,27,15,15,87,75,63,51,87,75,63,51,39,27,15,15
       defb 142,22,89,77,142,44,65,53,41,29,17,5,89,77,65,53,41,29,17,5,255,182,102,112
       defb 12,202,128,255,161,100,112,41,107,114,36,162,36,103,36,106,36,107,36,36,103,36,106,36
       defb 107,36,36,103,36,106,36,107,36,36,103,116,39,106,39,107,39,39,103,39,106,39,107,39
       defb 39,103,39,106,39,107,41,41,103,41,106,41,107,41,41,103,41,106,41,255,162,105,142,3
       defb 24,142,15,48,142,30,0,142,5,12,142,30,60,142,15,24,142,30,48,142,5,36,142,15
       defb 60,142,3,24,36,142,12,12,142,15,60,142,30,24,142,3,36,142,30,12,142,3,24,142
       defb 15,48,142,30,0,142,5,12,142,30,60,142,15,24,142,30,48,142,5,36,142,15,60,142
       defb 3,24,36,142,12,12,142,15,60,142,30,24,142,3,36,142,30,12,255,162,100,112,41,222
       defb 128,255,208,102,112,36,168,128,162,101,31,31,31,31,255,164,104,112,0,12,0,12,0,12
       defb 0,12,0,12,0,12,0,12,0,12,255,164,100,112,47,98,26,101,31,98,26,100,47,47
       defb 101,31,100,47,98,26,100,47,101,31,98,26,100,47,47,101,31,98,26,255,162,104,142,5
       defb 27,142,25,51,142,34,3,142,5,15,142,25,63,142,10,27,142,25,51,142,5,39,142,25
       defb 63,142,5,27,39,15,142,25,63,142,17,27,142,5,39,142,25,15,142,2,29,142,22,53
       defb 142,30,5,142,11,17,65,17,142,22,53,142,9,41,142,22,65,142,9,29,41,17,142,45
       defb 65,142,30,29,142,9,41,142,30,17,255,164,104,112,3,15,3,15,3,15,3,15,5,17
       defb 5,17,5,17,5,17,255,164,100,112,47,98,26,101,31,98,26,100,47,47,101,31,100,47
       defb 98,26,100,47,101,31,98,26,100,47,101,31,31,31,255,161,100,112,41,163,104,0,164,12
       defb 0,12,161,100,41,163,104,0,161,100,41,163,104,12,164,0,12,161,100,41,163,104,0,164
       defb 12,0,161,100,41,163,104,12,164,0,161,100,41,163,104,12,164,0,12,255,162,107,114,36
       defb 106,36,107,36,106,36,101,112,31,106,114,36,107,36,106,36,107,36,106,36,103,36,106,36
       defb 101,112,31,106,114,36,103,36,106,36,107,36,106,36,107,36,106,36,101,112,31,106,114,36
       defb 107,36,106,36,107,36,106,36,103,36,106,36,101,112,31,106,114,36,101,112,31,106,114,36
       defb 255,193,106,136,10,36,191,136,27,38,255,161,100,112,41,163,104,3,164,15,3,161,100,41
       defb 163,104,15,164,3,161,100,41,163,104,15,164,3,15,161,100,41,163,104,5,164,17,5,161
       defb 100,41,163,104,17,164,5,17,5,17,255,162,107,116,39,106,39,107,39,106,39,101,112,31
       defb 106,116,39,107,39,106,39,107,39,106,39,103,39,106,39,101,112,31,106,116,39,103,39,106
       defb 39,107,41,106,41,107,41,106,41,101,112,31,106,116,41,103,41,106,41,107,41,106,41,101
       defb 112,31,106,116,41,101,112,31,106,116,41,101,112,31,106,116,41,255,192,106,140,25,51,140
       defb 22,53,255,180,106,140,25,51,164,140,11,41,168,140,5,43,192,140,22,53,255,161,100,112
       defb 41,163,104,8,161,100,41,163,104,20,164,8,161,100,41,163,104,20,161,100,41,163,104,8
       defb 164,20,8,161,100,41,163,104,20,164,8,161,100,41,163,104,20,164,8,161,100,41,163,104
       defb 20,161,100,41,163,104,8,164,20,8,20,255,162,107,116,44,106,44,107,44,106,44,101,112
       defb 31,106,116,44,107,44,106,44,107,44,106,44,107,44,106,44,101,112,31,106,116,44,107,44
       defb 106,44,107,44,106,44,107,44,106,44,101,112,31,106,116,44,107,44,106,44,107,44,106,44
       defb 107,44,106,44,101,112,33,106,116,44,107,44,106,44,255,164,105,140,15,44,39,44,140,10
       defb 46,48,46,140,15,44,140,10,51,140,15,44,39,44,140,10,46,48,46,140,15,44,39,255
       defb 161,100,112,41,163,104,5,161,100,41,163,104,17,164,5,161,100,41,163,104,17,161,100,41
       defb 163,104,5,164,17,5,161,100,41,163,104,17,164,3,161,100,41,163,104,15,164,3,161,100
       defb 41,163,104,15,161,100,41,163,104,3,164,15,3,15,255,162,107,114,41,106,41,107,41,106
       defb 41,101,112,31,106,114,41,107,41,106,41,107,41,106,41,107,41,106,41,101,112,31,106,114
       defb 41,107,41,106,41,107,116,39,106,39,107,39,106,39,101,112,31,106,116,39,107,39,106,39
       defb 107,39,106,39,101,112,31,106,116,39,101,112,31,107,116,39,101,112,31,107,116,39,255,164
       defb 105,140,15,44,39,44,140,10,46,48,46,140,15,44,140,10,51,39,140,9,41,140,8,43
       defb 136,19,56,140,10,46,140,6,48,140,7,49,140,25,63,255,208,129,162,101,31,31,31,31
       defb 31,31,31,31,255,162,107,114,36,36,103,36,106,36,107,36,36,103,36,106,36,107,36,36
       defb 103,36,106,36,107,36,36,103,116,39,106,39,107,39,39,103,39,106,39,107,39,39,103,39
       defb 106,39,107,41,41,103,41,106,41,107,41,41,103,41,106,41,255,168,100,112,47,47,47,47
       defb 47,47,47,164,47,47,255,168,100,112,47,47,47,47,162,101,31,31,31,31,31,31,31,31
       defb 31,31,31,31,31,31,31,31,255,161,100,112,41,163,104,0,164,12,0,161,100,41,163,104
       defb 12,164,0,161,100,41,163,104,12,164,0,12,161,100,41,163,104,7,164,19,7,161,100,41
       defb 163,104,19,164,3,161,100,41,163,104,15,164,3,161,100,41,163,104,15,255,162,107,114,36
       defb 106,36,107,36,106,36,101,112,31,106,114,36,107,36,106,36,107,36,106,36,103,36,106,36
       defb 101,112,31,106,114,36,103,36,106,36,107,43,106,43,107,43,106,43,101,112,31,106,114,43
       defb 107,43,106,43,107,116,39,106,39,103,39,106,39,101,112,31,106,116,39,103,39,106,39,255
       defb 162,106,140,10,43,109,112,41,106,140,10,43,109,112,43,106,140,11,41,109,112,43,106,140
       defb 10,39,109,112,41,106,140,9,38,109,112,39,106,140,10,39,109,112,38,106,140,10,36,109
       defb 112,39,168,106,136,40,31,162,140,10,43,109,112,31,106,140,11,41,109,112,43,106,140,10
       defb 43,109,112,41,106,140,10,39,109,112,43,106,140,10,43,109,112,39,106,140,9,38,109,112
       defb 43,106,140,10,43,109,112,38,255,161,100,112,41,163,104,5,164,17,5,161,100,41,163,104
       defb 17,164,5,161,100,41,163,104,17,164,5,17,161,100,41,163,104,8,164,20,8,161,100,41
       defb 163,104,20,164,10,161,100,41,163,104,22,164,10,22,255,162,107,114,41,106,41,107,41,106
       defb 41,101,112,31,106,114,41,107,41,106,41,107,41,106,41,103,41,106,41,101,112,33,106,114
       defb 41,103,41,106,41,107,116,44,106,44,107,44,106,44,101,112,33,106,116,44,107,44,106,44
       defb 107,46,106,46,103,46,106,46,101,112,31,107,116,46,101,112,31,106,116,46,255,168,106,136
       defb 45,41,136,40,43,136,38,44,136,30,48,164,140,25,51,136,27,50,136,30,48,140,33,46
       defb 136,20,55,140,22,53,136,25,51,136,27,50,255,168,106,136,45,41,136,40,43,136,38,44
       defb 136,30,48,164,140,25,51,136,27,50,136,30,48,140,33,46,140,30,60,140,33,58,140,40
       defb 55,140,33,58,255,161,100,112,41,223,106,136,60,24,255,224,106,136,60,0,255,162,110,112
       defb 48,109,46,110,43,109,48,110,48,109,43,110,43,109,48,110,51,109,43,110,50,109,51,110
       defb 48,109,50,110,46,109,48,110,48,109,46,110,43,109,48,110,48,109,43,110,43,109,48,110
       defb 51,109,43,110,50,109,51,110,48,109,50,110,46,109,48,255,224,129,255,162,110,112,48,109
       defb 46,110,43,109,48,110,48,109,43,110,43,109,48,110,51,109,43,110,50,109,51,110,48,109
       defb 50,110,46,109,48,110,48,109,46,110,43,109,48,110,48,109,43,110,43,109,48,110,55,109
       defb 43,110,53,109,55,110,51,109,53,110,50,109,51,255,224,106,140,75,32,255,224,106,140,75
       defb 8,255,224,106,140,89,29,255,192,106,140,89,5,161,100,112,41,167,106,140,89,5,161,100
       defb 112,41,167,106,140,89,5,161,100,112,41,167,106,140,89,5,161,100,112,41,167,106,140,89
       defb 5,255,224,106,136,67,34,255,224,106,136,67,10,255,161,100,112,41,163,104,0,164,12,0
       defb 12,161,100,41,163,104,0,161,100,41,163,104,12,164,0,12,161,100,41,163,104,8,164,20
       defb 8,161,100,41,163,104,20,164,8,20,8,20,255,162,107,114,36,106,36,103,36,106,36,101
       defb 112,31,106,114,36,107,36,106,36,107,36,106,36,107,36,106,36,101,112,31,106,114,36,107
       defb 36,106,36,107,116,44,106,44,103,44,106,44,101,112,33,106,116,44,107,44,106,44,103,44
       defb 106,44,101,112,31,107,116,44,101,112,31,107,116,44,101,112,31,107,116,44,255,162,106,142
       defb 40,55,55,142,4,55,142,8,55,142,20,55,55,142,40,67,67,142,20,55,55,142,30,55
       defb 55,142,20,55,55,142,40,67,67,142,38,63,63,142,19,56,56,56,56,142,38,63,63,142
       defb 19,56,56,142,30,67,67,142,19,56,56,142,38,63,63,255,161,100,112,41,163,104,5,164
       defb 17,5,17,161,100,41,163,104,5,161,100,41,163,104,17,164,5,17,161,100,41,163,104,10
       defb 164,22,10,22,10,22,10,22,255,162,107,114,41,106,41,103,41,106,41,101,112,31,106,114
       defb 41,107,41,106,41,107,41,106,41,103,41,106,41,101,112,31,106,114,41,103,41,106,41,107
       defb 116,46,106,46,101,112,31,106,116,46,107,46,106,46,101,112,31,106,116,46,107,46,106,46
       defb 101,112,31,107,116,46,101,112,31,107,116,46,101,112,31,107,116,46,255,164,106,136,45,41
       defb 140,22,53,136,38,44,136,19,56,136,30,48,136,15,60,136,38,44,136,19,56,140,33,46
       defb 140,11,58,136,27,50,62,140,22,53,140,11,65,136,27,50,62,255,187,99,112,12,189,128
       defb 162,101,31,31,31,31,255,162,106,136,40,55,55,136,24,43,43,43,43,136,40,55,55,136
       defb 24,43,43,136,30,55,55,136,24,43,43,43,43,136,38,51,51,136,25,44,44,44,44,136
       defb 38,51,51,136,25,44,44,136,30,55,55,136,25,44,44,136,38,51,51,255,161,100,112,41
       defb 167,108,12,161,100,41,167,108,12,161,100,41,167,108,12,161,100,41,167,108,12,161,100,41
       defb 167,108,8,161,100,41,167,108,8,161,100,41,167,108,8,161,100,41,163,108,8,161,100,41
       defb 163,108,8,255,162,107,114,36,106,36,103,36,106,36,107,36,106,36,103,36,106,36,107,36
       defb 106,36,103,36,106,36,107,36,36,103,36,106,36,107,116,44,106,44,103,44,106,44,107,44
       defb 106,44,103,44,106,44,101,112,31,106,116,44,101,112,31,106,116,44,101,112,31,107,116,44
       defb 101,112,31,31,255,161,100,112,41,163,104,10,164,22,10,22,161,100,41,163,104,10,161,100
       defb 41,163,104,22,164,10,22,161,100,41,163,104,10,164,22,10,22,10,22,10,22,255,162,107
       defb 116,46,106,46,103,46,106,46,101,112,31,106,116,46,107,46,106,46,107,46,106,46,103,46
       defb 106,46,101,112,31,106,116,46,103,46,106,46,107,46,106,46,101,112,31,106,116,46,107,46
       defb 106,46,101,112,31,106,116,46,107,46,106,46,101,112,31,107,116,46,101,112,31,107,116,46
       defb 101,112,31,107,116,46,255,164,106,136,20,55,140,22,53,136,27,50,136,30,48,136,27,50
       defb 136,30,48,140,33,46,136,40,43,140,33,46,136,40,43,136,45,41,136,54,38,136,45,41
       defb 136,54,38,136,60,36,136,67,46,255,161,100,112,41,163,104,6,164,18,6,161,100,41,163
       defb 104,18,164,6,161,100,41,163,104,18,164,6,18,161,100,41,163,104,8,161,100,41,163,104
       defb 20,164,8,161,100,41,163,104,20,164,8,20,8,20,255,162,107,116,42,106,42,107,42,106
       defb 42,101,112,31,106,116,42,107,42,42,42,106,42,107,42,106,42,101,112,31,107,116,42,101
       defb 112,31,106,116,42,107,44,106,44,107,44,106,44,101,112,31,106,116,44,107,44,44,44,106
       defb 44,101,112,33,107,116,44,101,112,33,107,116,44,101,112,33,107,116,44,255,162,106,140,7
       defb 49,110,112,48,106,140,7,42,110,112,49,106,140,5,48,110,112,42,106,140,7,42,110,112
       defb 48,106,140,17,70,110,112,42,106,140,7,42,110,112,46,106,140,5,48,110,112,42,106,140
       defb 7,42,110,112,48,164,106,140,14,49,136,19,44,136,15,48,136,19,44,136,17,46,136,19
       defb 44,136,15,48,136,19,44,255,161,100,112,41,163,104,5,161,100,41,163,104,17,164,5,161
       defb 100,41,163,104,17,164,5,161,100,41,163,104,17,164,5,17,161,100,41,163,104,10,161,100
       defb 41,163,104,22,164,10,161,100,41,163,104,22,164,10,22,10,22,255,162,107,114,41,106,41
       defb 107,41,106,41,101,112,31,106,114,41,107,41,41,41,106,41,107,41,106,41,101,112,31,106
       defb 114,41,107,41,106,41,107,46,106,46,107,46,106,46,101,112,33,106,114,46,107,46,46,46
       defb 106,46,101,112,31,107,114,46,101,112,31,107,114,46,101,112,31,107,114,46,255,162,106,140
       defb 5,48,110,112,46,106,140,22,53,110,112,48,106,140,19,56,110,112,41,104,140,17,70,110
       defb 112,44,106,140,5,48,110,112,46,106,140,17,70,110,112,48,106,136,19,68,110,112,46,106
       defb 140,11,41,110,112,44,164,106,136,22,53,140,33,46,140,25,51,140,33,46,140,28,49,140
       defb 33,46,140,25,51,140,33,46,255,162,100,112,41,161,108,128,193,36,188,38,255,161,100,112
       defb 41,167,108,12,161,100,41,167,108,12,161,100,41,167,108,12,161,100,41,167,108,12,161,100
       defb 41,167,108,10,161,100,41,167,108,10,161,100,41,167,108,10,161,100,41,163,108,10,161,100
       defb 41,163,108,10,255,163,129,192,108,112,39,189,41,255,192,108,112,20,17,255,192,108,112,0
       defb 10,255,163,129,180,108,112,39,164,41,168,43,189,41,255,161,100,112,41,167,108,8,161,100
       defb 41,167,108,8,161,100,41,167,108,8,161,100,41,163,108,8,161,100,41,163,108,8,161,100
       defb 41,167,108,5,161,100,41,167,108,5,161,100,41,167,108,5,161,100,41,108,5,100,41,108
       defb 5,100,41,108,5,100,41,108,5,255,0,0,0,0,0,0,0,0,0,0,0,0,0

       org 59367           ; both bytes are 231.

; Interrupt routine.

int    push af
       push bc
       push de
       push hl
       push ix
       ld hl,23672         ; frame counter.
       inc (hl)            ; nudge it along.
       call music          ; play music.
       pop ix
       pop hl
       pop de
       pop bc
       pop af
       ei
       reti
       ret



; Here's the interrupt vector table.

       org 59392

       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int,int,int,int
       defw int


; ix +  0 = type.
; ix +  1 = sprite image number.
; ix +  2 = x.
; ix +  3 = y.
; ix +  4 = frame.
; ix +  5 = info.

; ix +  6 = new type.
; ix +  7 = new image.
; ix +  8 = new x.
; ix +  9 = new y.
; ix + 10 = new frame.

sprtab equ $

       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0
       defb 255,0,0,0,0,0,255,0,0,0,0

endspt equ $
SPRIT1 equ sprtab + SPRSIZ ; address of first non-player sprite.
SPRIT2 equ SPRIT1 + SPRSIZ ; address of next sprite.

